@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -20,7 +20,8 @@
 #if defined (__i386__) || defined (__x86_64__)
 
 #include "rawstudio.h"
-#include "rs-render.h"
+#include "rs-color-transform.h"
+#include "rs-image.h"
 
 #include "x86_cpu.h"
 
@@ -60,6 +61,12 @@ rs_bind_optimized_functions()
 
 	/* Bind functions according to available features */
 
+	/* Image size doubler */
+	if (cpuflags & _MMX)
+	{
+		rs_image16_copy_double = rs_image16_copy_double_mmx;
+	}
+
 	/* Black and shift applier */
 	if (cpuflags & _MMX)
 	{
@@ -70,14 +77,14 @@ rs_bind_optimized_functions()
 	if (cpuflags & _SSE)
 	{
 		/* SSE is favored over 3dnow in case both are available */
-		rs_render_cms   = rs_render_cms_sse;
-		rs_render_nocms = rs_render_nocms_sse;
+		transform_nocms8 = transform_nocms8_sse;
+		transform_cms8 = transform_cms8_sse;
 	}
 	else if (cpuflags & _3DNOW)
 	{
 		/* Only 3dnow */
-		rs_render_cms = rs_render_cms_3dnow;
-		rs_render_nocms = rs_render_nocms_3dnow;
+		transform_nocms8 = transform_nocms8_3dnow;
+		transform_cms8 = transform_cms8_3dnow;
 	}
 }
 
