@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -27,6 +27,7 @@
 #include "filename.h"
 #include "gtk-helper.h"
 #include "rs-cms.h"
+#include "rs-preview-widget.h"
 #include <gettext.h>
 #include <lcms.h>
 
@@ -48,8 +49,6 @@ static void gui_cms_intent_combobox_changed(GtkComboBox *combobox, gpointer user
 static void gui_cms_in_profile_button_clicked(GtkButton *button, gpointer user_data);
 static void gui_cms_di_profile_button_clicked(GtkButton *button, gpointer user_data);
 static void gui_cms_ex_profile_button_clicked(GtkButton *button, gpointer user_data);
-static void spinner_set_conf(GtkSpinButton *spinner, gpointer user_data);
-static void slider_with_spinner_set_conf(GtkSpinButton *spinner, gpointer user_data);
 static gboolean rs_gtk_tree_model_count_helper(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
 static gint rs_gtk_tree_model_count(GtkTreeModel *model);
 
@@ -314,7 +313,7 @@ gui_export_changed_helper(GtkLabel *label)
 	filename = rs_conf_get_string(CONF_EXPORT_FILENAME);
 	rs_conf_get_filetype(CONF_EXPORT_FILETYPE, &filetype);
 
-	parsed = filename_parse(filename, NULL);
+	parsed = filename_parse(filename, "filename", 0);
 
 	final = g_string_new("<small>");
 	if (directory)
@@ -324,7 +323,6 @@ gui_export_changed_helper(GtkLabel *label)
 	}
 	g_string_append(final, parsed);
 	g_free(parsed);
-	g_string_append(final, ".");
 	g_string_append(final, filetype->ext);
 	g_string_append(final, "</small>");
 
@@ -363,7 +361,8 @@ cms_enable_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 	RS_BLOB *rs = (RS_BLOB *) user_data;
 	rs_conf_set_boolean(CONF_CMS_ENABLED, togglebutton->active);
 	rs_cms_enable(rs->cms, togglebutton->active);
-	update_previewtable_callback(NULL, rs);
+	rs_preview_widget_set_cms(RS_PREVIEW_WIDGET(rs->preview), rs_cms_get_transform(rs->cms, PROFILE_DISPLAY));
+	rs_update_preview(rs);
 	return;
 }
 
@@ -376,7 +375,7 @@ gui_cms_in_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_IN);
 	rs_cms_set_profile(rs->cms, PROFILE_INPUT, filename);
 	g_free(filename);
-	update_preview_callback(NULL, rs);
+	rs_update_preview(rs);
 	return;
 }
 
@@ -390,7 +389,7 @@ gui_cms_di_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_DISPLAY);
 	rs_cms_set_profile(rs->cms, PROFILE_DISPLAY, filename);
 	g_free(filename);
-	update_preview_callback(NULL, rs);
+	rs_update_preview(rs);
 	return;
 }
 
@@ -404,7 +403,7 @@ gui_cms_ex_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_EXPORT);
 	rs_cms_set_profile(rs->cms, PROFILE_EXPORT, filename);
 	g_free(filename);
-	update_preview_callback(NULL, rs);
+	rs_update_preview(rs);
 	return;
 }
 
@@ -415,7 +414,7 @@ gui_cms_intent_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 	gint active = gtk_combo_box_get_active(combobox);
 	rs_conf_set_cms_intent(CONF_CMS_INTENT, &active);
 	rs_cms_set_intent(rs->cms, active);
-	update_preview_callback(NULL, rs);
+	rs_update_preview(rs);
 	return;
 }
 
@@ -731,77 +730,6 @@ gui_preferences_make_cms_page(RS_BLOB *rs)
 	return cms_page;
 }
 
-
-void
-spinner_set_conf(GtkSpinButton *spinner, gpointer user_data)
-{
-	const gchar *path = user_data;
-	rs_conf_set_integer(path, gtk_spin_button_get_value_as_int(spinner));
-	return;
-}
-
-GtkWidget *
-spinner_from_conf(const gchar *conf, gchar *labeltext, gint default_value)
-{
-	gint value = default_value;
-	GtkWidget *spinner;
-	GtkWidget *label;
-	GtkWidget *box;
-
-	box = gtk_hbox_new(FALSE, 4);
-
-	label = gtk_label_new(labeltext);
-
-	if (!rs_conf_get_integer(conf, &value))
-		value = default_value;
-	spinner = gtk_spin_button_new_with_range(1.0, 65535.0, 1.0);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner), value);
-	g_signal_connect ((gpointer) spinner, "value-changed",
-		G_CALLBACK (spinner_set_conf), (gpointer) conf);
-
-	gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (box), spinner, FALSE, TRUE, 0);
-
-	return(box);
-}
-
-void
-slider_with_spinner_set_conf(GtkSpinButton *spin, gpointer user_data)
-{
-	const gchar *path = user_data;
-	rs_conf_set_integer(path, gtk_spin_button_get_value_as_int(spin));
-	return;
-}
-
-GtkWidget *
-slider_with_spinner_from_conf(const gchar *conf, gchar *labeltext, gint default_value, 
-			gdouble min, gdouble max, gdouble step_increment, gdouble page_increment, gdouble page_size)
-{
-	GtkObject *adj;
-	GtkWidget *label;
-	GtkWidget *scale;
-	GtkWidget *spin;
-	GtkWidget *box;
-	gint conf_value;
-
-	if (!rs_conf_get_integer(conf, &conf_value))
-		conf_value = default_value;
-
-	adj = gtk_adjustment_new((gdouble) conf_value, min, max, step_increment, page_increment, page_size);
-	label = gtk_label_new(labeltext);
-	scale = gtk_hscale_new(GTK_ADJUSTMENT(adj));
-	gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
-	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
-	g_signal_connect((gpointer) spin, "value-changed", G_CALLBACK(slider_with_spinner_set_conf), (gpointer) conf);
-
-	box = gtk_hbox_new(FALSE, 2);
-	gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (box), scale, TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (box), spin, FALSE, TRUE, 0);
-
-	return(box);
-}
-
 /* copied verbatim from Gimp: app/widgets/gimpdock.c */
 gboolean
 window_key_press_event (GtkWidget   *widget,
@@ -890,3 +818,58 @@ rs_gtk_tree_model_count(GtkTreeModel *model)
 	gtk_tree_model_foreach (model, rs_gtk_tree_model_count_helper, &n);
 	return n;
 }
+
+GtkWidget *
+gui_aligned(GtkWidget *widget, const gfloat xalign, const gfloat yalign, const gfloat xscale, const gfloat yscale)
+{
+	GtkWidget *alignment;
+
+	g_assert(GTK_IS_WIDGET(widget));
+
+	alignment = gtk_alignment_new(xalign, yalign, xscale, yscale);
+	gtk_container_add (GTK_CONTAINER (alignment), widget);
+
+	return alignment;
+}
+
+/**
+ * Build and show a popup-menu
+ * @param widget A widget to pop up below or NULL to pop upat mouse pointer
+ * @param user_data Pointer to pass to callback
+ * @param ... Pairs of gchar labels and callbaks, terminated by -1
+ * @return The newly created menu
+ */
+GtkWidget *
+gui_menu_popup(GtkWidget *widget, gpointer user_data, ...)
+{
+	va_list ap;
+	GCallback cb;
+	gchar *label;
+	GtkWidget *item, *menu = gtk_menu_new();
+	gint n = 0;
+
+	va_start(ap, user_data);
+
+	/* Loop through arguments, abort on -1 */
+	while (1)
+	{
+		label = va_arg(ap, gchar *);
+		if (GPOINTER_TO_INT(label) == -1) break;
+		cb = va_arg(ap, GCallback);
+		if (GPOINTER_TO_INT(cb) == -1) break;
+
+		item = gtk_menu_item_new_with_label (label);
+		gtk_widget_show (item);
+		gtk_menu_attach (GTK_MENU (menu), item, 0, 1, n, n+1); n++;
+		g_signal_connect (item, "activate", cb, user_data);
+	}
+
+	va_end(ap);
+
+	if (widget)
+		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, pos_menu_below_widget, widget, 0, GDK_CURRENT_TIME);
+	else
+		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, GDK_CURRENT_TIME);
+
+	return (menu);
+}
