@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -38,8 +38,16 @@ extern void gui_export_changed_helper(GtkLabel *label);
 extern void gui_export_directory_entry_changed(GtkEntry *entry, gpointer user_data);
 extern void gui_export_filename_entry_changed(GtkComboBox *combobox, gpointer user_data);
 extern GtkWidget *gui_preferences_make_cms_page();
-extern GtkWidget *spinner_from_conf(const gchar *conf, gchar *labeltext, gint default_value);
-extern GtkWidget *slider_with_spinner_from_conf(const gchar *conf, gchar *labeltext, gint default_value, gdouble min, gdouble max, gdouble step_increment, gdouble page_increment, gdouble page_size);
 extern gboolean window_key_press_event(GtkWidget *widget, GdkEventKey *event);
 extern void pos_menu_below_widget(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data);
 extern GtkWidget *gui_framed(GtkWidget *widget, const gchar *title, GtkShadowType shadowtype);
+extern GtkWidget *gui_aligned(GtkWidget *widget, const gfloat xalign, const gfloat yalign, const gfloat xscale, const gfloat yscale);
+
+/**
+ * Build and show a popup-menu
+ * @param widget A widget to pop up below or NULL to pop upat mouse pointer
+ * @param user_data Pointer to pass to callback
+ * @param ... Pairs of gchar labels and callbaks, terminated by -1
+ * @return The newly created menu
+ */
+GtkWidget *gui_menu_popup(GtkWidget *widget, gpointer user_data, ...);
