@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -24,12 +24,10 @@
 #include <config.h>
 #include "color.h"
 #include "rawstudio.h"
-#include "rs-crop.h"
 #include "gtk-helper.h"
 #include "gtk-interface.h"
 #include "gtk-save-dialog.h"
 #include "gtk-progress.h"
-#include "drawingarea.h"
 #include "toolbox.h"
 #include "conf_interface.h"
 #include "rs-cache.h"
@@ -37,12 +35,14 @@
 #include "rs-batch.h"
 #include "gettext.h"
 #include "rs-batch.h"
-#include "rs-straighten.h"
 #include "rs-cms.h"
 #include <string.h>
 #include <unistd.h>
 #include "filename.h"
 #include "rs-store.h"
+#include "rs-preview-widget.h"
+#include "rs-histogram.h"
+#include "rs-preload.h"
 
 struct rs_callback_data_t {
 	RS_BLOB *rs;
@@ -75,6 +75,10 @@ static void gui_setprio(RS_BLOB *rs, guint prio);
 static gboolean gui_accel_setprio_callback(GtkAccelGroup *group, GObject *obj, guint keyval,
 	GdkModifierType mod, gpointer user_data);
 static void gui_menu_setprio_callback(gpointer callback_data, guint callback_action, GtkWidget *widget);
+#ifdef EXPERIMENTAL
+static void gui_menu_group_photos_callback(gpointer callback_data, guint callback_action, GtkWidget *widget);
+static void gui_menu_ungroup_photos_callback(gpointer callback_data, guint callback_action, GtkWidget *widget);
+#endif
 static void gui_widget_show(GtkWidget *widget, gboolean show, const gchar *conf_fullscreen_key, const gchar *conf_windowed_key);
 static gboolean gui_fullscreen_iconbox_callback(GtkWidget *widget, GdkEventWindowState *event, GtkWidget *iconbox);
 static gboolean gui_fullscreen_toolbox_callback(GtkWidget *widget, GdkEventWindowState *event, GtkWidget *toolbox);
@@ -97,8 +101,11 @@ static void gui_menu_show_exposure_mask_callback(gpointer callback_data, guint c
 static void gui_menu_paste_callback(gpointer callback_data, guint callback_action, GtkWidget *widget);
 static void gui_menu_copy_callback(gpointer callback_data, guint callback_action, GtkWidget *widget);
 static GtkWidget *gui_make_menubar(RS_BLOB *rs, GtkWidget *window, GtkWidget *iconbox, GtkWidget *toolbox);
+static void drag_data_received(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *selection_data, guint info, guint t,	RS_BLOB *rs);
 static GtkWidget *gui_window_make(RS_BLOB *rs);
 static GtkWidget *gui_dialog_make_from_widget(const gchar *stock_id, gchar *primary_text, GtkWidget *widget);
+static void rs_open_file(RS_BLOB *rs, const gchar *filename);
+static gboolean pane_position(GtkWidget* widget, gpointer dummy, gpointer user_data);
 
 void
 gui_set_busy(gboolean rawstudio_is_busy)
@@ -139,8 +146,10 @@ gui_is_busy(void)
 static gboolean
 gui_statusbar_remove_helper(guint *msgid)
 {
+	gdk_threads_enter();
 	gtk_statusbar_remove(statusbar, gtk_statusbar_get_context_id(statusbar, "generic"), *msgid);
 	g_free(msgid);
+	gdk_threads_leave();
 	return(FALSE);
 }
 
@@ -174,119 +183,12 @@ update_preview_callback(GtkAdjustment *do_not_use_this, RS_BLOB *rs)
 {
 	if (rs->photo)
 	{
-		rs_settings_to_rs_settings_double(rs->settings[rs->current_setting], rs->photo->settings[rs->photo->current_setting]);
-		update_preview(rs, FALSE, FALSE);
-		gui_set_values(rs, -1, -1);
+		rs_settings_to_rs_settings_double(rs->settings[rs->current_setting], rs->photo->settings[rs->current_setting]);
+		rs_update_preview(rs);
 	}
 	return(FALSE);
 }
 
-gboolean
-update_previewtable_callback(GtkAdjustment *do_not_use_this, RS_BLOB *rs)
-{
-	if (rs->photo)
-	{
-		rs_settings_to_rs_settings_double(rs->settings[rs->current_setting], rs->photo->settings[rs->photo->current_setting]);
-		update_preview(rs, TRUE, FALSE);
-		gui_set_values(rs, -1, -1);
-	}
-	return(FALSE);
-}
-
-gboolean
-update_scale_callback(GtkAdjustment *do_not_use_this, RS_BLOB *rs)
-{
-	rs->zoom_to_fit = FALSE;
-	update_preview_callback(NULL, rs);
-	return(FALSE);
-}
-
-void update_histogram(RS_BLOB *rs)
-{
-	guint c,i,x,y,rowstride;
-	guint max = 0;
-	guint factor = 0;
-	guint hist[3][256];
-	gint width;
-	gint height;
-	gint channels;
-	GdkPixbuf *pixbuf;
-	guchar *pixels, *p;
-	gint row,col;
-
-	pixbuf = gtk_image_get_pixbuf(rs->histogram_image);
-	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-	pixels = gdk_pixbuf_get_pixels (pixbuf);
-	width = gdk_pixbuf_get_width (pixbuf);
-	height = gdk_pixbuf_get_height (pixbuf);
-	channels = gdk_pixbuf_get_n_channels (pixbuf);
-
-	/* sets all the pixels black */
-	memset(pixels, 0x00, rowstride*height);
-
-	/* draw a grid with 7 bars with 32 pixels space */
-	p = pixels;
-	for(y = 0; y < height; y++)
-	{
-		for(x = 0; x < 256 * 3; x +=93)
-		{
-			p[x++] = 100;
-			p[x++] = 100;
-			p[x++] = 100;
-		}
-		p+=rowstride;
-	}
-
-	/* find the max value */
-	for (c = 0; c < 3; c++)
-	{
-		for (i = 1; i < 255; i++)
-		{
-			_MAX(rs->histogram_table[c][i], max);
-		}
-	}
-
-	/* find the factor to scale the histogram */
-	factor = (max+height)/height;
-
-	/* calculate the histogram values */
-	for (c = 0; c < 3; c++)
-	{
-		for (i = 1; i < 255; i++)
-		{
-			hist[c][i] = rs->histogram_table[c][i]/factor;
-		}
-	}
-
-	/* draw the histogram */
-	for (x = 1; x < 255; x++)
-	{
-		for (c = 0; c < 3; c++)
-		{
-			for (y = 0; y < hist[c][x]; y++)
-			{				
-				/* address the pixel - the (rs->hist_h-1)-y is to draw it from the bottom */
-				p = pixels + ((height-1)-y) * rowstride + x * 3;
-				p[c] = 0xFF;
-			}
-		}
-	}
-	/* draw under/over-exposed "triangles" */
-	for (c = 0; c < 3; c++)
-	{
-		if (rs->histogram_table[c][0] > 100)
-			for(row = 0; row < 10; row++)
-				for(col = 0; col < (10-row); col++)
-						pixels[row*rowstride + col*channels+c] = 0xFF;
-		if (rs->histogram_table[c][255] > 100)
-			for(row = 0; row < 10; row++)
-				for(col = (width-10+row); col < width; col++)
-					pixels[row*rowstride + col*channels+c] = 0xFF;
-	}
-	gtk_image_set_from_pixbuf((GtkImage *) rs->histogram_image, pixbuf);
-
-}
-
 static void
 icon_activated(gpointer instance, const gchar *name, RS_BLOB *rs)
 {
@@ -300,13 +202,27 @@ icon_activated(gpointer instance, const gchar *name, RS_BLOB *rs)
 	if (name!=NULL)
 	{
 		GString *window_title;
-		rs_render_idle_stop(rs);
+		GList *selected = NULL;
+		g_signal_handlers_block_by_func(instance, icon_activated, rs);
 		gui_set_busy(TRUE);
 		msgid = gui_status_push(_("Opening photo ..."));
-		GUI_CATCHUP();
+		GTK_CATCHUP();
+		g_signal_handlers_unblock_by_func(instance, icon_activated, rs);
+
+		/* Read currently selected filename, it may or may not (!) be the same as served in name */
+		selected = rs_store_get_selected_names(rs->store);
+		if (g_list_length(selected)==1)
+		{
+			name = g_list_nth_data(selected, 0);
+			g_list_free(selected);
+		}
+
 		if ((filetype = rs_filetype_get(name, TRUE)))
 		{
-			photo = filetype->load(name);
+			rs_preview_widget_set_photo(RS_PREVIEW_WIDGET(rs->preview), NULL);
+			photo = rs_get_preloaded(name);
+			if (!photo)
+				photo = filetype->load(name, FALSE);
 			if (photo)
 			{
 				rs->in_use = FALSE;
@@ -314,7 +230,6 @@ icon_activated(gpointer instance, const gchar *name, RS_BLOB *rs)
 				rs_photo_free(rs->photo);
 				rs->photo = NULL;
 				rs_reset(rs);
-				rs_mark_roi(rs, FALSE);
 			}
 			else
 			{
@@ -323,8 +238,7 @@ icon_activated(gpointer instance, const gchar *name, RS_BLOB *rs)
 				gui_set_busy(FALSE);
 				return;
 			}
-			photo->current_setting = rs->current_setting;
-			rs_image16_free(rs->histogram_dataset); rs->histogram_dataset = NULL;
+
 			if (filetype->load_meta)
 			{
 				filetype->load_meta(name, photo->metadata);
@@ -359,25 +273,26 @@ icon_activated(gpointer instance, const gchar *name, RS_BLOB *rs)
 			rs_settings_double_to_rs_settings(photo->settings[1], rs->settings[1]);
 			rs_settings_double_to_rs_settings(photo->settings[2], rs->settings[2]);
 			rs->photo = photo;
+			rs_preview_widget_set_photo(RS_PREVIEW_WIDGET(rs->preview), rs->photo);
 
 			if (!cache_loaded)
 			{
 				if (rs->photo->metadata->cam_mul[R] == -1.0)
 				{
-					rs->photo->current_setting = 2;
+					rs->current_setting = 2;
 					rs_set_wb_auto(rs);
-					rs->photo->current_setting = 1;
+					rs->current_setting = 1;
 					rs_set_wb_auto(rs);
-					rs->photo->current_setting = 0;
+					rs->current_setting = 0;
 					rs_set_wb_auto(rs);
 				}
 				else
 				{
-					rs->photo->current_setting = 2;
+					rs->current_setting = 2;
 					rs_set_wb_from_mul(rs, rs->photo->metadata->cam_mul);
-					rs->photo->current_setting = 1;
+					rs->current_setting = 1;
 					rs_set_wb_from_mul(rs, rs->photo->metadata->cam_mul);
-					rs->photo->current_setting = 0;
+					rs->current_setting = 0;
 					rs_set_wb_from_mul(rs, rs->photo->metadata->cam_mul);
 				}
 				if (rs->photo->metadata->contrast != -1.0)
@@ -393,13 +308,16 @@ icon_activated(gpointer instance, const gchar *name, RS_BLOB *rs)
 					SETVAL(rs->settings[0]->saturation,rs->photo->metadata->saturation);
 				}
 			}
+
+			if (rs->histogram_dataset)
+				rs_image16_free(rs->histogram_dataset);
+
 			rs->histogram_dataset = rs_image16_scale(rs->photo->input, NULL,
 				(gdouble)HISTOGRAM_DATASET_WIDTH/(gdouble)rs->photo->input->w);
+			rs_histogram_set_image(RS_HISTOGRAM_WIDGET(rs->histogram), rs->histogram_dataset);
 		}
 		rs->in_use = TRUE;
-		if (rs->zoom_to_fit)
-			rs_zoom_to_fit(rs);
-		update_previewtable_callback(NULL, rs);
+		rs_update_preview(rs);
 		gui_status_pop(msgid);
 		gui_status_notify(_("Image opened"));
 		window_title = g_string_new(_("Rawstudio"));
@@ -533,8 +451,7 @@ gui_preview_bg_color_changed(GtkColorButton *widget, RS_BLOB *rs)
 {
 	GdkColor color;
 	gtk_color_button_get_color(GTK_COLOR_BUTTON(widget), &color);
-	gtk_widget_modify_bg(rs->preview_drawingarea->parent->parent,
-		GTK_STATE_NORMAL, &color);
+	rs_preview_widget_set_bgcolor(RS_PREVIEW_WIDGET(rs->preview), &color);
 	rs_conf_set_color(CONF_PREBGCOLOR, &color);
 	return;
 }
@@ -547,31 +464,32 @@ gui_menu_zoom_callback(gpointer callback_data, guint callback_action, GtkWidget
 	switch (callback_action)
 	{
 		case 0: /* zoom to fit */
-			rs_zoom_to_fit(rs);
-			rs->zoom_to_fit = TRUE;
+			rs_preview_widget_set_zoom_to_fit(RS_PREVIEW_WIDGET(rs->preview));
 			break;
 		case 1: /* zoom in */
-			SETVAL(rs->scale, GETVAL(rs->scale)+0.1);
-			rs->zoom_to_fit = FALSE;
+			rs_preview_widget_zoom_in(RS_PREVIEW_WIDGET(rs->preview));
 			break;
 		case 2: /* zoom out */
-			SETVAL(rs->scale, GETVAL(rs->scale)-0.1);
-			rs->zoom_to_fit = FALSE;
+			rs_preview_widget_zoom_out(RS_PREVIEW_WIDGET(rs->preview));
 			break;
 		case 100: /* zoom 100% */
-			SETVAL(rs->scale, 1.0);
-			rs->zoom_to_fit = FALSE;
+			rs_preview_widget_set_zoom(RS_PREVIEW_WIDGET(rs->preview), 1.0);
 			break;
 	}
-	scale_expand_set(!rs->zoom_to_fit);
 	return;
 }
 
 static void
 gui_menu_prevnext_callback(gpointer callback_data, guint callback_action, GtkWidget *widget)
 {
+	gchar *current_filename = NULL;
 	RS_BLOB *rs = (RS_BLOB *)((struct rs_callback_data_t*)callback_data)->rs;
-	rs_store_select_prevnext(rs->store, callback_action);
+	
+	/* Get current filename if a photo is loaded */
+	if (rs->photo)
+		current_filename = rs->photo->filename;
+
+	rs_store_select_prevnext(rs->store, current_filename, callback_action);
 	return;
 }
 
@@ -639,7 +557,7 @@ static void
 gui_menu_crop_callback(gpointer callback_data, guint callback_action, GtkWidget *widget)
 {
 	RS_BLOB *rs = (RS_BLOB *)((struct rs_callback_data_t*)callback_data)->rs;
-	rs_crop_start(rs);
+	rs_preview_widget_crop_start(RS_PREVIEW_WIDGET(rs->preview));
 	return;
 }
 
@@ -647,7 +565,7 @@ static void
 gui_menu_uncrop_callback(gpointer callback_data, guint callback_action, GtkWidget *widget)
 {
 	RS_BLOB *rs = (RS_BLOB *)((struct rs_callback_data_t*)callback_data)->rs;
-	rs_crop_uncrop(rs);
+	rs_preview_widget_uncrop(RS_PREVIEW_WIDGET(rs->preview));
 	return;
 }
 
@@ -655,13 +573,32 @@ static void
 gui_menu_straighten_callback(gpointer callback_data, guint callback_action, GtkWidget *widget)
 {
 	RS_BLOB *rs = (RS_BLOB *)((struct rs_callback_data_t*)callback_data)->rs;
-	if (callback_action)
-		rs_straighten_start(rs);
+
+	if (callback_action == 1)
+		rs_preview_widget_straighten(RS_PREVIEW_WIDGET(rs->preview));
 	else
-		rs_straighten_unstraighten(rs);
+		rs_preview_widget_unstraighten(RS_PREVIEW_WIDGET(rs->preview));
 	return;
 }
 
+#ifdef EXPERIMENTAL
+static void
+gui_menu_group_photos_callback(gpointer callback_data, guint callback_action, GtkWidget *widget)
+{
+	RS_BLOB *rs = (RS_BLOB *)((struct rs_callback_data_t*)callback_data)->rs;
+	rs_store_group_photos(rs->store);
+	return;
+}
+
+static void
+gui_menu_ungroup_photos_callback(gpointer callback_data, guint callback_action, GtkWidget *widget)
+{
+	RS_BLOB *rs = (RS_BLOB *)((struct rs_callback_data_t*)callback_data)->rs;
+	rs_store_ungroup_photos(rs->store);
+	return;
+}
+#endif
+
 static void
 gui_widget_show(GtkWidget *widget, gboolean show, const gchar *conf_fullscreen_key, const gchar *conf_windowed_key)
 {
@@ -761,11 +698,8 @@ gui_menu_fullscreen_callback(gpointer callback_data, guint callback_action, GtkW
 static gboolean
 gui_histogram_height_changed(GtkAdjustment *caller, RS_BLOB *rs)
 {
-	GdkPixbuf *pixbuf;
 	const gint newheight = (gint) caller->value;
-	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, 256, newheight);
-	gtk_image_set_from_pixbuf((GtkImage *) rs->histogram_image, pixbuf);
-	update_histogram(rs);
+	gtk_widget_set_size_request(rs->histogram, 64, newheight);
 	rs_conf_set_integer(CONF_HISTHEIGHT, newheight);
 	return(FALSE);
 }
@@ -789,6 +723,16 @@ gui_preference_iconview_show_filenames_changed(GtkToggleButton *togglebutton, gp
 	return;
 }
 
+static void
+gui_preference_preload_changed(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	if (togglebutton->active)
+		rs_preload_set_maximum_memory(200*1024*1024);
+	else
+		rs_preload_set_maximum_memory(0);
+
+	return;
+}
 
 static void
 gui_menu_preference_callback(gpointer callback_data, guint callback_action, GtkWidget *widget)
@@ -809,6 +753,7 @@ gui_menu_preference_callback(gpointer callback_data, guint callback_action, GtkW
 	gint histogram_height;
 	GtkWidget *local_cache_check;
 	GtkWidget *load_gdk_check;
+	GtkWidget *preload_check;
 	GtkWidget *show_filenames;
 
 /*
@@ -903,6 +848,13 @@ gui_menu_preference_callback(gpointer callback_data, guint callback_action, GtkW
 	load_gdk_check = checkbox_from_conf(CONF_LOAD_GDK, _("Load 8 bit photos (jpeg, png, etc)"), FALSE);
 	gtk_box_pack_start (GTK_BOX (preview_page), load_gdk_check, FALSE, TRUE, 0);
 
+	preload_check = checkbox_from_conf(CONF_PRELOAD, _("Preload photos"), FALSE);
+#ifdef EXPERIMENTAL
+	gtk_box_pack_start (GTK_BOX (preview_page), preload_check, FALSE, TRUE, 0);
+#endif
+	g_signal_connect ((gpointer) preload_check, "toggled",
+		G_CALLBACK (gui_preference_preload_changed), rs);
+
 /*
 	batch_page = gtk_vbox_new(FALSE, 4);
 	gtk_container_set_border_width (GTK_CONTAINER (batch_page), 6);
@@ -966,7 +918,7 @@ gui_menu_preference_callback(gpointer callback_data, guint callback_action, GtkW
 	gtk_container_set_border_width (GTK_CONTAINER (export_page), 6);
 
 	export_directory_hbox = gtk_hbox_new(FALSE, 0);
-	export_directory_label = gtk_label_new(_("Export directory:"));
+	export_directory_label = gtk_label_new(_("Directory:"));
 	gtk_misc_set_alignment(GTK_MISC(export_directory_label), 0.0, 0.5);
 	export_directory_entry = gtk_entry_new();
 	conf_temp = rs_conf_get_string(CONF_EXPORT_DIRECTORY);
@@ -982,7 +934,7 @@ gui_menu_preference_callback(gpointer callback_data, guint callback_action, GtkW
 
 
 	export_filename_hbox = gtk_hbox_new(FALSE, 0);
-	export_filename_label = gtk_label_new(_("Export filename:"));
+	export_filename_label = gtk_label_new(_("Filename:"));
 	gtk_misc_set_alignment(GTK_MISC(export_filename_label), 0.0, 0.5);
 	export_filename_entry = gtk_combo_box_entry_new_text();
 	conf_temp = rs_conf_get_string(CONF_EXPORT_FILENAME);
@@ -1008,7 +960,7 @@ gui_menu_preference_callback(gpointer callback_data, guint callback_action, GtkW
 	gtk_box_pack_start (GTK_BOX (export_page), export_filename_hbox, FALSE, TRUE, 0);
 
 	export_filetype_hbox = gtk_hbox_new(FALSE, 0);
-	export_filetype_label = gtk_label_new(_("Export filetype:"));
+	export_filetype_label = gtk_label_new(_("File type:"));
 	gtk_misc_set_alignment(GTK_MISC(export_filetype_label), 0.0, 0.5);
 
 	if (!rs_conf_get_filetype(CONF_EXPORT_FILETYPE, &filetype))
@@ -1046,7 +998,7 @@ gui_menu_preference_callback(gpointer callback_data, guint callback_action, GtkW
 	gtk_container_set_border_width (GTK_CONTAINER (notebook), 6);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), preview_page, gtk_label_new(_("General")));
 	//gtk_notebook_append_page(GTK_NOTEBOOK(notebook), batch_page, gtk_label_new(_("Batch")));
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), export_page, gtk_label_new(_("Export")));
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), export_page, gtk_label_new(_("Quick export")));
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), cms_page, gtk_label_new(_("Colors")));
 	gtk_box_pack_start (GTK_BOX (vbox), notebook, FALSE, FALSE, 0);
 
@@ -1073,31 +1025,22 @@ gui_menu_add_to_batch_queue_callback(gpointer callback_data, guint callback_acti
 	GList *selected = NULL;
 	gint num_selected, cur;
 
-	if (rs->in_use)
+	selected = rs_store_get_selected_names(rs->store);
+	num_selected = g_list_length(selected);
+
+	if (rs->in_use && num_selected == 1)
 	{
-		RS_QUEUE_ELEMENT *element = g_new(RS_QUEUE_ELEMENT, 1);
-	
 		rs_cache_save(rs->photo);
 
-		element->filename = rs->photo->filename;
-		element->setting_id = rs->photo->current_setting;
-
-		if (rs_batch_add_element_to_queue(rs->queue, element))
+		if (rs_batch_add_to_queue(rs->queue, rs->photo->filename, rs->current_setting))
 			gui_status_notify(_("Added to batch queue"));
 		else
 			gui_status_notify(_("Already added to batch queue"));
 	}
 
 	/* Deal with selected icons */
-	selected = rs_store_get_selected_names(rs->store);
-	num_selected = g_list_length(selected);
 	for(cur=0;cur<num_selected;cur++)
-	{
-		RS_QUEUE_ELEMENT *element = g_new(RS_QUEUE_ELEMENT, 1);
-		element->filename = g_list_nth_data(selected, cur);
-		element->setting_id = rs->current_setting;
-		rs_batch_add_element_to_queue(rs->queue, element);
-	}
+		rs_batch_add_to_queue(rs->queue, g_list_nth_data(selected, cur), rs->current_setting);
 	g_list_free(selected);
 }
 
@@ -1107,7 +1050,7 @@ gui_menu_remove_from_batch_queue_callback(gpointer callback_data, guint callback
 	RS_BLOB *rs = (RS_BLOB *)((struct rs_callback_data_t*)callback_data)->rs;
 	if (rs->in_use)
 	{
-		if (rs_batch_remove_from_queue(rs->queue, rs->photo->filename, rs->photo->current_setting))
+		if (rs_batch_remove_from_queue(rs->queue, rs->photo->filename, rs->current_setting))
 			gui_status_notify(_("Removed from batch queue"));
 		else
 			gui_status_notify(_("Not in batch queue"));
@@ -1157,6 +1100,7 @@ gui_menu_add_view_to_batch_queue_callback(gpointer callback_data, guint callback
 		gint num_selected, i;
 
 		rs_store_get_names(rs->store, NULL, &selected, NULL);
+		selected = rs_store_sort_selected(selected);
 		num_selected = g_list_length(selected);
 
 		for (i=0;i<num_selected;i++)
@@ -1164,28 +1108,13 @@ gui_menu_add_view_to_batch_queue_callback(gpointer callback_data, guint callback
 			gchar *fullname = g_list_nth_data(selected, i);
 
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cb_a)))
-			{
-				RS_QUEUE_ELEMENT *element_a = g_new(RS_QUEUE_ELEMENT, 1);
-				element_a->filename = fullname;
-				element_a->setting_id = 0;
-				rs_batch_add_element_to_queue(rs->queue, element_a);
-			}
+				rs_batch_add_to_queue(rs->queue, fullname, 0);
 
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cb_b)))
-			{
-				RS_QUEUE_ELEMENT *element_b = g_new(RS_QUEUE_ELEMENT, 1);
-				element_b->filename = fullname;
-				element_b->setting_id = 1;
-				rs_batch_add_element_to_queue(rs->queue, element_b);
-			}
+				rs_batch_add_to_queue(rs->queue, fullname, 1);
 
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cb_c)))
-			{
-				RS_QUEUE_ELEMENT *element_c = g_new(RS_QUEUE_ELEMENT, 1);
-				element_c->filename = fullname;
-				element_c->setting_id = 2;
-				rs_batch_add_element_to_queue(rs->queue, element_c);
-			}
+				rs_batch_add_to_queue(rs->queue, fullname, 2);
 		}
 		g_list_free(selected);
 
@@ -1327,10 +1256,10 @@ gui_quick_save_file_callback(gpointer callback_data, guint callback_action, GtkW
 
 	g_string_append(save, filetype->ext);
 
-	parsed_filename = filename_parse(save->str, rs->photo);
+	parsed_filename = filename_parse(save->str, rs->photo->filename, rs->current_setting);
 	g_string_free(save, TRUE);
 
-	rs_photo_save(rs->photo, parsed_filename, filetype->filetype, -1, -1, 1.0, rs->cms);
+	rs_photo_save(rs->photo, parsed_filename, filetype->filetype, -1, -1, FALSE, 1.0, rs->current_setting, rs->cms);
 	gui_status_notify(_("File exported"));
 	g_free(parsed_filename);
 
@@ -1347,7 +1276,7 @@ gui_reset_current_settings_callback(gpointer callback_data, guint callback_actio
 	rs->in_use = FALSE;
 	rs_settings_reset(rs->settings[rs->current_setting], MASK_ALL);
 	rs->in_use = in_use;
-	update_preview(rs, TRUE, FALSE);
+	rs_update_preview(rs);
 	return;
 }
 
@@ -1372,8 +1301,7 @@ gui_menu_show_exposure_mask_callback(gpointer callback_data, guint callback_acti
 	  gui_status_notify(_("Showing exposure mask"));
 	else
 	  gui_status_notify(_("Hiding exposure mask"));
-	rs->show_exposure_overlay = GTK_CHECK_MENU_ITEM(widget)->active;
-	update_preview(rs, FALSE, FALSE);
+	rs_preview_widget_set_show_exposure_mask(RS_PREVIEW_WIDGET(rs->preview), GTK_CHECK_MENU_ITEM(widget)->active);
 	return;
 }
 
@@ -1389,11 +1317,11 @@ gui_menu_revert_callback(gpointer callback_data, guint callback_action, GtkWidge
 	photo->filename = rs->photo->filename;
 
 	rs_cache_load(photo);
-	rs_settings_double_to_rs_settings(photo->settings[rs->photo->current_setting],
-		rs->settings[rs->photo->current_setting]);
+	rs_settings_double_to_rs_settings(photo->settings[rs->current_setting],
+		rs->settings[rs->current_setting]);
 	photo->filename = NULL;
 	rs_photo_free(photo);
-	update_preview(rs, TRUE, FALSE);
+	rs_update_preview(rs);
 	return;
 }
 
@@ -1406,7 +1334,7 @@ gui_menu_copy_callback(gpointer callback_data, guint callback_action, GtkWidget
 	{
 		if (!rs->settings_buffer)
 			rs->settings_buffer = g_malloc(sizeof(RS_SETTINGS_DOUBLE));
-		rs_settings_to_rs_settings_double(rs->settings[rs->photo->current_setting], rs->settings_buffer);
+		rs_settings_to_rs_settings_double(rs->settings[rs->current_setting], rs->settings_buffer);
 		gui_status_notify(_("Copied settings"));
 	}
 	return;
@@ -1497,7 +1425,6 @@ gui_menu_paste_callback(gpointer callback_data, guint callback_action, GtkWidget
 			{
 				/* This is nothing but a hack around rs_cache_*() */
 				photo = rs_photo_new();
-				photo->active = TRUE;
 				photo->filename = g_strdup(g_list_nth_data(selected, cur));
 				if ((filetype = rs_filetype_get(photo->filename, TRUE)))
 				{
@@ -1515,7 +1442,7 @@ gui_menu_paste_callback(gpointer callback_data, guint callback_action, GtkWidget
 						}
 					}
 					rs_cache_load(photo);
-					rs_settings_double_copy(rs->settings_buffer, photo->settings[rs->photo->current_setting], mask);
+					rs_settings_double_copy(rs->settings_buffer, photo->settings[rs->current_setting], mask);
 					rs_cache_save(photo);
 				}
 				rs_photo_free(photo);
@@ -1527,9 +1454,9 @@ gui_menu_paste_callback(gpointer callback_data, guint callback_action, GtkWidget
 			{
 				gboolean in_use = rs->in_use;
 				rs->in_use = FALSE;
-				rs_apply_settings_from_double(rs->settings[rs->photo->current_setting], rs->settings_buffer, mask);
+				rs_apply_settings_from_double(rs->settings[rs->current_setting], rs->settings_buffer, mask);
 				rs->in_use = in_use;
-				update_preview(rs, TRUE, FALSE);
+				rs_update_preview(rs);
 			}
 
 			gui_status_notify(_("Pasted settings"));
@@ -1548,7 +1475,7 @@ gui_make_menubar(RS_BLOB *rs, GtkWidget *window, GtkWidget *iconbox, GtkWidget *
 	struct menu_item_t menu_items[] = {
 		{{ _("/_File"), NULL, NULL, 0, "<Branch>"}, NULL},
 		{{ _("/File/_Open directory..."), "<CTRL>O", (gpointer)&gui_menu_open_callback, 1, "<StockItem>", GTK_STOCK_OPEN}, NULL},
-		{{ _("/File/_Export"), "<CTRL>S", (gpointer)&gui_quick_save_file_callback, 1, "<StockItem>", GTK_STOCK_SAVE}, NULL},
+		{{ _("/File/_Quick export"), "<CTRL>S", (gpointer)&gui_quick_save_file_callback, 1, "<StockItem>", GTK_STOCK_SAVE}, NULL},
 		{{ _("/File/_Export as..."), "<CTRL><SHIFT>S", (gpointer)&gui_save_file_callback, 1, "<StockItem>", GTK_STOCK_SAVE_AS}, NULL},
 		{{ _("/File/_Reload"), "<CTRL>R", (gpointer)&gui_menu_reload_callback, 1, "<StockItem>", GTK_STOCK_REFRESH}, NULL},
 		{{ _("/File/_Delete flagged photos"), "<CTRL><SHIFT>D", (gpointer)&gui_menu_purge_d_callback, 0, "<StockItem>", GTK_STOCK_DELETE}, NULL},
@@ -1571,6 +1498,10 @@ gui_make_menubar(RS_BLOB *rs, GtkWidget *window, GtkWidget *iconbox, GtkWidget *
 		{{ _("/_Photo/_Uncrop"),  "<Shift>V", (gpointer)&gui_menu_uncrop_callback, PRIO_U}, NULL},
 		{{ _("/_Photo/_Straighten"),  NULL, (gpointer)&gui_menu_straighten_callback, 1}, NULL},
 		{{ _("/_Photo/_Unstraighten"),  NULL, (gpointer)&gui_menu_straighten_callback, 0}, NULL},
+#ifdef EXPERIMENTAL
+		{{ _("/_Photo/_Group photos"),  "<Ctrl>G", (gpointer)&gui_menu_group_photos_callback, PRIO_1}, NULL},
+		{{ _("/_Photo/_Ungroup photos"),  "<Ctrl><SHIFT>G", (gpointer)&gui_menu_ungroup_photos_callback, PRIO_1}, NULL},
+#endif
 		{{ _("/_View"), NULL, NULL, 0, "<Branch>"}, NULL},
 		{{ _("/_View/_Previous photo"), "<CTRL>Left", (gpointer)&gui_menu_prevnext_callback, 1, "<StockItem>", GTK_STOCK_GO_BACK}, NULL},
 		{{ _("/_View/_Next photo"), "<CTRL>Right", (gpointer)&gui_menu_prevnext_callback, 2, "<StockItem>", GTK_STOCK_GO_FORWARD}, NULL},
@@ -1643,14 +1574,56 @@ gui_make_menubar(RS_BLOB *rs, GtkWidget *window, GtkWidget *iconbox, GtkWidget *
 	return(gtk_item_factory_get_widget (item_factory, "<main>"));
 }
 
+static void
+drag_data_received(GtkWidget *widget, GdkDragContext *drag_context,
+	gint x, gint y, GtkSelectionData *selection_data, guint info, guint t,
+	RS_BLOB *rs)
+{
+	gchar *uris = (gchar *) selection_data->data;
+	gchar *tmp;
+	gchar *filename;
+
+	if (!uris)
+	{
+		gtk_drag_finish (drag_context, FALSE, FALSE, t);
+		return;
+	}
+
+	tmp = uris;
+	while(tmp)
+	{
+		if ((*tmp == '\r') || (*tmp == '\n'))
+		{
+			*tmp = '\0';
+			break;
+		}
+		tmp++;
+	}
+
+	filename = g_filename_from_uri(uris, NULL, NULL);
+
+	rs_open_file(rs, filename);
+
+	g_free(filename);
+
+	gtk_drag_finish(drag_context, TRUE, FALSE, t);
+	return;
+}
+
 static GtkWidget *
 gui_window_make(RS_BLOB *rs)
 {
+	static const GtkTargetEntry targets[] = { { "text/uri-list", 0, 0 } };
+
 	rawstudio_window = GTK_WINDOW(gtk_window_new (GTK_WINDOW_TOPLEVEL));
 	gtk_window_resize((GtkWindow *) rawstudio_window, 800, 600);
 	gtk_window_set_title (GTK_WINDOW (rawstudio_window), _("Rawstudio"));
 	g_signal_connect((gpointer) rawstudio_window, "delete_event", G_CALLBACK(rs_shutdown), rs);
 	g_signal_connect((gpointer) rawstudio_window, "key_press_event", G_CALLBACK(window_key_press_event), NULL);
+
+	gtk_drag_dest_set(GTK_WIDGET(rawstudio_window), GTK_DEST_DEFAULT_ALL, targets, 1, GDK_ACTION_COPY);
+	g_signal_connect((gpointer) rawstudio_window, "drag_data_received", G_CALLBACK(drag_data_received), rs);
+
 	return(GTK_WIDGET(rawstudio_window));
 }
 
@@ -1709,33 +1682,71 @@ gui_dialog_make_from_widget(const gchar *stock_id, gchar *primary_text, GtkWidge
 }
 
 void
-gui_set_values(RS_BLOB *rs, gint x, gint y)
+preview_wb_picked(RSPreviewWidget *preview, RS_PREVIEW_CALLBACK_DATA *cbdata, RS_BLOB *rs)
 {
-	static gchar tmp[20];
-	static gint lx=-1, ly=-1;
-	guchar values[3];
+	rs_set_wb_from_color(rs, cbdata->pixelfloat[R], cbdata->pixelfloat[G], cbdata->pixelfloat[B]);
+}
 
-	if (rs->photo)
-	{
-		if ((x>0) && (y>0))
+void
+preview_motion(RSPreviewWidget *preview, RS_PREVIEW_CALLBACK_DATA *cbdata, RS_BLOB *rs)
+{
+	gchar tmp[20];
+
+	g_snprintf(tmp, 20, "%u %u %u", cbdata->pixel8[R], cbdata->pixel8[G], cbdata->pixel8[B]);
+	gtk_label_set_text(GTK_LABEL(valuefield), tmp);
+}
+
+static void
+rs_open_file(RS_BLOB *rs, const gchar *filename)
+{
+	if (filename)
+	{	
+		gchar *abspath;
+		gchar *temppath = g_strdup(filename);
+		gchar *lwd;
+
+		if (g_path_is_absolute(temppath))
+			abspath = g_strdup(temppath);
+		else
 		{
-			rs_render_pixel_to_srgb(rs, x, y, values);
-			lx = x;
-			ly = y;
-			g_snprintf(tmp, 20, "%u %u %u", values[0], values[1], values[2]);
+			gchar *tmpdir = g_get_current_dir ();
+			abspath = g_build_filename (tmpdir, temppath, NULL);
+			g_free (tmpdir);
 		}
-		else if (lx!=-1)
+		g_free(temppath);
+
+		if (g_file_test(abspath, G_FILE_TEST_IS_DIR))
 		{
-			rs_render_pixel_to_srgb(rs, lx, ly, values);
-			g_snprintf(tmp, 20, "%u %u %u", values[0], values[1], values[2]);
+			rs_store_remove(rs->store, NULL, NULL);
+			if (rs_store_load_directory(rs->store, abspath) >= 0)
+				rs_conf_set_string(CONF_LWD, abspath);
+		}
+		else if (g_file_test(abspath, G_FILE_TEST_IS_REGULAR))
+		{
+			lwd = g_path_get_dirname(abspath);
+			filename = g_path_get_basename(abspath);
+			rs_store_remove(rs->store, NULL, NULL);
+			if (rs_store_load_directory(rs->store, lwd) >= 0)
+				rs_conf_set_string(CONF_LWD, lwd);
+			rs_store_set_selected_name(rs->store, abspath);
+			g_free(lwd);
 		}
 		else
-			tmp[0] = '\0';
+			rs_store_load_directory(rs->store, NULL);
+		g_free(abspath);
 	}
-	else
-		tmp[0] = '\0';
-	gtk_label_set_text(GTK_LABEL(valuefield), tmp);
-	return;
+}
+
+static gboolean
+pane_position(GtkWidget* widget, gpointer dummy, gpointer user_data)
+{
+	GtkPaned *paned = GTK_PANED(widget);
+	gint pos;
+	gint window_width;
+	gtk_window_get_size(rawstudio_window, &window_width, NULL);
+	pos = gtk_paned_get_position(paned);
+	rs_conf_set_integer(CONF_TOOLBOX_WIDTH, window_width - pos);
+	return TRUE;
 }
 
 int
@@ -1749,14 +1760,13 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 	GtkWidget *toolbox;
 	GtkWidget *batchbox;
 	GtkWidget *iconbox;
-	GtkWidget *preview;
 	GtkWidget *menubar;
-	gchar *lwd = NULL;
-	gchar *filename;
+	gint window_width = 0, toolbox_width = 0;
 	GdkColor dashed_bg = {0, 0, 0, 0 };
 	GdkColor dashed_fg = {0, 0, 65535, 0};
 	GdkColor grid_bg = {0, 0, 0, 0 };
 	GdkColor grid_fg = {0, 32767, 32767, 32767};
+	GdkColor bgcolor = {0, 0, 0, 0 };
 
 #ifdef PACKAGE_DATA_DIR
 	gtk_window_set_default_icon_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio.png", NULL);
@@ -1798,11 +1808,18 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 	g_signal_connect((gpointer) window, "window-state-event", G_CALLBACK(gui_fullscreen_toolbox_callback), tools);
 
 	menubar = gui_make_menubar(rs, window, iconbox, tools);
-	preview = gui_drawingarea_make(rs);
+
+	rs->preview = rs_preview_widget_new();
+	rs_preview_widget_set_cms(RS_PREVIEW_WIDGET(rs->preview), rs_cms_get_transform(rs->cms, PROFILE_DISPLAY));
+	rs_conf_get_color(CONF_PREBGCOLOR, &bgcolor);
+	rs_preview_widget_set_bgcolor(RS_PREVIEW_WIDGET(rs->preview), &bgcolor);
+	g_signal_connect(G_OBJECT(rs->preview), "wb-picked", G_CALLBACK(preview_wb_picked), rs);
+	g_signal_connect(G_OBJECT(rs->preview), "motion", G_CALLBACK(preview_motion), rs);
 
 	pane = gtk_hpaned_new ();
+	g_signal_connect_after(G_OBJECT(pane), "notify::position", G_CALLBACK(pane_position), NULL);
 
-	gtk_paned_pack1 (GTK_PANED (pane), preview, TRUE, TRUE);
+	gtk_paned_pack1 (GTK_PANED (pane), rs->preview, TRUE, TRUE);
 	gtk_paned_pack2 (GTK_PANED (pane), tools, FALSE, TRUE);
 
 	vbox = gtk_vbox_new (FALSE, 0);
@@ -1815,8 +1832,6 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 
 	gui_status_push(_("Ready"));
 
-	gtk_widget_show_all (window);
-
 	// arrange rawstudio as the user left it
 	gboolean show_iconbox;
 	gboolean show_toolbox;
@@ -1830,6 +1845,10 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 		gui_widget_show(tools, show_toolbox, CONF_SHOW_TOOLBOX_FULLSCREEN, CONF_SHOW_TOOLBOX);  } 
 	else
 	{
+		gtk_window_get_size(rawstudio_window, &window_width, NULL);
+		if (rs_conf_get_integer(CONF_TOOLBOX_WIDTH, &toolbox_width))
+			gtk_paned_set_position(GTK_PANED(pane), window_width - toolbox_width);
+
 		gtk_window_unfullscreen(GTK_WINDOW(window));
 		rs_conf_get_boolean_with_default(CONF_SHOW_ICONBOX, &show_iconbox, DEFAULT_CONF_SHOW_TOOLBOX);
 		rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX, &show_toolbox, DEFAULT_CONF_SHOW_ICONBOX);
@@ -1837,47 +1856,42 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 		gui_widget_show(tools, show_toolbox, CONF_SHOW_TOOLBOX_FULLSCREEN, CONF_SHOW_TOOLBOX);
 	}
 
+	gtk_widget_show_all (window);
+
+	{
+		gboolean preload = FALSE;
+		rs_conf_get_boolean(CONF_PRELOAD, &preload);
+		if (preload)
+			rs_preload_set_maximum_memory(200*1024*1024);
+		else
+			rs_preload_set_maximum_memory(0);
+	}
+
 	if (argc > 1)
-	{	
-		gchar *abspath;
-		gchar *temppath = g_strdup(argv[1]);
-
-		if (g_path_is_absolute(temppath))
-			abspath = g_strdup(temppath);
-		else
-		{
-			gchar *tmpdir = g_get_current_dir ();
-			abspath = g_build_filename (tmpdir, temppath, NULL);
-			g_free (tmpdir);
-		}
-		g_free(temppath);
-
-		if (g_file_test(abspath, G_FILE_TEST_IS_DIR))
-		{
-			if (rs_store_load_directory(rs->store, abspath) >= 0)
-				rs_conf_set_string(CONF_LWD, abspath);
-		}
-		else if (g_file_test(abspath, G_FILE_TEST_IS_REGULAR))
-		{
-			lwd = g_path_get_dirname(abspath);
-			filename = g_path_get_basename(abspath);
-			rs_store_load_directory(rs->store, lwd);
-			rs_store_set_selected_name(rs->store, abspath);
-			g_free(lwd);
-		}
-		else
-			rs_store_load_directory(rs->store, NULL);
-		g_free(abspath);
+	{
+		rs_open_file(rs, argv[1]);
+		rs_conf_set_integer(CONF_LAST_PRIORITY_PAGE, 0);
 	}
 	else
-	{
-		lwd = rs_conf_get_string(CONF_LWD);
-		if (!lwd)
-			lwd = g_get_current_dir();
-		rs_store_load_directory(rs->store, lwd);
-		g_free(lwd);
-	}
+		{
+			gchar *lwd;
+			lwd = rs_conf_get_string(CONF_LWD);
+			if (!lwd)
+				lwd = g_get_current_dir();
+			if (rs_store_load_directory(rs->store, lwd))
+			{
+				gint last_priority_page = 0;
+				rs_conf_get_integer(CONF_LAST_PRIORITY_PAGE, &last_priority_page);
+				gtk_notebook_set_current_page(GTK_NOTEBOOK(rs->store), last_priority_page);
+			}
+			else
+				rs_conf_set_integer(CONF_LAST_PRIORITY_PAGE, 0);		
+			g_free(lwd);
+		}
+
 	gui_set_busy(FALSE);
+	gdk_threads_enter();
 	gtk_main();
+	gdk_threads_leave();
 	return(0);
 }
