@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -19,6 +19,7 @@
 
 #ifndef GTK_INTERFACE_H
 #define GTK_INTERFACE_H
+#include <gdk/gdkx.h>
 
 enum {
 	OP_NONE = 0,
@@ -26,16 +27,17 @@ enum {
 	OP_MOVE
 };
 
-#define GUI_CATCHUP() while (gtk_events_pending()) gtk_main_iteration()
+#define GUI_CATCHUP() do { \
+  GdkDisplay *__gui_catchup_display = gdk_display_get_default (); \
+  XFlush (GDK_DISPLAY_XDISPLAY (__gui_catchup_display)); } while (0)
+#define GTK_CATCHUP() while (gtk_events_pending()) gtk_main_iteration()
 
 extern void gui_set_busy(gboolean rawstudio_is_busy);
 extern gboolean gui_is_busy();
 extern void gui_status_notify(const char *text);
 extern guint gui_status_push(const char *text);
 extern void gui_status_pop(const guint msgid);
-extern void update_histogram(RS_BLOB *rs);
 extern gboolean update_preview_callback(GtkAdjustment *caller, RS_BLOB *rs);
-extern gboolean update_previewtable_callback(GtkAdjustment *do_not_use_this, RS_BLOB *rs);
 extern gboolean update_scale_callback(GtkAdjustment *do_not_use_this, RS_BLOB *rs);
 extern void icon_set_flags(const gchar *filename, GtkTreeIter *iter, const guint *priority, const gboolean *exported);
 extern void gui_dialog_simple(gchar *title, gchar *message);
