@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -19,8 +19,10 @@
 
 #include <gtk/gtk.h>
 #include <config.h>
+#include "rawstudio.h"
 #include "gettext.h"
 #include "gtk-progress.h"
+#include "gtk-interface.h"
 
 struct _RS_PROGRESS {
 	GtkWidget *window;
@@ -156,6 +158,13 @@ gui_progress_set_current(RS_PROGRESS *rsp, gint current)
 	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(rsp->progressbar), gs->str);
 	g_string_free(gs, TRUE);
 
-	while (gtk_events_pending())
-		gtk_main_iteration();
+	/* Lock if needed */
+	if (g_main_depth() == 0)
+	{
+		gdk_threads_enter();
+		GTK_CATCHUP();
+		gdk_threads_leave();
+	}
+	else
+		GTK_CATCHUP();
 }
