@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -23,9 +23,7 @@
 #include <string.h> /* memset() */
 #include <time.h>
 #include <config.h>
-#include "matrix.h"
 #include "rawstudio.h"
-#include "rs-crop.h"
 #include "gtk-interface.h"
 #include "gtk-helper.h"
 #include "rs-cache.h"
@@ -42,39 +40,33 @@
 #include "filename.h"
 #include "rs-jpeg.h"
 #include "rs-tiff.h"
-#include "rs-render.h"
 #include "rs-arch.h"
 #include "rs-batch.h"
 #include "rs-cms.h"
 #include "rs-store.h"
+#include "rs-color-transform.h"
+#include "rs-preview-widget.h"
+#include "rs-histogram.h"
+#include "rs-curve.h"
 
-static void update_scaled(RS_BLOB *rs, gboolean force);
-static gboolean rs_render_idle(RS_BLOB *rs);
 static RS_SETTINGS *rs_settings_new();
-static RS_SETTINGS_DOUBLE *rs_settings_double_new();
 static void rs_settings_double_free(RS_SETTINGS_DOUBLE *rssd);
-static RS_PHOTO *rs_photo_open_dcraw(const gchar *filename);
-static RS_PHOTO *rs_photo_open_gdk(const gchar *filename);
+static RS_PHOTO *rs_photo_open_dcraw(const gchar *filename, gboolean half_size);
+void rs_photo_open_dcraw_apply_black_and_shift_half_size(dcraw_data *raw, RS_PHOTO *photo);
+static RS_PHOTO *rs_photo_open_gdk(const gchar *filename, gboolean half_size);
 static GdkPixbuf *rs_thumb_gdk(const gchar *src);
-static gboolean rs_mark_roi_configure (GtkWidget *widget, GdkEventExpose *event, RS_BLOB *rs);
 static void rs_rect_flip(RS_RECT *in, RS_RECT *out, gint w, gint h);
 static void rs_rect_mirror(RS_RECT *in, RS_RECT *out, gint w, gint h);
 static void rs_rect_rotate(RS_RECT *in, RS_RECT *out, gint w, gint h, gint quarterturns);
 
 RS_FILETYPE *filetypes;
 
-GdkPixbuf *icon_priority_1;
-GdkPixbuf *icon_priority_2;
-GdkPixbuf *icon_priority_3;
-GdkPixbuf *icon_priority_D;
-GdkPixbuf *icon_exported;
-
 static void
 rs_add_filetype(gchar *id, gint filetype, const gchar *ext, gchar *description,
-	RS_PHOTO *(*load)(const gchar *),
+	RS_PHOTO *(*load)(const gchar *, gboolean),
 	GdkPixbuf *(*thumb)(const gchar *),
 	void (*load_meta)(const gchar *, RS_METADATA *),
-	gboolean (*save)(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width, gint height, gdouble scale, RS_CMS *cms))
+	gboolean (*save)(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width, gint height, gboolean keep_aspect, gdouble scale, gint snapshot, RS_CMS *cms))
 {
 	RS_FILETYPE *cur = filetypes;
 	if (filetypes==NULL)
@@ -122,7 +114,7 @@ rs_init_filetypes(void)
 	rs_add_filetype("raw", FILETYPE_RAW, ".raw", _("Panasonic raw"),
 		rs_photo_open_dcraw, rs_panasonic_load_thumb, rs_panasonic_load_meta, NULL);
 	rs_add_filetype("pef", FILETYPE_RAW, ".pef", _("Pentax raw"),
-		rs_photo_open_dcraw, rs_tiff_load_thumb, NULL, NULL);
+		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("dng", FILETYPE_RAW, "dng", _("Adobe Digital negative"),
 		rs_photo_open_dcraw, rs_tiff_load_thumb, rs_tiff_load_meta, NULL);
 	rs_add_filetype("jpeg", FILETYPE_JPEG, ".jpg", _("JPEG (Joint Photographic Experts Group)"),
@@ -136,404 +128,18 @@ rs_init_filetypes(void)
 	return;
 }
 
-static void
-update_scaled(RS_BLOB *rs, gboolean force)
-{
-	/* scale if needed */
-	if ((rs->preview_scale != GETVAL(rs->scale)) || force)
-	{
-		gdouble scale;
-		rs->preview_scale = GETVAL(rs->scale);
-
-		if (rs->zoom_to_fit)
-			scale = -1.0;
-		else
-			scale = rs->preview_scale;
-
-		if (rs->photo->scaled)
-			rs_image16_free(rs->photo->scaled);
-		rs->photo->scaled = rs_image16_transform(rs->photo->input, NULL,
-			&rs->photo->affine, &rs->photo->inverse_affine, rs->photo->crop,
-			rs->preview_width-12, rs->preview_height-12, TRUE,
-			scale, rs->photo->angle, rs->photo->orientation, NULL);
-		matrix3_affine_transform_point_int(&rs->photo->affine,
-			rs->roi.x1, rs->roi.y1,
-			&rs->roi_scaled.x1, &rs->roi_scaled.y1);
-		matrix3_affine_transform_point_int(&rs->photo->affine,
-			rs->roi.x2, rs->roi.y2,
-			&rs->roi_scaled.x2, &rs->roi_scaled.y2);
-	}
-
-	/* allocate 8 bit buffers if needed */
-	if (!rs_image16_8_cmp_size(rs->photo->scaled, rs->photo->preview))
-	{
-		rs_image8_free(rs->photo->preview);
-		rs->photo->preview = rs_image8_new(rs->photo->scaled->w, rs->photo->scaled->h, 3, 3);
-		gtk_widget_set_size_request(rs->preview_drawingarea, rs->photo->scaled->w, rs->photo->scaled->h);
-	}
-
-	return;
-}
-
-inline void
-rs_photo_prepare(RS_PHOTO *photo)
-{
-	matrix4_identity(&photo->mat);
-#ifdef USE_ADOBE_COEFF
-	matrix4_multiply(&photo->mat, &photo->metadata->adobe_coeff, &photo->mat);
-#endif
-	matrix4_color_exposure(&photo->mat, photo->settings[photo->current_setting]->exposure);
-
-	photo->pre_mul[R] = (1.0+photo->settings[photo->current_setting]->warmth)
-		*(2.0-photo->settings[photo->current_setting]->tint);
-	photo->pre_mul[G] = 1.0;
-	photo->pre_mul[B] = (1.0-photo->settings[photo->current_setting]->warmth)
-		*(2.0-photo->settings[photo->current_setting]->tint);
-	photo->pre_mul[G2] = 1.0;
-
-	matrix4_color_saturate(&photo->mat, photo->settings[photo->current_setting]->saturation);
-	matrix4_color_hue(&photo->mat, photo->settings[photo->current_setting]->hue);
-}
-
 void
-update_preview(RS_BLOB *rs, gboolean update_table, gboolean update_scale)
+rs_update_preview(RS_BLOB *rs)
 {
-	if(unlikely(!rs->photo)) return;
+	if (!rs->in_use) return;
 
-	rs_render_idle_stop(rs);
-	if (update_table)
-		rs_render_previewtable(rs->photo->settings[rs->photo->current_setting]->contrast,
-			rs->photo->settings[rs->photo->current_setting]->curve_samples, rs->previewtable8, rs->previewtable16);
-	update_scaled(rs, update_scale);
-	rs_photo_prepare(rs->photo);
-	update_preview_region(rs, rs->preview_exposed, TRUE);
+	rs_preview_widget_update(RS_PREVIEW_WIDGET(rs->preview));
 
-	/* Reset histogram_table */
-	if (GTK_WIDGET_VISIBLE(rs->histogram_image))
-	{
-		memset(rs->histogram_table, 0x00, sizeof(guint)*3*256);
-		rs_render_histogram_table(&rs->photo->mat, rs->photo->pre_mul, rs->previewtable8, rs->histogram_dataset, (guint *) rs->histogram_table);
-		update_histogram(rs);
-	}
+	/* Update histogram */
+	rs_color_transform_set_from_settings(rs->histogram_transform, rs->photo->settings[rs->current_setting], MASK_ALL);
+	rs_histogram_set_color_transform(RS_HISTOGRAM_WIDGET(rs->histogram), rs->histogram_transform);
 
-	rs->preview_done = FALSE;
-	rs->preview_idle_render_lastrow = 0;
-	if (!rs->preview_idle_render)
-	{
-		rs->preview_idle_render = TRUE;
-		gui_set_busy(TRUE);
-		g_idle_add((GSourceFunc) rs_render_idle, rs);
-	}
-
-	return;
-}	
-
-void
-update_preview_region(RS_BLOB *rs, RS_RECT *region, gboolean force_render)
-{
-	guchar *pixels;
-	gushort *in;
-	gint w, h;
-	extern GdkPixmap *blitter;
-	GdkGC *gc = rs->preview_drawingarea->style->fg_gc[GTK_WIDGET_STATE (rs->preview_drawingarea)];
-
-	if (unlikely(!rs->in_use)) return;
-
-	_CLAMP(region->x2, rs->photo->scaled->w-1);
-	_CLAMP(region->y2, rs->photo->scaled->h-1);
-	w = region->x2-region->x1+1;
-	h = region->y2-region->y1+1;
-
-	/* evil hack to fix crash after zoom */
-	if (unlikely(region->y2 < region->y1)) /* FIXME: this is not good */
-		return;
-	if (unlikely(region->x2 < region->x1))
-		return;
-
-	pixels = GET_PIXEL(rs->photo->preview, region->x1, region->y1);
-	in = GET_PIXEL(rs->photo->scaled, region->x1, region->y1);
-
-	if (likely((!rs->preview_done) || force_render))
-	{
-		rs_render(&rs->photo->mat, rs->photo->pre_mul,
-			rs->previewtable8, rs->previewtable16,
-			w, h, in, rs->photo->scaled->rowstride,
-			pixels, rs->photo->preview->rowstride,
-			rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
-		if (unlikely(rs->show_exposure_overlay))
-			rs_image8_render_exposure_mask(rs->photo->preview, -1);
-
-		if (unlikely(rs->mark_roi))
-		{
-			guchar *buffer;
-			gint srcoffset, dstoffset;
-			gint row=h-1, col;
-
-			buffer = g_malloc(h * w * rs->photo->preview->pixelsize);
-			while(row--) /* render pixels outside ROI */
-			{
-				col = w * rs->photo->preview->pixelsize;
-				dstoffset = col * row;
-				srcoffset = rs->photo->preview->rowstride * (region->y1+row);
-				while(col--)
-				{
-					buffer[dstoffset] = ((rs->photo->preview->pixels[srcoffset]+63)*3)>>3;
-					srcoffset++;
-					dstoffset++;
-				}
-			}
-			gdk_draw_rgb_image(rs->preview_backing_notroi, gc, /* not ROI */
-				region->x1, region->y1, w, h,
-				GDK_RGB_DITHER_NONE, buffer, w * rs->photo->preview->pixelsize);
-			g_free(buffer);
-		}
-	}
-
-	if (unlikely(rs->mark_roi))
-	{
-		static PangoLayout *text_layout = NULL;
-		gint text_width;
-		gint text_height;
-		static GString *text = NULL;
-		gint x1, x2, y1, y2;
-		extern GdkGC *dashed;
-		extern GdkGC *grid;
-		x1 = rs->roi_scaled.x1;
-		y1 = rs->roi_scaled.y1;
-		x2 = rs->roi_scaled.x2;
-		y2 = rs->roi_scaled.y2;
-
-		if (unlikely(!text))
-			text = g_string_new("");
-
-		if (unlikely(!text_layout))
-			text_layout = gtk_widget_create_pango_layout(rs->preview_drawingarea, "");
-
-		g_string_printf(text, "%d x %d", rs->roi.x2-rs->roi.x1+1, rs->roi.y2-rs->roi.y1+1);
-		pango_layout_set_text(text_layout, text->str, -1);
-		pango_layout_get_pixel_size(text_layout, &text_width, &text_height);
-
-		pixels = GET_PIXEL(rs->photo->preview, rs->roi_scaled.x1, rs->roi_scaled.y1);
-		/* draw all our stuff to blit-buffer */
-		gdk_draw_drawable(blitter, gc, /* not ROI */
-			rs->preview_backing_notroi,
-			region->x1, region->y1,
-			region->x1, region->y1,
-			region->x2-region->x1+1,
-			region->y2-region->y1+1+text_height+2);
-		if (likely(((rs->roi.x2-rs->roi.x1)*(rs->roi.y2-rs->roi.y1)) > 0))
-		{
-			gdk_draw_rgb_image(blitter, gc, /* ROI */
-				rs->roi_scaled.x1, rs->roi_scaled.y1,
-				rs->roi_scaled.x2-rs->roi_scaled.x1,
-				rs->roi_scaled.y2-rs->roi_scaled.y1,
-				GDK_RGB_DITHER_NONE, pixels, rs->photo->preview->rowstride);
-			gdk_draw_rectangle(blitter, dashed, FALSE,
-				rs->roi_scaled.x1, rs->roi_scaled.y1,
-				rs->roi_scaled.x2-rs->roi_scaled.x1-1,
-				rs->roi_scaled.y2-rs->roi_scaled.y1-1);
-			
-			if (rs->photo->scaled->h-text_height-4 > y2)
-			{
-				gdk_draw_layout(blitter, dashed,
-					rs->roi_scaled.x1+(rs->roi_scaled.x2-rs->roi_scaled.x1-text_width)/2,
-					rs->roi_scaled.y2+2,
-					text_layout);
-			}
-			else
-			{
-				gdk_draw_layout(blitter, dashed,
-					rs->roi_scaled.x1+(rs->roi_scaled.x2-rs->roi_scaled.x1-text_width)/2,
-					rs->roi_scaled.y2-text_height-2,
-					text_layout);
-			}
-		}
-
-/*
-		We should support all these:
-		http://powerretouche.com/Divine_proportion_tutorial.htm
-*/
-		switch(rs->roi_grid)
-		{
-			case ROI_GRID_NONE:
-				break;
-			case ROI_GRID_GOLDEN:
-			{
-				gdouble goldenratio = ((1+sqrt(5))/2);
-				gint t, golden;
-
-				/* vertical */
-				golden = ((x2-x1)/goldenratio);
-
-				t = (x1+golden);
-				gdk_draw_line(blitter, grid, t, y1, t, y2);
-				t = (x2-golden);
-				gdk_draw_line(blitter, grid, t, y1, t, y2);
-
-				/* horizontal */
-				golden = ((y2-y1)/goldenratio);
-
-				t = (y1+golden);
-				gdk_draw_line(blitter, grid, x1, t, x2, t);
-				t = (y2-golden);
-				gdk_draw_line(blitter, grid, x1, t, x2, t);
-				break;
-			}
-
-			case ROI_GRID_THIRDS:
-			{
-				gint t;
-
-				/* vertical */
-				t = ((x2-x1+1)/3*1+x1);
-				gdk_draw_line(blitter, grid, t, y1, t, y2);
-				t = ((x2-x1+1)/3*2+x1);
-				gdk_draw_line(blitter, grid, t, y1, t, y2);
-
-				/* horizontal */
-				t = ((y2-y1+1)/3*1+y1);
-				gdk_draw_line(blitter, grid, x1, t, x2, t);
-				t = ((y2-y1+1)/3*2+y1);
-				gdk_draw_line(blitter, grid, x1, t, x2, t);
-				break;
-			}
-
-			case ROI_GRID_GOLDEN_TRIANGLES1:
-			{
-				gdouble goldenratio = ((1+sqrt(5))/2);
-				gint t, golden;
-
-				golden = ((x2-x1)/goldenratio);
-
-				gdk_draw_line(blitter, grid, x1, y1, x2, y2);
-
-				t = (x2-golden);
-				gdk_draw_line(blitter, grid, x1, y2, t, y1);
-
-				t = (x1+golden);
-				gdk_draw_line(blitter, grid, x2, y1, t, y2);
-				break;
-			}
-
-			case ROI_GRID_GOLDEN_TRIANGLES2:
-			{
-				gdouble goldenratio = ((1+sqrt(5))/2);
-				gint t, golden;
-
-				golden = ((x2-x1)/goldenratio);
-
-				gdk_draw_line(blitter, grid, x2, y1, x1, y2);
-
-				t = (x2-golden);
-				gdk_draw_line(blitter, grid, x1, y1, t, y2);
-
-				t = (x1+golden);
-				gdk_draw_line(blitter, grid, x2, y2, t, y1);
-				break;
-			}
-
-			case ROI_GRID_HARMONIOUS_TRIANGLES1:
-			{
-				gdouble goldenratio = ((1+sqrt(5))/2);
-				gint t, golden;
-
-				golden = ((x2-x1)/goldenratio);
-
-				gdk_draw_line(blitter, grid, x1, y1, x2, y2);
-
-				t = (x1+golden);
-				gdk_draw_line(blitter, grid, x1, y2, t, y1);
-
-				t = (x2-golden);
-				gdk_draw_line(blitter, grid, x2, y1, t, y2);
-				break;
-			}
-
-			case ROI_GRID_HARMONIOUS_TRIANGLES2:
-			{
-				gdouble goldenratio = ((1+sqrt(5))/2);
-				gint t, golden;
-
-				golden = ((x2-x1)/goldenratio);
-
-				gdk_draw_line(blitter, grid, x1, y2, x2, y1);
-
-				t = (x1+golden);
-				gdk_draw_line(blitter, grid, x1, y1, t, y2);
-
-				t = (x2-golden);
-				gdk_draw_line(blitter, grid, x2, y2, t, y1);
-				break;
-			}
-		}
-		/* blit to screen */
-		gdk_draw_drawable(rs->preview_drawingarea->window, gc, blitter,
-			region->x1, region->y1,
-			region->x1, region->y1,
-			w, h+text_height+2);
-	}
-	else
-	{
-		gdk_draw_rgb_image(rs->preview_drawingarea->window, gc,
-			region->x1, region->y1, w, h, GDK_RGB_DITHER_NONE,
-			pixels, rs->photo->preview->rowstride);
-	}
-	return;
-}
-
-void
-rs_render_idle_stop(RS_BLOB *rs)
-{
-	rs->preview_done = TRUE; /* stop rs_render_idle() */
-	rs_render_idle(rs);
-}
-
-static gboolean
-rs_render_idle(RS_BLOB *rs)
-{
-	gint row;
-	gushort *in;
-	guchar *out;
-
-	if (rs->in_use && (!rs->preview_done))
-		for(row=rs->preview_idle_render_lastrow; row<rs->photo->scaled->h; row++)
-		{
-			in = rs->photo->scaled->pixels + row*rs->photo->scaled->rowstride;
-			out = rs->photo->preview->pixels + row*rs->photo->preview->rowstride;
-
-			rs_render(&rs->photo->mat, rs->photo->pre_mul,
-				rs->previewtable8, rs->previewtable16,
-				rs->photo->scaled->w, 1, in, rs->photo->scaled->rowstride,
-				out, rs->photo->preview->rowstride, rs_cms_get_transform(rs->cms, TRANSFORM_DISPLAY));
-			if (unlikely(rs->show_exposure_overlay))
-				rs_image8_render_exposure_mask(rs->photo->preview, row);
-	
-			gdk_draw_rgb_image(rs->preview_backing,
-				rs->preview_drawingarea->style->fg_gc[GTK_STATE_NORMAL], 0, row,
-				rs->photo->scaled->w, 1, GDK_RGB_DITHER_NONE, out,
-				rs->photo->preview->rowstride);
-			if (rs->mark_roi)
-			{
-				gint size = rs->photo->preview->rowstride;
-				guchar *buffer;
-		
-				buffer = g_malloc(size);
-				while(size--) /* render backing store for pixels outside ROI */
-					buffer[size] = ((out[size]+63)*3)>>3;
-				gdk_draw_rgb_image(rs->preview_backing_notroi,
-					rs->preview_drawingarea->style->fg_gc[GTK_STATE_NORMAL], 0, row,
-					rs->photo->scaled->w, 1, GDK_RGB_DITHER_NONE, buffer,
-					rs->photo->preview->rowstride);
-				g_free(buffer);
-			}
-			rs->preview_idle_render_lastrow=row+1;
-			if (gtk_events_pending()) return(TRUE);
-		}
-	rs->preview_idle_render_lastrow = 0;
-	rs->preview_done = TRUE;
-	rs->preview_idle_render = FALSE;
-	gui_set_busy(FALSE);
-	return(FALSE);
+	rs_curve_draw_histogram(RS_CURVE_WIDGET(rs->settings[rs->current_setting]->curve), rs->histogram_dataset, rs->photo->settings[rs->current_setting]);
 }
 
 void
@@ -542,11 +148,9 @@ rs_reset(RS_BLOB *rs)
 	gboolean in_use = rs->in_use;
 	gint c;
 	rs->in_use = FALSE;
-	rs->preview_scale = 0;
 	for(c=0;c<3;c++)
 		rs_settings_reset(rs->settings[c], MASK_ALL);
 	rs->in_use = in_use;
-	update_preview(rs, TRUE, FALSE);
 	return;
 }
 
@@ -575,7 +179,6 @@ rs_settings_to_rs_settings_double(RS_SETTINGS *rs_settings, RS_SETTINGS_DOUBLE *
 	rs_settings_double->contrast = GETVAL(rs_settings->contrast);
 	rs_settings_double->warmth = GETVAL(rs_settings->warmth);
 	rs_settings_double->tint = GETVAL(rs_settings->tint);
-	rs_settings_double->curve_samples = rs_settings->curve_samples;
 	rs_curve_widget_get_knots(RS_CURVE_WIDGET(rs_settings->curve), &rs_settings_double->curve_knots, &rs_settings_double->curve_nknots);
 	return;
 }
@@ -636,7 +239,6 @@ rs_settings_reset(RS_SETTINGS *rss, guint mask)
 static RS_SETTINGS *
 rs_settings_new(void)
 {
-	gint i;
 	RS_SETTINGS *rss;
 	rss = g_malloc(sizeof(RS_SETTINGS));
 	rss->exposure = gtk_adjustment_new(0.0, -3.0, 3.0, 0.1, 0.5, 0.0);
@@ -646,14 +248,11 @@ rs_settings_new(void)
 	rss->warmth = gtk_adjustment_new(0.0, -2.0, 2.0, 0.1, 0.5, 0.0);
 	rss->tint = gtk_adjustment_new(0.0, -2.0, 2.0, 0.1, 0.5, 0.0);
 	rss->curve = rs_curve_widget_new();
-	rs_curve_widget_set_array(RS_CURVE_WIDGET(rss->curve), rss->curve_samples, 65536);
-	for (i=0;i<65536;i++)
-		rss->curve_samples[i] = ((gfloat) i)/65535.0f;
 	return(rss);
 }
 
-static RS_SETTINGS_DOUBLE
-*rs_settings_double_new(void)
+RS_SETTINGS_DOUBLE *
+rs_settings_double_new(void)
 {
 	RS_SETTINGS_DOUBLE *rssd;
 	rssd = g_malloc(sizeof(RS_SETTINGS_DOUBLE));
@@ -663,7 +262,6 @@ static RS_SETTINGS_DOUBLE
 	rssd->contrast = 1.0;
 	rssd->warmth = 0.0;
 	rssd->tint = 0.0;
-	rssd->curve_samples = NULL;
 	rssd->curve_nknots = 0;
 	rssd->curve_knots = NULL;
 	return rssd;
@@ -778,13 +376,9 @@ rs_photo_new()
 	RS_PHOTO *photo;
 	photo = g_malloc(sizeof(RS_PHOTO));
 	photo->filename = NULL;
-	photo->active = FALSE;
 	if (!photo) return(NULL);
 	photo->input = NULL;
-	photo->scaled = NULL;
-	photo->preview = NULL;
 	ORIENTATION_RESET(photo->orientation);
-	photo->current_setting = 0;
 	photo->priority = PRIO_U;
 	photo->metadata = rs_metadata_new();
 	for(c=0;c<3;c++)
@@ -792,7 +386,6 @@ rs_photo_new()
 	photo->crop = NULL;
 	photo->angle = 0.0;
 	photo->exported = FALSE;
-	matrix3_identity(&photo->inverse_affine);
 	return(photo);
 }
 
@@ -840,8 +433,28 @@ rs_photo_rotate(RS_PHOTO *photo, gint quarterturns, gdouble angle)
 
 }
 
+void
+rs_photo_set_crop(RS_PHOTO *photo, RS_RECT *crop)
+{
+	if (photo->crop)
+		g_free(photo->crop);
+	photo->crop = NULL;
+
+	if (crop)
+	{
+		photo->crop = g_new(RS_RECT, 1);
+		*photo->crop = *crop;
+	}
+}
+
+RS_RECT *
+rs_photo_get_crop(RS_PHOTO *photo)
+{
+	return photo->crop;
+}
+
 gboolean
-rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width, gint height, gdouble scale, RS_CMS *cms)
+rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width, gint height, gboolean keep_aspect, gdouble scale, gint snapshot, RS_CMS *cms)
 {
 	GdkPixbuf *pixbuf;
 	RS_IMAGE16 *rsi;
@@ -849,24 +462,30 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 	RS_IMAGE16 *image16;
 	gint quality = 100;
 	gboolean uncompressed_tiff = FALSE;
-	guchar table8[65536];
-	gushort table16[65536];
+	RS_COLOR_TRANSFORM *rct;
+	void *transform = NULL;
 
 	/* transform and crop */
 	rsi = rs_image16_transform(photo->input, NULL,
-			NULL, NULL, photo->crop, width, height, FALSE, scale,
+			NULL, NULL, photo->crop, width, height, keep_aspect, scale,
 			photo->angle, photo->orientation, NULL);
 
-	rs_render_previewtable(photo->settings[photo->current_setting]->contrast,
-		photo->settings[photo->current_setting]->curve_samples, table8, table16);
+	if (cms)
+		transform = rs_cms_get_transform(cms, TRANSFORM_EXPORT);
+
+	/* Initialize color transform */
+	rct = rs_color_transform_new();
+	rs_color_transform_set_from_settings(rct, photo->settings[snapshot], MASK_ALL);
+	rs_color_transform_set_output_format(rct, 8);
+	rs_color_transform_set_cms_transform(rct, transform);
+
 	/* actually save */
 	switch (filetype)
 	{
 		case FILETYPE_JPEG:
 			image8 = rs_image8_new(rsi->w, rsi->h, 3, 3);
-			rs_render(&photo->mat, photo->pre_mul, table8, table16, rsi->w, rsi->h, rsi->pixels,
-				rsi->rowstride, image8->pixels, image8->rowstride,
-				rs_cms_get_transform(cms, TRANSFORM_EXPORT));
+			rct->transform(rct, rsi->w, rsi->h, rsi->pixels,
+				rsi->rowstride, image8->pixels, image8->rowstride);
 
 			rs_conf_get_integer(CONF_EXPORT_JPEG_QUALITY, &quality);
 			if (quality > 100)
@@ -879,33 +498,33 @@ rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width,
 			break;
 		case FILETYPE_PNG:
 			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, rsi->w, rsi->h);
-			rs_render(&photo->mat, photo->pre_mul, table8, table16, rsi->w, rsi->h, rsi->pixels, rsi->rowstride,
-				gdk_pixbuf_get_pixels(pixbuf), gdk_pixbuf_get_rowstride(pixbuf),
-				rs_cms_get_transform(cms, TRANSFORM_EXPORT));
+			rct->transform(rct, rsi->w, rsi->h, rsi->pixels, rsi->rowstride,
+				gdk_pixbuf_get_pixels(pixbuf), gdk_pixbuf_get_rowstride(pixbuf));
 			gdk_pixbuf_save(pixbuf, filename, "png", NULL, NULL);
 			g_object_unref(pixbuf);
 			break;
 		case FILETYPE_TIFF8:
 			rs_conf_get_boolean(CONF_EXPORT_TIFF_UNCOMPRESSED, &uncompressed_tiff);
 			image8 = rs_image8_new(rsi->w, rsi->h, 3, 3);
-			rs_render(&photo->mat, photo->pre_mul, table8, table16, rsi->w, rsi->h, rsi->pixels,
-				rsi->rowstride, image8->pixels, image8->rowstride,
-				rs_cms_get_transform(cms, TRANSFORM_EXPORT));
+			rct->transform(rct, rsi->w, rsi->h, rsi->pixels,
+				rsi->rowstride, image8->pixels, image8->rowstride);
 			rs_tiff8_save(image8, filename, rs_cms_get_profile_filename(cms, PROFILE_EXPORT), uncompressed_tiff);
 			rs_image8_free(image8);
 			break;
 		case FILETYPE_TIFF16:
 			rs_conf_get_boolean(CONF_EXPORT_TIFF_UNCOMPRESSED, &uncompressed_tiff);
 			image16 = rs_image16_new(rsi->w, rsi->h, 3, 3);
-			rs_render16(&photo->mat, photo->pre_mul, table16, rsi->w, rsi->h, rsi->pixels,
-				rsi->rowstride, image16->pixels, image16->rowstride,
-				rs_cms_get_transform(cms, TRANSFORM_EXPORT16));
+			rs_color_transform_set_output_format(rct, 16);
+			rct->transform(rct, rsi->w, rsi->h,
+				rsi->pixels, rsi->rowstride,
+				image16->pixels, image16->rowstride*2);
 			rs_tiff16_save(image16, filename, rs_cms_get_profile_filename(cms, PROFILE_EXPORT), uncompressed_tiff);
 			rs_image16_free(image16);
 			break;
 	}
 
 	rs_image16_free(rsi);
+	rs_color_transform_free(rct);
 
 	photo->exported = TRUE;
 	rs_cache_save(photo);
@@ -931,16 +550,6 @@ rs_photo_free(RS_PHOTO *photo)
 		rs_image16_free(photo->input);
 		photo->input = NULL;
 	}
-	if (photo->scaled)
-	{
-		rs_image16_free(photo->scaled);
-		photo->scaled = NULL;
-	}
-	if (photo->preview)
-	{
-		rs_image8_free(photo->preview);
-		photo->preview = NULL;
-	}
 	for(c=0;c<3;c++)
 		rs_settings_double_free(photo->settings[c]);
 	if (photo->crop)
@@ -958,47 +567,28 @@ rs_new(void)
 	RS_BLOB *rs;
 	guint c;
 	rs = g_malloc(sizeof(RS_BLOB));
-	rs->scale = gtk_adjustment_new(0.5, 0.1, 1.0, 0.01, 0.1, 0.0);
-	g_signal_connect(G_OBJECT(rs->scale), "value_changed",
-		G_CALLBACK(update_scale_callback), rs);
 	rs->histogram_dataset = NULL;
-	rs->preview_exposed = (RS_RECT *) g_malloc(sizeof(RS_RECT));
-	rs->preview_backing = NULL;
-	rs->preview_backing_notroi = NULL;
-	rs->preview_done = FALSE;
-	rs->preview_idle_render = FALSE;
+	rs->histogram_transform = rs_color_transform_new();
 	rs->settings_buffer = NULL;
 	rs->in_use = FALSE;
-	rs->mark_roi = FALSE;
-	rs->roi_grid = ROI_GRID_NONE;
-	rs->show_exposure_overlay = FALSE;
 	rs->photo = NULL;
 	rs->queue = rs_batch_new_queue();
-	rs->zoom_to_fit = TRUE;
 	rs->current_setting = 0;
 	for(c=0;c<3;c++)
 		rs->settings[c] = rs_settings_new();
 	return(rs);
 }
 
-void
-rs_zoom_to_fit(RS_BLOB *rs)
-{
-	rs->zoom_to_fit = TRUE;
-	update_preview(rs, FALSE, TRUE);
-}
-
 void
 rs_photo_close(RS_PHOTO *photo)
 {
 	if (!photo) return;
 	rs_cache_save(photo);
-	photo->active = FALSE;
 	return;
 }
 
 static RS_PHOTO *
-rs_photo_open_dcraw(const gchar *filename)
+rs_photo_open_dcraw(const gchar *filename, gboolean half_size)
 {
 	dcraw_data *raw;
 	RS_PHOTO *photo=NULL;
@@ -1008,18 +598,58 @@ rs_photo_open_dcraw(const gchar *filename)
 	{
 		dcraw_load_raw(raw);
 		photo = rs_photo_new(NULL);
-		photo->input = rs_image16_new(raw->raw.width, raw->raw.height, 4, 4);
 
-		rs_photo_open_dcraw_apply_black_and_shift(raw, photo);
+		if (half_size)
+		{
+			photo->input = rs_image16_new(raw->raw.width, raw->raw.height, raw->raw.colors, 4);
+			rs_photo_open_dcraw_apply_black_and_shift_half_size(raw, photo);
+		}
+		else
+		{
+			photo->input = rs_image16_new(raw->raw.width*2, raw->raw.height*2, raw->raw.colors, 4);
+			rs_photo_open_dcraw_apply_black_and_shift(raw, photo);
+		}
 
+		photo->input->filters = raw->filters;
+		photo->input->fourColorFilters = raw->fourColorFilters;
 		photo->filename = g_strdup(filename);
 		dcraw_close(raw);
-		photo->active = TRUE;
 	}
 	g_free(raw);
 	return(photo);
 }
 
+void
+rs_photo_open_dcraw_apply_black_and_shift_half_size(dcraw_data *raw, RS_PHOTO *photo)
+{
+	gushort *dst, *src;
+	gint row, col;
+	gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
+
+	for(row=0;row<(raw->raw.height);row++)
+	{
+		src = (gushort *) raw->raw.image + row * raw->raw.width * 4;
+		dst = GET_PIXEL(photo->input, 0, row);
+		col = raw->raw.width;
+		while(col--)
+		{
+			register gint r, g, b, g2;
+			r  = *src++ - raw->black;
+			g  = *src++ - raw->black;
+			b  = *src++ - raw->black;
+			g2 = *src++ - raw->black;
+			r  = MAX(0, r);
+			g  = MAX(0, g);
+			b  = MAX(0, b);
+			g2 = MAX(0, g2);
+			*dst++ = (gushort)( r<<shift);
+			*dst++ = (gushort)( g<<shift);
+			*dst++ = (gushort)( b<<shift);
+			*dst++ = (gushort)(g2<<shift);
+		}
+	}
+}
+
 /* Function pointer. Initiliazed by arch binder */
 void
 (*rs_photo_open_dcraw_apply_black_and_shift)(dcraw_data *raw, RS_PHOTO *photo);
@@ -1027,32 +657,43 @@ void
 void
 rs_photo_open_dcraw_apply_black_and_shift_c(dcraw_data *raw, RS_PHOTO *photo)
 {
-	guint srcoffset;
-	guint destoffset;
-	guint x;
-	guint y;
-	gushort *src = (gushort*)raw->raw.image;
+	gushort *dst1, *dst2, *src;
+	gint row, col;
 	gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
 
-	for (y=0; y<raw->raw.height; y++)
+	for(row=0;row<(raw->raw.height*2);row+=2)
 	{
-		destoffset = y*photo->input->rowstride;
-		srcoffset = y * raw->raw.width * 4;
-		for (x=0; x<raw->raw.width; x++)
+		src = (gushort *) raw->raw.image + row/2 * raw->raw.width * 4;
+		dst1 = GET_PIXEL(photo->input, 0, row);
+		dst2 = GET_PIXEL(photo->input, 0, row+1);
+		col = raw->raw.width;
+		while(col--)
 		{
 			register gint r, g, b, g2;
-			r  = src[srcoffset++] - raw->black;
-			g  = src[srcoffset++] - raw->black;
-			b  = src[srcoffset++] - raw->black;
-			g2 = src[srcoffset++] - raw->black;
+			r  = *src++ - raw->black;
+			g  = *src++ - raw->black;
+			b  = *src++ - raw->black;
+			g2 = *src++ - raw->black;
 			r  = MAX(0, r);
 			g  = MAX(0, g);
 			b  = MAX(0, b);
 			g2 = MAX(0, g2);
-			photo->input->pixels[destoffset++] = (gushort)( r<<shift);
-			photo->input->pixels[destoffset++] = (gushort)( g<<shift);
-			photo->input->pixels[destoffset++] = (gushort)( b<<shift);
-			photo->input->pixels[destoffset++] = (gushort)(g2<<shift);
+			*dst1++ = (gushort)( r<<shift);
+			*dst1++ = (gushort)( g<<shift);
+			*dst1++ = (gushort)( b<<shift);
+			*dst1++ = (gushort)(g2<<shift);
+			*dst1++ = (gushort)( r<<shift);
+			*dst1++ = (gushort)( g<<shift);
+			*dst1++ = (gushort)( b<<shift);
+			*dst1++ = (gushort)(g2<<shift);
+			*dst2++ = (gushort)( r<<shift);
+			*dst2++ = (gushort)( g<<shift);
+			*dst2++ = (gushort)( b<<shift);
+			*dst2++ = (gushort)(g2<<shift);
+			*dst2++ = (gushort)( r<<shift);
+			*dst2++ = (gushort)( g<<shift);
+			*dst2++ = (gushort)( b<<shift);
+			*dst2++ = (gushort)(g2<<shift);
 		}
 	}
 }
@@ -1062,23 +703,23 @@ void
 rs_photo_open_dcraw_apply_black_and_shift_mmx(dcraw_data *raw, RS_PHOTO *photo)
 {
 	char b[8];
-	gushort *sub = (gushort *) b;
+	volatile gushort *sub = (gushort *) b;
 	void *srcoffset;
 	void *destoffset;
 	guint x;
 	guint y;
 	gushort *src = (gushort*)raw->raw.image;
-	gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
+	volatile gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
 
 	sub[0] = raw->black;
 	sub[1] = raw->black;
 	sub[2] = raw->black;
 	sub[3] = raw->black;
 
-	for (y=0; y<raw->raw.height; y++)
+	for (y=0; y<(raw->raw.height*2); y++)
 	{
 		destoffset = (void*) (photo->input->pixels + y*photo->input->rowstride);
-		srcoffset = (void*) (src + y * photo->input->w * photo->input->pixelsize);
+		srcoffset = (void*) (src + y/2 * raw->raw.width * photo->input->pixelsize);
 		x = raw->raw.width;
 		asm volatile (
 			"mov %3, %%"REG_a"\n\t" /* copy x to %eax */
@@ -1098,12 +739,16 @@ rs_photo_open_dcraw_apply_black_and_shift_mmx(dcraw_data *raw, RS_PHOTO *photo)
 			"psllw %%mm6, %%mm1\n\t"
 			"psllw %%mm6, %%mm2\n\t"
 			"psllw %%mm6, %%mm3\n\t"
-			"movq %%mm0, (%0)\n\t" /* write destination */
-			"movq %%mm1, 8(%0)\n\t"
-			"movq %%mm2, 16(%0)\n\t"
-			"movq %%mm3, 24(%0)\n\t"
+			"movq %%mm0, (%0)\n\t" /* write destination (twice) */
+			"movq %%mm0, 8(%0)\n\t"
+			"movq %%mm1, 16(%0)\n\t"
+			"movq %%mm1, 24(%0)\n\t"
+			"movq %%mm2, 32(%0)\n\t"
+			"movq %%mm2, 40(%0)\n\t"
+			"movq %%mm3, 48(%0)\n\t"
+			"movq %%mm3, 56(%0)\n\t"
 			"sub $4, %%"REG_a"\n\t"
-			"add $32, %0\n\t"
+			"add $64, %0\n\t"
 			"add $32, %1\n\t"
 			"cmp $3, %%"REG_a"\n\t"
 			"jg load_raw_inner_loop\n\t"
@@ -1158,7 +803,7 @@ rs_filetype_get(const gchar *filename, gboolean load)
 }
 
 static RS_PHOTO *
-rs_photo_open_gdk(const gchar *filename)
+rs_photo_open_gdk(const gchar *filename, gboolean half_size)
 {
 	RS_PHOTO *photo=NULL;
 	GdkPixbuf *pixbuf;
@@ -1205,6 +850,25 @@ rs_photo_open_gdk(const gchar *filename)
 	return(photo);
 }
 
+gchar *
+rs_confdir_get()
+{
+	static gchar *dir = NULL;
+
+	if (!dir)
+	{
+		GString *gs = g_string_new(g_get_home_dir());
+		g_string_append(gs, G_DIR_SEPARATOR_S);
+		g_string_append(gs, ".rawstudio/");
+		dir = gs->str;
+		g_string_free(gs, FALSE);
+	}
+
+	g_mkdir_with_parents(dir, 00755);
+
+	return dir;
+}
+
 gchar *
 rs_dotdir_get(const gchar *filename)
 {
@@ -1293,6 +957,49 @@ rs_thumb_gdk(const gchar *src)
 	return(pixbuf);
 }
 
+void
+rs_white_black_point(RS_BLOB *rs)
+{
+	if (rs->photo)
+	{
+		guint hist[4][256];
+		gint i = 0;
+		gdouble black_threshold = 0.003; // Percent underexposed pixels
+		gdouble white_threshold = 0.01; // Percent overexposed pixels
+		gdouble blackpoint;
+		gdouble whitepoint;
+		guint total = 0;
+		RS_COLOR_TRANSFORM *rct;
+
+		rct = rs_color_transform_new();
+		rs_color_transform_set_from_settings(rct, rs->photo->settings[rs->current_setting], MASK_ALL ^ MASK_CURVE);
+		rs_color_transform_make_histogram(rct, rs->histogram_dataset, hist);
+		rs_color_transform_free(rct);
+
+		// calculate black point
+		while(i < 256) {
+			total += hist[R][i]+hist[G][i]+hist[B][i];
+			if ((total/3) > ((250*250*3)/100*black_threshold))
+				break;
+			i++;
+		}
+		blackpoint = (gdouble) i / (gdouble) 255;
+		
+		// calculate white point
+		i = 255;
+		while(i) {
+			total += hist[R][i]+hist[G][i]+hist[B][i];
+			if ((total/3) > ((250*250*3)/100*white_threshold))
+				break;
+			i--;
+		}
+		whitepoint = (gdouble) i / (gdouble) 255;
+
+		rs_curve_widget_move_knot(RS_CURVE_WIDGET(rs->settings[rs->current_setting]->curve),0,blackpoint,0.0);
+		rs_curve_widget_move_knot(RS_CURVE_WIDGET(rs->settings[rs->current_setting]->curve),-1,whitepoint,1.0);
+	}
+}
+
 void
 rs_set_wb_auto(RS_BLOB *rs)
 {
@@ -1333,30 +1040,6 @@ skip_block:
 	return;
 }
 
-void
-rs_set_wb_from_pixels(RS_BLOB *rs, gint x, gint y)
-{
-	gint row, col;
-	gushort *pixel;
-	gdouble r=0.0, g=0.0, b=0.0;
-
-	for(row=-1; row<2; row++)
-	{
-		for(col=-1; col<2; col++)
-		{
-			pixel = rs_image16_get_pixel(rs->photo->scaled, x+col, y+row, TRUE);
-			r += pixel[R]/65535.0;
-			g += pixel[G]/65535.0;
-			b += pixel[B]/65535.0;
-		}
-	}
-	r /= 9;
-	g /= 9;
-	b /= 9;
-	rs_set_wb_from_color(rs, r, g, b);
-	return;
-}
-
 void
 rs_set_wb_from_color(RS_BLOB *rs, gdouble r, gdouble g, gdouble b)
 {
@@ -1402,15 +1085,6 @@ rs_set_wb(RS_BLOB *rs, gfloat warmth, gfloat tint)
 	return;
 }
 
-void
-rs_render_pixel_to_srgb(RS_BLOB *rs, gint x, gint y, guchar *dest)
-{
-	gushort *pixel;
-	pixel = rs_image16_get_pixel(rs->photo->scaled, x, y, TRUE);
-	rs_render_pixel(&rs->photo->mat, rs->photo->pre_mul, rs->previewtable8, rs->previewtable16, pixel, dest, rs_cms_get_transform(rs->cms, TRANSFORM_SRGB));
-	return;
-}
-
 void
 rs_apply_settings_from_double(RS_SETTINGS *rss, RS_SETTINGS_DOUBLE *rsd, gint mask)
 {
@@ -1580,95 +1254,6 @@ rs_rect_rotate(RS_RECT *in, RS_RECT *out, gint w, gint h, gint quarterturns)
 	return;
 }
 
-void
-rs_roi_orientation(RS_BLOB *rs)
-{
-	gint x1,x2,y1,y2;
-	const gint rot = (rs->photo->orientation&3)%4;
-	x1 = rs->roi.x1;
-	y1 = rs->roi.y1;
-	x2 = rs->roi.x2;
-	y2 = rs->roi.y2;
-
-	/* rotate and flip */
-	switch(rot)
-	{
-		case 1:
-			x1 = rs->roi.y1;
-			y1 = rs->photo->input->h - rs->roi.x1;
-			x2 = rs->roi.y2;
-			y2 = rs->photo->input->h - rs->roi.x2;
-			break;
-		case 2:
-			x1 = rs->photo->input->w - rs->roi.x2;
-			y1 = rs->photo->input->h - rs->roi.y2;
-			x2 = rs->photo->input->w - rs->roi.x1;
-			y2 = rs->photo->input->h - rs->roi.y1;
-			break;
-		case 3:
-			x1 = rs->photo->input->w - rs->roi.y1;
-			y1 = rs->roi.x1;
-			x2 = rs->photo->input->w - rs->roi.y2;
-			y2 = rs->roi.x2;
-			break;
-	}
-
-	if (rs->photo->orientation&4)
-	{
-		y1 = rs->photo->input->h-y1;
-		y2 = rs->photo->input->h-y2;
-	}
-
-	rs->roi.x1 = x1;
-	rs->roi.y1 = y1;
-	rs->roi.x2 = x2;
-	rs->roi.y2 = y2;
-
-	/* normalize */
-	if (rs->roi.x1 > rs->roi.x2)
-		SWAP(rs->roi.x1, rs->roi.x2);
-	if (rs->roi.y1 > rs->roi.y2)
-		SWAP(rs->roi.y1, rs->roi.y2);
-	return;
-}
-
-static gboolean
-rs_mark_roi_configure (GtkWidget *widget, GdkEventExpose *event, RS_BLOB *rs)
-{
-	if (rs->preview_backing_notroi)
-		g_object_unref(rs->preview_backing_notroi);
-	rs->preview_backing_notroi = gdk_pixmap_new(rs->preview_drawingarea->window,
-		rs->preview_drawingarea->allocation.width,
-		rs->preview_drawingarea->allocation.height, -1);
-	return(FALSE);
-}
-
-void
-rs_mark_roi(RS_BLOB *rs, gboolean mark)
-{
-	static gint configure;
-
-	if (mark && (!rs->mark_roi))
-	{
-		if (rs->preview_backing_notroi)
-			g_object_unref(rs->preview_backing_notroi);
-		rs->preview_backing_notroi = gdk_pixmap_new(rs->preview_drawingarea->window,
-			rs->preview_drawingarea->allocation.width,
-			rs->preview_drawingarea->allocation.height, -1);
-		rs->mark_roi = TRUE;
-		configure = g_signal_connect (GTK_OBJECT (rs->preview_drawingarea), "configure-event",
-			GTK_SIGNAL_FUNC (rs_mark_roi_configure), rs);
-	}
-	else if (rs->mark_roi)
-	{
-		g_signal_handler_disconnect(rs->preview_drawingarea, configure);
-		if (rs->preview_backing_notroi)
-			g_object_unref(rs->preview_backing_notroi);
-		rs->preview_backing_notroi = NULL;
-		rs->mark_roi = FALSE;
-	}
-}
-
 void
 check_install()
 {
@@ -1692,12 +1277,27 @@ int
 main(int argc, char **argv)
 {
 	RS_BLOB *rs;
+	int optimized = 1;
+	int opt;
+
+	while ((opt = getopt(argc, argv, "n")) != -1) {
+		switch (opt) {
+		case 'n':
+			optimized = 0;
+			break;
+		}
+	}
+
+	g_thread_init(NULL);
+	gdk_threads_init();
 
 	/* Bind default C functions */
 	rs_bind_default_functions();
 
 	/* Bind optimized functions if any */
-	rs_bind_optimized_functions();
+	if (likely(optimized)) {
+		rs_bind_optimized_functions();
+	}
 
 #ifdef ENABLE_NLS
 	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
@@ -1708,23 +1308,19 @@ main(int argc, char **argv)
 	gtk_init(&argc, &argv);
 	check_install();
 
-	// initialize overlay icons for thumbnails.
-	icon_priority_1	= gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority1.png", NULL);
-	icon_priority_2 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority2.png", NULL);
-	icon_priority_3 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority3.png", NULL);
-	icon_priority_D = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_deleted.png", NULL);
-	icon_exported = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_exported.png", NULL);
-
 	rs = rs_new();
 	rs->queue->cms = rs->cms = rs_cms_init();
 	gui_init(argc, argv, rs);
-	return(0);
+
+	/* This is so fucking evil, but Rawstudio will deadlock in some GTK atexit() function from time to time :-/ */
+	_exit(0);
 }
 
 gboolean
 rs_shutdown(GtkWidget *dummy1, GdkEvent *dummy2, RS_BLOB *rs)
 {
 	rs_photo_close(rs->photo);
+	rs_conf_set_integer(CONF_LAST_PRIORITY_PAGE, gtk_notebook_get_current_page(GTK_NOTEBOOK(rs->store)));
 	gtk_main_quit();
 	return(TRUE);
 }
