@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -25,7 +25,13 @@
 #include "dcraw_api.h"
 #include "rs-arch.h"
 #include "rs-cms.h"
-#include "rs-curve.h"
+
+#define BUH printf("%s:%d\n", __FILE__, __LINE__);
+
+/* Check for thread support */
+#if (!defined(G_THREADS_ENABLED) || defined(G_THREADS_IMPL_NONE))
+#error GLib was not compiled with thread support, Rawstudio needs threads - sorry.
+#endif
 
 #define PITCH(width) ((((width)+15)/16)*16)
 
@@ -65,6 +71,7 @@ enum {
 	MAKE_NIKON,
 	MAKE_MINOLTA,
 	MAKE_PANASONIC,
+	MAKE_PENTAX,
 };
 
 enum {
@@ -103,9 +110,11 @@ enum {
 typedef struct _RSStore RSStore;
 
 /* Opaque definition, declared in rs-batch.h */
-typedef struct _RS_QUEUE_ELEMENT RS_QUEUE_ELEMENT;
 typedef struct _RS_QUEUE RS_QUEUE;
 
+/* Defined in rs-color-transform.c */
+typedef struct _RS_COLOR_TRANSFORM RS_COLOR_TRANSFORM;
+
 typedef struct {double coeff[3][3]; } RS_MATRIX3;
 typedef struct {int coeff[3][3]; } RS_MATRIX3Int;
 typedef struct {double coeff[4][4]; } RS_MATRIX4;
@@ -119,10 +128,11 @@ typedef struct {
 	guint pixelsize; /* the size of a pixel in CHARS */
 	guint orientation;
 	guchar *pixels;
-	guint *pixels4;
+	gint reference_count;
 } RS_IMAGE8;
 
 typedef struct _rs_image16 {
+	GObject parent;
 	gint w;
 	gint h;
 	gint pitch;
@@ -131,6 +141,10 @@ typedef struct _rs_image16 {
 	guint pixelsize; /* the size of a pixel in SHORTS */
 	guint orientation;
 	gushort *pixels;
+	guint filters;
+	guint fourColorFilters;
+	gboolean preview;
+	gboolean dispose_has_run;
 } RS_IMAGE16;
 
 typedef struct {
@@ -148,7 +162,6 @@ typedef struct {
 	GtkObject *warmth;
 	GtkObject *tint;
 	GtkWidget *curve;
-	gfloat curve_samples[65536];
 } RS_SETTINGS;
 
 typedef struct {
@@ -158,7 +171,6 @@ typedef struct {
 	gdouble contrast;
 	gdouble warmth;
 	gdouble tint;
-	gfloat *curve_samples;
 	guint curve_nknots;
 	gfloat *curve_knots;
 } RS_SETTINGS_DOUBLE;
@@ -190,21 +202,13 @@ typedef struct _metadata {
 } RS_METADATA;
 
 typedef struct _photo {
-	gboolean active;
 	gchar *filename;
 	RS_IMAGE16 *input;
-	RS_IMAGE16 *scaled;
-	RS_IMAGE8 *preview;
 	RS_SETTINGS_DOUBLE *settings[3];
-	gint current_setting;
 	gint priority;
 	guint orientation;
 	RS_METADATA *metadata;
-	RS_MATRIX4 mat;
-	gfloat pre_mul[4];
 	RS_RECT *crop;
-	RS_MATRIX3 affine;
-	RS_MATRIX3 inverse_affine;
 	gdouble angle;
 	gboolean exported;
 } RS_PHOTO;
@@ -212,35 +216,16 @@ typedef struct _photo {
 typedef struct {
 	gboolean in_use;
 	RS_PHOTO *photo;
-	RS_RECT roi;
-	RS_RECT roi_scaled;
-	gint roi_grid;
 	RS_SETTINGS_DOUBLE *settings_buffer;
 	RS_SETTINGS *settings[3];
 	gint current_setting;
-	GtkObject *scale;
-	gboolean mark_roi;
-	gdouble preview_scale;
-	gboolean zoom_to_fit;
-	RS_RECT *preview_exposed;
 	RS_IMAGE16 *histogram_dataset;
-	guint histogram_table[3][256];
-	GtkImage *histogram_image;
-	GtkWidget *preview_drawingarea;
-	gint preview_width;
-	gint preview_height;
-	gboolean preview_idle_render;
-	gboolean preview_done;
-	GdkPixmap *preview_backing;
-	GdkPixmap *preview_backing_notroi;
-	gint preview_idle_render_lastrow;
-	gboolean show_exposure_overlay;
-	GArray *batch_queue;
+	GtkWidget *histogram;
+	RS_COLOR_TRANSFORM *histogram_transform;
 	RS_QUEUE *queue;
 	RS_CMS *cms;
-	guchar previewtable8[65536];
-	gushort previewtable16[65536];
 	RSStore *store;
+	GtkWidget *preview;
 } RS_BLOB;
 
 enum {
@@ -256,33 +241,25 @@ typedef struct _rs_filetype {
 	gint filetype;
 	const gchar *ext;
 	gchar *description;
-	RS_PHOTO *(*load)(const gchar *);
+	RS_PHOTO *(*load)(const gchar *, gboolean);
 	GdkPixbuf *(*thumb)(const gchar *);
 	void (*load_meta)(const gchar *, RS_METADATA *);
-	gboolean (*save)(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width, gint height, gdouble scale, RS_CMS *cms);
+	gboolean (*save)(RS_PHOTO *photo, const gchar *filename, gint filetype, gint width, gint height, gboolean keep_aspect, gdouble scale, gint snapshot, RS_CMS *cms);
 	struct _rs_filetype *next;
 } RS_FILETYPE;
 
-extern GdkPixbuf *icon_priority_1;
-extern GdkPixbuf *icon_priority_2;
-extern GdkPixbuf *icon_priority_3;
-extern GdkPixbuf *icon_priority_D;
-extern GdkPixbuf *icon_exported;
-
 void rs_local_cachedir(gboolean new_value);
 void rs_load_gdk(gboolean new_value);
-void update_preview(RS_BLOB *rs, gboolean update_table, gboolean update_scale);
-void update_preview_region(RS_BLOB *rs, RS_RECT *region, gboolean force_render);
-void rs_render_idle_stop(RS_BLOB *rs);
+void rs_update_preview(RS_BLOB *rs);
 void rs_reset(RS_BLOB *rs);
 void rs_settings_to_rs_settings_double(RS_SETTINGS *rs_settings, RS_SETTINGS_DOUBLE *rs_settings_double);
 void rs_settings_double_to_rs_settings(RS_SETTINGS_DOUBLE *rs_settings_double, RS_SETTINGS *rs_settings);
 void rs_settings_reset(RS_SETTINGS *rss, guint mask);
-inline void rs_photo_prepare(RS_PHOTO *photo);
 RS_PHOTO *rs_photo_new();
 void rs_photo_free(RS_PHOTO *photo);
 gboolean rs_photo_save(RS_PHOTO *photo, const gchar *filename, gint filetype,
-	gint width, gint height, gdouble scale, RS_CMS *cms);
+	gint width, gint height, gboolean keep_aspect, gdouble scale, gint snapshot, RS_CMS *cms);
+RS_SETTINGS_DOUBLE *rs_settings_double_new(void);
 void rs_settings_double_copy(RS_SETTINGS_DOUBLE *in, RS_SETTINGS_DOUBLE *out, gint mask);
 RS_METADATA *rs_metadata_new();
 void rs_metadata_free(RS_METADATA *metadata);
@@ -290,15 +267,17 @@ void rs_metadata_normalize_wb(RS_METADATA *meta);
 void rs_photo_flip(RS_PHOTO *photo);
 void rs_photo_mirror(RS_PHOTO *photo);
 void rs_photo_rotate(RS_PHOTO *photo, gint quarterturns, gdouble angle);
+void rs_photo_set_crop(RS_PHOTO *photo, RS_RECT *crop);
+RS_RECT *rs_photo_get_crop(RS_PHOTO *photo);
 RS_BLOB *rs_new();
 void rs_free(RS_BLOB *rs);
-void rs_zoom_to_fit(RS_BLOB *rs);
 void rs_photo_close(RS_PHOTO *photo);
 RS_FILETYPE *rs_filetype_get(const gchar *filename, gboolean load);
+gchar *rs_confdir_get();
 gchar *rs_dotdir_get(const gchar *filename);
 gchar *rs_thumb_get_name(const gchar *src);
+void rs_white_black_point(RS_BLOB *rs);
 void rs_set_wb_auto(RS_BLOB *rs);
-void rs_set_wb_from_pixels(RS_BLOB *rs, gint x, gint y);
 void rs_set_wb_from_color(RS_BLOB *rs, gdouble r, gdouble g, gdouble b);
 void rs_set_wb_from_mul(RS_BLOB *rs, gdouble *mul);
 void rs_set_wb(RS_BLOB *rs, gfloat warmth, gfloat tint);
@@ -307,8 +286,6 @@ void rs_apply_settings_from_double(RS_SETTINGS *rss, RS_SETTINGS_DOUBLE *rsd, gi
 void rs_rect_scale(RS_RECT *in, RS_RECT *out, gdouble scale);
 void rs_rect_union(RS_RECT *a, RS_RECT *b, RS_RECT *destination);
 void rs_rect_normalize(RS_RECT *in, RS_RECT *out);
-void rs_roi_orientation(RS_BLOB *rs);
-void rs_mark_roi(RS_BLOB *rs, gboolean mark);
 gboolean rs_shutdown(GtkWidget *dummy1, GdkEvent *dummy2, RS_BLOB *rs);
 #if !GLIB_CHECK_VERSION(2,8,0)
 int g_mkdir_with_parents (const gchar *pathname, int mode);
