@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -21,6 +21,8 @@
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <config.h>
+#include <libxml/encoding.h>
+#include <libxml/xmlwriter.h>
 #include "rawstudio.h"
 #include "rs-batch.h"
 #include "conf_interface.h"
@@ -28,26 +30,128 @@
 #include "gtk-helper.h"
 #include "filename.h"
 #include "rs-cache.h"
-#include "rs-render.h"
+#include "rs-color-transform.h"
 #include "rs-image.h"
-#include "rs-curve.h"
 
 extern GtkWindow *rawstudio_window;
 
 static gboolean batch_exists_in_queue(RS_QUEUE *queue, const gchar *filename, gint setting_id);
 static GtkWidget *make_batchview(RS_QUEUE *queue);
-static GtkSpinButton *size_spin;
-static GtkWidget *size_label;
+static void size_update_infolabel(RS_QUEUE *queue);
+static gchar *batch_queue_filename = NULL;
+
+static void
+batch_queue_save(RS_QUEUE *queue)
+{
+	xmlTextWriterPtr writer;
+	GtkTreeIter iter;
+	gchar *filename;
+	gint setting_id;
+
+	g_assert(queue != NULL);
+	g_assert(batch_queue_filename != NULL);
+
+	writer = xmlNewTextWriterFilename(batch_queue_filename, 0);
+	if (!writer)
+		return;
+	xmlTextWriterSetIndent(writer, 1);
+	xmlTextWriterStartDocument(writer, NULL, "ISO-8859-1", NULL);
+	xmlTextWriterStartElement(writer, BAD_CAST "rawstudio-batch-queue");
+
+	if (gtk_tree_model_get_iter_first(queue->list, &iter))
+		do
+		{
+			gtk_tree_model_get(queue->list, &iter,
+				RS_QUEUE_ELEMENT_FILENAME, &filename,
+				RS_QUEUE_ELEMENT_SETTING_ID, &setting_id,
+				-1);
+			xmlTextWriterStartElement(writer, BAD_CAST "entry");
+				xmlTextWriterWriteFormatElement(writer, BAD_CAST "filename", "%s", filename);
+				xmlTextWriterWriteFormatElement(writer, BAD_CAST "snapshot", "%d", setting_id);
+			xmlTextWriterEndElement(writer);
+		} while(gtk_tree_model_iter_next(queue->list, &iter));
+
+	xmlTextWriterEndDocument(writer);
+	xmlFreeTextWriter(writer);
+
+	return;
+}
+
+static void
+batch_queue_load(RS_QUEUE *queue)
+{
+	xmlDocPtr doc;
+	xmlNodePtr cur;
+	xmlNodePtr entry = NULL;
+	xmlChar *val;
+
+	g_assert(queue != NULL);
+
+	if (!batch_queue_filename)
+	{
+		const gchar *confdir = rs_confdir_get();
+		GString *gs = g_string_new(confdir);
+		g_string_append(gs, "batch-queue.xml");
+		batch_queue_filename = gs->str;
+		g_string_free(gs, FALSE);
+	}
+
+	if (!g_file_test(batch_queue_filename, G_FILE_TEST_IS_REGULAR))
+		return;
+
+	doc = xmlParseFile(batch_queue_filename);
+	if (!doc)
+		return;
+
+	cur = xmlDocGetRootElement(doc);
+	cur = cur->xmlChildrenNode;
+
+	while(cur)
+	{
+		if ((!xmlStrcmp(cur->name, BAD_CAST "entry")))
+		{
+			xmlChar *filename = NULL;
+			gint setting_id = -1;
+
+			entry = cur->xmlChildrenNode;
+
+			while (entry)
+			{
+				if ((!xmlStrcmp(entry->name, BAD_CAST "filename")))
+				{
+					filename = xmlNodeListGetString(doc, entry->xmlChildrenNode, 1);
+				}
+				if ((!xmlStrcmp(entry->name, BAD_CAST "snapshot")))
+				{
+					val = xmlNodeListGetString(doc, entry->xmlChildrenNode, 1);
+					setting_id = atoi((char *)val);
+					if (setting_id > 2) setting_id = 2;
+					if (setting_id < 0) setting_id = 0;
+					xmlFree(val);
+				}
+				entry = entry->next;
+			}
+			if (filename && (setting_id >= 0))
+			{
+				rs_batch_add_to_queue(queue, (gchar *) filename, setting_id);
+				xmlFree(filename);
+			}
+		}
+		cur = cur->next;
+	}
+
+	xmlFreeDoc(doc);
+	return;
+}
 
 RS_QUEUE* rs_batch_new_queue(void)
 {
+	gchar *tmp;
 	RS_QUEUE *queue = g_new(RS_QUEUE, 1);
 	RS_FILETYPE *filetype;
 
-	queue->list = GTK_TREE_MODEL(gtk_list_store_new(6, G_TYPE_STRING,G_TYPE_STRING,
-									G_TYPE_INT,G_TYPE_STRING,
-									G_TYPE_POINTER, GDK_TYPE_PIXBUF));
-
+	queue->list = GTK_TREE_MODEL(gtk_list_store_new(5, G_TYPE_STRING,G_TYPE_STRING,
+									G_TYPE_INT,G_TYPE_STRING, GDK_TYPE_PIXBUF));
 
 	queue->directory = rs_conf_get_string(CONF_BATCH_DIRECTORY);
 	if (queue->directory == NULL)
@@ -67,22 +171,43 @@ RS_QUEUE* rs_batch_new_queue(void)
 	queue->filetype = filetype->filetype;
 	queue->size_lock = LOCK_SCALE;
 	queue->size = 100;
+	queue->size_window = NULL;
+	queue->scale = 100;
+	queue->width = 600;
+	queue->height = 600;
+
+	/* Load last values */
+	rs_conf_get_integer(CONF_BATCH_SIZE_SCALE, &queue->scale);
+	rs_conf_get_integer(CONF_BATCH_SIZE_WIDTH, &queue->width);
+	rs_conf_get_integer(CONF_BATCH_SIZE_HEIGHT, &queue->height);
+	tmp = rs_conf_get_string(CONF_BATCH_SIZE_LOCK);
+	if (tmp)
+	{
+		if (g_str_equal(tmp, "bounding-box"))
+			queue->size_lock = LOCK_BOUNDING_BOX;
+		else if (g_str_equal(tmp, "width"))
+			queue->size_lock = LOCK_WIDTH;
+		else if (g_str_equal(tmp, "height"))
+			queue->size_lock = LOCK_HEIGHT;
+		g_free(tmp);
+	}
 
 	return queue;
 }
 
 gboolean
-rs_batch_add_element_to_queue(RS_QUEUE *queue, RS_QUEUE_ELEMENT *element)
+rs_batch_add_to_queue(RS_QUEUE *queue, const gchar *filename, const gint setting_id)
 {
-	if (!batch_exists_in_queue(queue, element->filename, element->setting_id))
+	gboolean ret = FALSE;
+	if (!batch_exists_in_queue(queue, filename, setting_id))
 	{
 		gchar *filename_short, *setting_id_abc;
 		RS_FILETYPE *filetype;
 		GdkPixbuf *pixbuf = NULL, *missing_thumb, *pixbuf_temp;
 
-		filename_short = g_path_get_basename(element->filename);
+		filename_short = g_path_get_basename(filename);
 
-		switch(element->setting_id)
+		switch(setting_id)
 		{
 			case 0:
 				setting_id_abc = _("A");
@@ -94,17 +219,17 @@ rs_batch_add_element_to_queue(RS_QUEUE *queue, RS_QUEUE_ELEMENT *element)
 				setting_id_abc = _("C");
 				break;
 			default:
-				return FALSE;
+				return ret;
 		}
 
-		filetype = rs_filetype_get(element->filename, TRUE);
+		filetype = rs_filetype_get(filename, TRUE);
 		if (filetype)
 		{
 			missing_thumb = gtk_widget_render_icon(GTK_WIDGET(rawstudio_window),
 				GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG, NULL);
 
 			if (filetype->thumb)
-				pixbuf = filetype->thumb(element->filename);
+				pixbuf = filetype->thumb(filename);
 			if (pixbuf)
 			{
 				gint w,h,temp,size = 48;
@@ -134,83 +259,31 @@ rs_batch_add_element_to_queue(RS_QUEUE *queue, RS_QUEUE_ELEMENT *element)
 			}
 		}
 
-		rs_batch_add_to_queue(queue, element->filename, filename_short, element->setting_id, setting_id_abc, element, pixbuf);
-
-		g_free(element);
-		return TRUE;
-	}
-	else
-	{
-		g_free(element);
-		return FALSE;
-	}
-}
-
-gboolean
-rs_batch_remove_element_from_queue(RS_QUEUE *queue, RS_QUEUE_ELEMENT *element)
-{
-	return rs_batch_remove_from_queue(queue, element->filename, element->setting_id);
-	g_free(element);
-}
-
-RS_QUEUE_ELEMENT*
-rs_batch_get_first_element_in_queue(RS_QUEUE *queue)
-{
-	GtkTreeIter iter;
-	gchar *filename_temp;
-	gint setting_id_temp;
-
-	if (gtk_tree_model_get_iter_first(queue->list, &iter))
-	{	
-		if (gtk_list_store_iter_is_valid(GTK_LIST_STORE(queue->list), &iter)) 
+		if (!batch_exists_in_queue(queue, filename, setting_id))
 		{
-			RS_QUEUE_ELEMENT *element = g_new(RS_QUEUE_ELEMENT,1);
+			GtkTreeIter iter;
 
-			gtk_tree_model_get(queue->list, &iter,
-					RS_QUEUE_ELEMENT_FILENAME, &filename_temp,
-					RS_QUEUE_ELEMENT_SETTING_ID, &setting_id_temp,
-					-1);
-
-			element->filename = filename_temp;
-			element->setting_id = setting_id_temp;
-
-			return element;
+			gtk_list_store_append (GTK_LIST_STORE(queue->list), &iter);
+ 			gtk_list_store_set (GTK_LIST_STORE(queue->list), &iter,
+				RS_QUEUE_ELEMENT_FILENAME, filename,
+				RS_QUEUE_ELEMENT_FILENAME_SHORT, filename_short,
+				RS_QUEUE_ELEMENT_SETTING_ID, setting_id,
+				RS_QUEUE_ELEMENT_SETTING_ID_ABC, setting_id_abc,
+				RS_QUEUE_ELEMENT_THUMBNAIL, pixbuf,
+				-1);
+			ret = TRUE;
 		}
-		else
-			return NULL;
 	}
-	else
-		return NULL;
-}
 
-gboolean
-rs_batch_add_to_queue(RS_QUEUE *queue, const gchar *filename, 
-						const gchar *filename_short, gint setting_id, 
-						const gchar *setting_id_abc, RS_QUEUE_ELEMENT *element,
-						GdkPixbuf *thumbnail)
-{
-	if (!batch_exists_in_queue(queue, filename, setting_id))
-	{
-		GtkTreeIter iter;
-		
-		gtk_list_store_append (GTK_LIST_STORE(queue->list), &iter);
- 		gtk_list_store_set (GTK_LIST_STORE(queue->list), &iter,
- 					RS_QUEUE_ELEMENT_FILENAME, filename,
-					RS_QUEUE_ELEMENT_FILENAME_SHORT, filename_short,
- 					RS_QUEUE_ELEMENT_SETTING_ID, setting_id,
-					RS_QUEUE_ELEMENT_SETTING_ID_ABC, setting_id_abc,
-					RS_QUEUE_ELEMENT_ELEMENT, element,
-					RS_QUEUE_ELEMENT_THUMBNAIL, thumbnail,
- 					-1);
-		return TRUE;
-	}
-	else
-		return FALSE;
+	batch_queue_save(queue);
+
+	return ret;
 }
 
 gboolean
 rs_batch_remove_from_queue(RS_QUEUE *queue, const gchar *filename, gint setting_id)
 {
+	gboolean ret = FALSE;
 	GtkTreeIter iter;
 
 	gchar *filename_temp = "init";
@@ -232,14 +305,15 @@ rs_batch_remove_from_queue(RS_QUEUE *queue, const gchar *filename, gint setting_
 				if (setting_id == setting_id_temp)
 				{
 					gtk_list_store_remove(GTK_LIST_STORE(queue->list), &iter);
-					return TRUE;
+					ret = TRUE;
 				}
 			}
 		} while (gtk_tree_model_iter_next(queue->list, &iter));
-		return FALSE;
 	}
-	else
-		return FALSE;
+
+	batch_queue_save(queue);
+
+	return ret;
 }
 
 static gboolean
@@ -292,10 +366,12 @@ cancel_clicked(GtkButton *button, gpointer user_data)
 void
 rs_batch_process(RS_QUEUE *queue)
 {
-	RS_QUEUE_ELEMENT *e;
 	RS_PHOTO *photo = NULL;
 	RS_FILETYPE *filetype;
 	RS_IMAGE16 *image;
+	GtkTreeIter iter;
+	gchar *filename_in;
+	gint setting_id;
 	GtkWidget *preview = gtk_image_new();
 	GdkPixbuf *pixbuf = NULL;
 	gint width = -1, height = -1;
@@ -308,11 +384,26 @@ rs_batch_process(RS_QUEUE *queue)
 	GtkWidget *vbox = gtk_vbox_new(FALSE, 4);
 	GtkWidget *cancel;
 	gboolean abort_render = FALSE;
-	guchar table8[65536];
-	gushort table16[65536];
-	RSCurveWidget *curve = NULL;
-	gint i;
 	gboolean fullscreen = FALSE;
+	RS_COLOR_TRANSFORM *rct = rs_color_transform_new();
+
+	/* Initialize dimensions */
+	switch (queue->size_lock)
+	{
+		case LOCK_SCALE:
+			scale = queue->scale/100.0;
+			break;
+		case LOCK_WIDTH:
+			width = queue->width;
+			break;
+		case LOCK_HEIGHT:
+			height = queue->height;
+			break;
+		case LOCK_BOUNDING_BOX:
+			width = queue->width;
+			height = queue->height;
+			break;
+	}
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_transient_for(GTK_WINDOW(window), rawstudio_window);
@@ -340,23 +431,26 @@ rs_batch_process(RS_QUEUE *queue)
 	while (gtk_events_pending()) gtk_main_iteration();
 
 	g_mkdir_with_parents(queue->directory, 00755);
-	while((e = rs_batch_get_first_element_in_queue(queue)) && (!abort_render))
+	while(gtk_tree_model_get_iter_first(queue->list, &iter) && (!abort_render))
 	{
-		if ((filetype = rs_filetype_get(e->filename, TRUE)))
+		gtk_tree_model_get(queue->list, &iter,
+			RS_QUEUE_ELEMENT_FILENAME, &filename_in,
+			RS_QUEUE_ELEMENT_SETTING_ID, &setting_id,
+			-1);
+		if ((filetype = rs_filetype_get(filename_in, TRUE)))
 		{
-			basename = g_path_get_basename(e->filename);
+			basename = g_path_get_basename(filename_in);
 			g_string_printf(status, _("Loading %s ..."), basename);
 			gtk_label_set_text(GTK_LABEL(label), status->str);
 			while (gtk_events_pending()) gtk_main_iteration();
 			g_free(basename);
 
-			photo = filetype->load(e->filename);
+			photo = filetype->load(filename_in, FALSE);
+			rs_image16_demosaic(photo->input, RS_DEMOSAIC_PPG);
 			if (photo)
 			{
-				gint current_setting;
-				
 				if (filetype->load_meta)
-					filetype->load_meta(e->filename, photo->metadata);
+					filetype->load_meta(filename_in, photo->metadata);
 				filename = g_string_new(queue->directory);
 				g_string_append(filename, G_DIR_SEPARATOR_S);
 				g_string_append(filename, queue->filename);
@@ -378,25 +472,9 @@ rs_batch_process(RS_QUEUE *queue)
 						break;
 				}
 
-				parsed_filename = filename_parse(filename->str, photo);
 				rs_cache_load(photo);
 
-				current_setting = photo->current_setting;
-				photo->current_setting = e->setting_id;
-				rs_photo_prepare(photo);
-
-				switch (queue->size_lock)
-				{
-					case LOCK_SCALE:
-						scale = gtk_spin_button_get_value(size_spin)/100.0;
-						break;
-					case LOCK_WIDTH:
-						width = (gint) gtk_spin_button_get_value(size_spin);
-						break;
-					case LOCK_HEIGHT:
-						height = (gint) gtk_spin_button_get_value(size_spin);
-						break;
-				}
+				parsed_filename = filename_parse(filename->str, filename_in, setting_id);
 
 				image = rs_image16_transform(photo->input, NULL,
 					NULL, NULL, photo->crop, 200, 200, TRUE, -1.0,
@@ -404,30 +482,11 @@ rs_batch_process(RS_QUEUE *queue)
 				if (pixbuf) g_object_unref(pixbuf);
 				pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, image->w, image->h);
 
-				if (!curve)
-					curve = RS_CURVE_WIDGET(rs_curve_widget_new()); /* FIXME: free this at some point */
-				rs_curve_widget_reset(curve);
-				if (photo->settings[photo->current_setting]->curve_nknots < 2)
-				{
-					rs_curve_widget_add_knot(curve, 0.0f, 0.0f);
-					rs_curve_widget_add_knot(curve, 1.0f, 1.0f);
-				}
-				else for (i=0;i<photo->settings[photo->current_setting]->curve_nknots;i++)
-				{
-					rs_curve_widget_add_knot(curve,
-						photo->settings[photo->current_setting]->curve_knots[i*2+0],
-						photo->settings[photo->current_setting]->curve_knots[i*2+1]);
-				}
-				photo->settings[photo->current_setting]->curve_samples = g_new(gfloat, 65536);
-				rs_curve_widget_sample(curve, photo->settings[photo->current_setting]->curve_samples, 65536);
-
-				rs_render_previewtable(photo->settings[photo->current_setting]->contrast,
-					photo->settings[photo->current_setting]->curve_samples, table8, table16);
-				
-				rs_render(&photo->mat, photo->pre_mul, table8, table16, image->w, image->h, image->pixels,
+				/* Render preview image */
+				rs_color_transform_set_from_settings(rct, photo->settings[setting_id], MASK_ALL);
+				rct->transform(rct, image->w, image->h, image->pixels,
 					image->rowstride, gdk_pixbuf_get_pixels(pixbuf),
-					gdk_pixbuf_get_rowstride(pixbuf),
-					rs_cms_get_transform(queue->cms, TRANSFORM_DISPLAY));
+					gdk_pixbuf_get_rowstride(pixbuf));
 				gtk_image_set_from_pixbuf((GtkImage *) preview, pixbuf);
 				rs_image16_free(image);
 
@@ -438,17 +497,17 @@ rs_batch_process(RS_QUEUE *queue)
 				g_free(basename);
 
 				rs_photo_save(photo, parsed_filename, queue->filetype,
-					width, height, scale, queue->cms);
+					width, height, TRUE, scale, setting_id, queue->cms);
 				g_free(parsed_filename);
 				g_string_free(filename, TRUE);
-				g_free(photo->settings[photo->current_setting]->curve_samples);
-				photo->current_setting = current_setting;
 				rs_photo_free(photo);
 			}
 			photo = NULL;
 		}
-		rs_batch_remove_element_from_queue(queue, e);
+		gtk_list_store_remove(GTK_LIST_STORE(queue->list), &iter);
+		batch_queue_save(queue);
 	}
+	rs_color_transform_free(rct);
 	gtk_widget_destroy(window);
 
 	/* Restore fullscreen state if needed */
@@ -527,7 +586,10 @@ batch_button_remove_clicked(GtkWidget *button, RS_QUEUE *queue)
 		GtkTreeIter iter;
 
 		if(gtk_tree_model_get_iter(queue->list,&iter,path))
+		{
 			gtk_list_store_remove(GTK_LIST_STORE(queue->list), &iter);
+			batch_queue_save(queue);
+		}
 	}
 	return;
 }
@@ -536,6 +598,7 @@ static void
 batch_button_remove_all_clicked(GtkWidget *button, RS_QUEUE *queue)
 {
 	gtk_list_store_clear(GTK_LIST_STORE(queue->list));
+	batch_queue_save(queue);
 	return;
 }
 
@@ -574,59 +637,6 @@ make_batchbuttons(RS_QUEUE *queue)
 		return box;
 }
 
-static void
-lockbox_changed(gpointer selected, gpointer user_data)
-{
-	gdouble tmp;
-	RS_QUEUE *queue = (RS_QUEUE *) user_data;
-	queue->size_lock = GPOINTER_TO_INT(selected);
-
-	switch (queue->size_lock)
-	{
-		case LOCK_SCALE:
-			gtk_spin_button_set_range(size_spin, 10.0, 100.0);
-			if (rs_conf_get_double(CONF_BATCH_SIZE_SCALE, &tmp))
-				gtk_spin_button_set_value(size_spin, tmp);
-			break;
-		case LOCK_WIDTH:
-			gtk_spin_button_set_range(size_spin, 10.0, 65535.0);
-			if (rs_conf_get_double(CONF_BATCH_SIZE_WIDTH, &tmp))
-				gtk_spin_button_set_value(size_spin, tmp);
-			break;
-		case LOCK_HEIGHT:
-			gtk_spin_button_set_range(size_spin, 10.0, 65535.0);
-			if (rs_conf_get_double(CONF_BATCH_SIZE_HEIGHT, &tmp))
-				gtk_spin_button_set_value(size_spin, tmp);
-			break;
-	}
-	return;
-}
-
-static void
-size_spin_changed(GtkSpinButton *spinbutton, gpointer user_data)
-{
-	RS_QUEUE *queue = (RS_QUEUE *) user_data;
-	queue->size = gtk_spin_button_get_value(spinbutton);
-
-	switch (queue->size_lock)
-	{
-		case LOCK_SCALE:
-			rs_conf_set_double(CONF_BATCH_SIZE_SCALE, gtk_spin_button_get_value(size_spin));
-			gtk_label_set_text(GTK_LABEL(size_label), _("%"));
-			break;
-		case LOCK_WIDTH:
-			rs_conf_set_double(CONF_BATCH_SIZE_WIDTH, gtk_spin_button_get_value(size_spin));
-			gtk_label_set_text(GTK_LABEL(size_label), _("px"));
-			break;
-		case LOCK_HEIGHT:
-			rs_conf_set_double(CONF_BATCH_SIZE_HEIGHT, gtk_spin_button_get_value(size_spin));
-			gtk_label_set_text(GTK_LABEL(size_label), _("px"));
-			break;
-	}
-
-	return;
-}
-
 static void
 chooser_changed(GtkFileChooser *chooser, gpointer user_data)
 {
@@ -645,18 +655,207 @@ filetype_changed(gpointer active, gpointer user_data)
 	queue->filetype = filetype->filetype;
 }
 
+static void
+size_lockbox_changed(gpointer selected, gpointer user_data)
+{
+	RS_QUEUE *queue = (RS_QUEUE *) user_data;
+	gint i;
+	queue->size_lock = GPOINTER_TO_INT(selected);
+
+	for(i=0;i<3;i++)
+	{
+		gtk_widget_hide(queue->size_width[i]);
+		gtk_widget_hide(queue->size_height[i]);
+		gtk_widget_hide(queue->size_scale[i]);
+	}
+
+	/* Show needed spinners */
+	switch (queue->size_lock)
+	{
+		case LOCK_WIDTH:
+			for(i=0;i<3;i++)
+				gtk_widget_show(queue->size_width[i]);
+			break;
+		case LOCK_HEIGHT:
+			for(i=0;i<3;i++)
+				gtk_widget_show(queue->size_height[i]);
+			break;
+		case LOCK_SCALE:
+			for(i=0;i<3;i++)
+				gtk_widget_show(queue->size_scale[i]);
+			break;
+		case LOCK_BOUNDING_BOX:
+			for(i=0;i<3;i++)
+			{
+				gtk_widget_show(queue->size_width[i]);
+				gtk_widget_show(queue->size_height[i]);
+			}
+			break;
+	}
+
+	size_update_infolabel(queue);
+
+	return;
+}
+
+static void
+size_width_changed(GtkSpinButton *spinbutton, RS_QUEUE *queue)
+{
+	queue->width = gtk_spin_button_get_value_as_int(spinbutton);
+	size_update_infolabel(queue);
+	rs_conf_set_integer(CONF_BATCH_SIZE_WIDTH, queue->width);
+}
+
+static void
+size_height_changed(GtkSpinButton *spinbutton, RS_QUEUE *queue)
+{
+	queue->height = gtk_spin_button_get_value_as_int(spinbutton);
+	size_update_infolabel(queue);
+	rs_conf_set_integer(CONF_BATCH_SIZE_HEIGHT, queue->height);
+}
+
+static void
+size_scale_changed(GtkSpinButton *spinbutton, RS_QUEUE *queue)
+{
+	queue->scale = gtk_spin_button_get_value_as_int(spinbutton);
+	size_update_infolabel(queue);
+	rs_conf_set_integer(CONF_BATCH_SIZE_SCALE, queue->scale);
+}
+
+static void
+size_close_clicked(GtkButton *button, RS_QUEUE *queue)
+{
+	gtk_widget_hide(queue->size_window);
+}
+
+static void
+batch_size_selection(GtkWidget *button, RS_QUEUE *queue)
+{
+	RS_CONFBOX *lockbox;
+	GtkWidget *vbox = gtk_vbox_new(FALSE, 4);
+	GtkWidget *table;
+	GtkWidget *close;
+
+	/* Only open one at a time */
+	if (queue->size_window)
+	{
+		/* Leave the window at its last position */
+		gtk_window_set_position(GTK_WINDOW(queue->size_window), GTK_WIN_POS_NONE);
+		gtk_widget_show(queue->size_window);
+		return;
+	}
+
+	/* Make window */
+	queue->size_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	g_signal_connect(G_OBJECT(queue->size_window), "delete_event", G_CALLBACK(gtk_widget_hide_on_delete), queue);
+	gtk_window_set_title (GTK_WINDOW(queue->size_window), _("Photo size"));
+	gtk_window_set_position(GTK_WINDOW(queue->size_window), GTK_WIN_POS_MOUSE);
+	gtk_widget_realize (queue->size_window);
+	gdk_window_set_type_hint(queue->size_window->window, GDK_WINDOW_TYPE_HINT_UTILITY);
+	gtk_window_set_transient_for(GTK_WINDOW(queue->size_window), rawstudio_window);
+
+	/* Chooser */
+	lockbox = gui_confbox_new(CONF_BATCH_SIZE_LOCK);
+	gui_confbox_add_entry(lockbox, "scale", _("Constant scale"), GINT_TO_POINTER(LOCK_SCALE));
+	gui_confbox_add_entry(lockbox, "width", _("Constant width"), GINT_TO_POINTER(LOCK_WIDTH));
+	gui_confbox_add_entry(lockbox, "height", _("Constant height"), GINT_TO_POINTER(LOCK_HEIGHT));
+	gui_confbox_add_entry(lockbox, "bounding-box", _("Maximum size"), GINT_TO_POINTER(LOCK_BOUNDING_BOX));
+	gui_confbox_load_conf(lockbox, "scale");
+	gtk_widget_show(gui_confbox_get_widget(lockbox));
+
+	gtk_box_pack_start (GTK_BOX (vbox), gui_confbox_get_widget(lockbox), FALSE, TRUE, 0);
+
+	/* Spinners */
+	table = gtk_table_new(3, 3, FALSE);
+	gtk_table_set_col_spacings(GTK_TABLE(table), 0);
+	gtk_table_set_row_spacings(GTK_TABLE(table), 0);
+	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, TRUE, 0);
+
+	queue->size_width[0] = gtk_label_new(_("Photo width:"));
+	queue->size_width[1] = gtk_spin_button_new_with_range(10.0, 10000.0, 1.0);
+	queue->size_width[2] = gtk_label_new(_("pixels"));
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(queue->size_width[1]), (gdouble) queue->width);
+	g_signal_connect(G_OBJECT(queue->size_width[1]), "value_changed", G_CALLBACK(size_width_changed), queue);
+	gtk_table_attach_defaults(GTK_TABLE(table), queue->size_width[0], 0, 1, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(table), queue->size_width[1], 1, 2, 0, 1);
+	gtk_table_attach_defaults(GTK_TABLE(table), queue->size_width[2], 2, 3, 0, 1);
+
+	queue->size_height[0] = gtk_label_new(_("Photo height:"));
+	queue->size_height[1] = gtk_spin_button_new_with_range(10.0, 10000.0, 1.0);
+	queue->size_height[2] = gtk_label_new(_("pixels"));
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(queue->size_height[1]), (gdouble) queue->height);
+	g_signal_connect(G_OBJECT(queue->size_height[1]), "value_changed", G_CALLBACK(size_height_changed), queue);
+	gtk_table_attach_defaults(GTK_TABLE(table), queue->size_height[0], 0, 1, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(table), queue->size_height[1], 1, 2, 1, 2);
+	gtk_table_attach_defaults(GTK_TABLE(table), queue->size_height[2], 2, 3, 1, 2);
+
+	queue->size_scale[0] = gtk_label_new(_("Photo scale:"));
+	queue->size_scale[1] = gtk_spin_button_new_with_range(10.0, 10000.0, 1.0);
+	queue->size_scale[2] = gtk_label_new(_("%"));
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(queue->size_scale[1]), (gdouble) queue->scale);
+	g_signal_connect(G_OBJECT(queue->size_scale[1]), "value_changed", G_CALLBACK(size_scale_changed), queue);
+	gtk_table_attach_defaults(GTK_TABLE(table), queue->size_scale[0], 0, 1, 2, 3);
+	gtk_table_attach_defaults(GTK_TABLE(table), queue->size_scale[1], 1, 2, 2, 3);
+	gtk_table_attach_defaults(GTK_TABLE(table), queue->size_scale[2], 2, 3, 2, 3);
+
+	/* Align everything nicely */
+	gtk_misc_set_alignment(GTK_MISC(queue->size_height[0]), 1.0, 0.5);
+	gtk_misc_set_alignment(GTK_MISC(queue->size_height[2]), 0.0, 0.5);
+	gtk_misc_set_alignment(GTK_MISC(queue->size_width[0]), 1.0, 0.5);
+	gtk_misc_set_alignment(GTK_MISC(queue->size_width[2]), 0.0, 0.5);
+	gtk_misc_set_alignment(GTK_MISC(queue->size_scale[0]), 1.0, 0.5);
+	gtk_misc_set_alignment(GTK_MISC(queue->size_scale[2]), 0.0, 0.5);
+
+	/* Close button */
+	close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+	g_signal_connect (G_OBJECT(close), "clicked", G_CALLBACK (size_close_clicked), queue);
+	gtk_box_pack_end (GTK_BOX (vbox), gui_aligned(close, 1.0, 0.5, 0.0, 0.0), FALSE, TRUE, 0);
+
+	gtk_container_add (GTK_CONTAINER (queue->size_window), vbox);
+	gtk_widget_show_all(queue->size_window);
+
+	size_lockbox_changed(gui_confbox_get_active(lockbox), queue);
+	gui_confbox_set_callback(lockbox, queue, size_lockbox_changed);
+}
+
+static void
+size_update_infolabel(RS_QUEUE *queue)
+{
+	GString *gs = g_string_new("");
+
+	switch (queue->size_lock)
+	{
+		case LOCK_WIDTH:
+			g_string_printf(gs, "Constant width:\n%d", queue->width);
+			break;
+		case LOCK_HEIGHT:
+			g_string_printf(gs, "Constant height:\n%d", queue->height);
+			break;
+		case LOCK_SCALE:
+			g_string_printf(gs, "Constant Scale:\n%d", queue->scale);
+			break;
+		case LOCK_BOUNDING_BOX:
+			g_string_printf(gs, "Maximum size:\n%d x %d", queue->width, queue->height);
+			break;
+	}
+
+	gtk_label_set_justify(GTK_LABEL(queue->size_label), GTK_JUSTIFY_CENTER);
+	gtk_label_set_label(GTK_LABEL(queue->size_label), gs->str);
+
+	g_string_free(gs, TRUE);
+
+	return;
+}
+
 static GtkWidget *
 make_batch_options(RS_QUEUE *queue)
 {
-	RS_CONFBOX *lockbox = gui_confbox_new(CONF_BATCH_SIZE_LOCK);
 	GtkWidget *chooser;
 	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
 	GtkWidget *vbox = gtk_vbox_new(FALSE, 4);
 	GtkWidget *filename;
 	RS_CONFBOX *filetype_confbox;
-
-	size_spin = GTK_SPIN_BUTTON(gtk_spin_button_new_with_range(10.0, 132.0, 1.0));
-	size_label = gtk_label_new(NULL);
+	GtkWidget *size_button;
 
 	chooser = gtk_file_chooser_button_new(_("Choose output directory"),
 		GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
@@ -672,26 +871,22 @@ make_batch_options(RS_QUEUE *queue)
 	gtk_box_pack_start (GTK_BOX (vbox), gui_framed(filename,
 		_("Filename template:"), GTK_SHADOW_NONE), FALSE, FALSE, 0);
 
-	gui_confbox_set_callback(lockbox, queue, lockbox_changed);
-	gui_confbox_add_entry(lockbox, "scale", _("Set image size by scale:"), GINT_TO_POINTER(LOCK_SCALE));
-	gui_confbox_add_entry(lockbox, "width", _("Set image size by width:"), GINT_TO_POINTER(LOCK_WIDTH));
-	gui_confbox_add_entry(lockbox, "height", _("Set image size by height:"), GINT_TO_POINTER(LOCK_HEIGHT));
-	gui_confbox_load_conf(lockbox, "scale");
-
-	gtk_widget_set(GTK_WIDGET(size_spin), "receives-default", TRUE, NULL);
-	g_signal_connect(G_OBJECT(size_spin), "value_changed",
-		G_CALLBACK(size_spin_changed), queue);
-
-	gtk_box_pack_start (GTK_BOX (hbox), gui_confbox_get_widget(lockbox), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(size_spin), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), size_label, FALSE, FALSE, 0);
-
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-
 	filetype_confbox = gui_confbox_filetype_new(CONF_BATCH_FILETYPE);
 	gui_confbox_set_callback(filetype_confbox, queue, filetype_changed);
 	gtk_box_pack_start (GTK_BOX (vbox), gui_confbox_get_widget(filetype_confbox), FALSE, TRUE, 0);
 
+	/* Export size */
+	hbox = gtk_hbox_new(FALSE, 1);
+	queue->size_label = gtk_label_new(NULL);
+	size_update_infolabel(queue);
+	size_button = gtk_button_new();
+	gtk_button_set_label(GTK_BUTTON(size_button), _("Change"));
+	g_signal_connect ((gpointer) size_button, "clicked", G_CALLBACK (batch_size_selection), queue);
+	gtk_box_pack_start (GTK_BOX (hbox), queue->size_label, FALSE, FALSE, 1);
+	gtk_box_pack_end (GTK_BOX (hbox), size_button, FALSE, FALSE, 1);
+
+	gtk_box_pack_start (GTK_BOX (vbox), gui_framed(hbox, _("Export dimensions"), GTK_SHADOW_IN), FALSE, TRUE, 0);
+
 	return(vbox);
 }
 
@@ -704,6 +899,7 @@ make_batchbox(RS_QUEUE *queue)
 	gtk_box_pack_start (GTK_BOX (batchbox), make_batch_options(queue), FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (batchbox), make_batchview(queue), TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (batchbox), make_batchbuttons(queue), FALSE, FALSE, 0);
+	batch_queue_load(queue);
 
 	return batchbox;
 }
