@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -26,14 +26,9 @@ typedef enum {
 	LOCK_SCALE = 0,
 	LOCK_WIDTH,
 	LOCK_HEIGHT,
+	LOCK_BOUNDING_BOX,
 } RS_QUEUE_SIZE_LOCK;
 
-/* Opaque typedefs in rawstudio.h */
-struct _RS_QUEUE_ELEMENT {
-	const gchar *filename;
-	gint setting_id;
-};
-
 struct _RS_QUEUE {
 	GtkTreeModel *list;
 	GtkTreeView *view;
@@ -43,6 +38,14 @@ struct _RS_QUEUE {
 	RS_QUEUE_SIZE_LOCK size_lock;
 	gdouble size;
 	void *cms;
+	gint width;
+	gint height;
+	gint scale;
+	GtkWidget *size_window;
+	GtkWidget *size_label;
+	GtkWidget *size_width[3];
+	GtkWidget *size_height[3];
+	GtkWidget *size_scale[3];
 };
 
 enum {
@@ -50,15 +53,11 @@ enum {
 	RS_QUEUE_ELEMENT_FILENAME_SHORT,
 	RS_QUEUE_ELEMENT_SETTING_ID,
 	RS_QUEUE_ELEMENT_SETTING_ID_ABC,
-	RS_QUEUE_ELEMENT_ELEMENT,
 	RS_QUEUE_ELEMENT_THUMBNAIL
 };
 
 extern RS_QUEUE* rs_batch_new_queue(void);
-extern gboolean rs_batch_add_element_to_queue(RS_QUEUE *queue, RS_QUEUE_ELEMENT *element);
-extern gboolean rs_batch_remove_element_from_queue(RS_QUEUE *queue, RS_QUEUE_ELEMENT *element);
-extern RS_QUEUE_ELEMENT* rs_batch_get_first_element_in_queue(RS_QUEUE *queue);
-extern gboolean rs_batch_add_to_queue(RS_QUEUE *queue, const gchar *filename, const gchar *filename_short, gint setting_id, const gchar *setting_id_abc, RS_QUEUE_ELEMENT *element, GdkPixbuf *thumbnail);
+extern gboolean rs_batch_add_to_queue(RS_QUEUE *queue, const gchar *filename, const gint setting_id);
 extern gboolean rs_batch_remove_from_queue(RS_QUEUE *queue, const gchar *filename, gint setting_id);
 extern void rs_batch_process(RS_QUEUE *queue);
 extern GtkWidget *make_batchbox(RS_QUEUE *queue);
