@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -20,7 +20,31 @@
 #ifndef RS_IMAGE_H
 #define RS_IMAGE_H
 
+#include <glib-object.h>
+
+#define RS_TYPE_IMAGE16        (rs_image16_get_type ())
+#define RS_IMAGE16(obj)        (G_TYPE_CHECK_INSTANCE_CAST ((obj), RS_TYPE_IMAGE16, RS_IMAGE16))
+#define RS_IMAGE16_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), RS_TYPE_IMAGE16, RS_IMAGE16Class))
+#define RS_IS_IMAGE16(obj)     (G_TYPE_CHECK_INSTANCE_TYPE ((obj), RS_TYPE_IMAGE16))
+#define RS_IS_IMAGE16_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), RS_TYPE_IMAGE16))
+#define RS_IMAGE16_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), RS_TYPE_IMAGE16, RS_IMAGE16Class))
+
+typedef struct _RS_IMAGE16Class RS_IMAGE16Class;
+
+struct _RS_IMAGE16Class {
+	GObjectClass parent;
+};
+
+GType rs_image16_get_type (void);
+
+typedef enum {
+	RS_DEMOSAIC_BILINEAR,
+	RS_DEMOSAIC_PPG,
+} RS_DEMOSAIC;
+
 #define rs_image16_scale(in, out, scale) rs_image16_scale_double(in, out, scale)
+#define rs_image16_free(image) g_object_unref(image)
+#define rs_image8_free(image) rs_image8_unref(image)
 
 /**
  * Convenience macro to get a pixel at specific position
@@ -30,10 +54,13 @@
  */
 #define GET_PIXEL(image, x, y) ((image)->pixels + (y)*(image)->rowstride + (x)*(image)->pixelsize)
 
+#define rs_image16_ref(image) g_object_ref(image)
+#define rs_image16_unref(image) g_object_unref(image)
+extern void rs_image8_ref(RS_IMAGE8 *image);
+extern void rs_image8_unref(RS_IMAGE8 *image);
+
 extern RS_IMAGE16 *rs_image16_new(const guint width, const guint height, const guint channels, const guint pixelsize);
-extern void rs_image16_free(RS_IMAGE16 *rsi);
 extern RS_IMAGE8 *rs_image8_new(const guint width, const guint height, const guint channels, const guint pixelsize);
-extern void rs_image8_free(RS_IMAGE8 *rsi);
 
 /**
  * Renders a shaded image
@@ -72,6 +99,7 @@ extern RS_IMAGE16 *rs_image16_transform(RS_IMAGE16 *in, RS_IMAGE16 *out, RS_MATR
 	RS_RECT *crop, gint width, gint height, gboolean keep_aspect, gdouble scale, gdouble angle, gint orientation, gdouble *actual_scale);
 extern RS_IMAGE16 *rs_image16_scale_double(RS_IMAGE16 *in, RS_IMAGE16 *out, gdouble scale);
 extern RS_IMAGE16 *rs_image16_copy(RS_IMAGE16 *rsi);
+extern RS_IMAGE16 *rs_image16_convolve(RS_IMAGE16 *input, RS_IMAGE16 *output, RS_MATRIX3 *matrix, gfloat scaler);
 
 /**
  * Returns a single pixel from a RS_IMAGE16
@@ -83,4 +111,24 @@ extern RS_IMAGE16 *rs_image16_copy(RS_IMAGE16 *rsi);
 extern inline gushort *rs_image16_get_pixel(RS_IMAGE16 *image, gint x, gint y, gboolean extend_edges);
 extern gboolean rs_image16_8_cmp_size(RS_IMAGE16 *a, RS_IMAGE8 *b);
 
+extern size_t rs_image16_get_footprint(RS_IMAGE16 *image);
+extern RS_IMAGE16 *rs_image16_sharpen(RS_IMAGE16 *in, RS_IMAGE16 *out, gdouble amount);
+
+/**
+ * Copies an RS_IMAGE16, making it double size in the process
+ * @param in The input image
+ * @param out The output image or NULL
+ */
+extern RS_IMAGE16 *(*rs_image16_copy_double)(RS_IMAGE16 *in, RS_IMAGE16 *out);
+extern RS_IMAGE16 *rs_image16_copy_double_c(RS_IMAGE16 *in, RS_IMAGE16 *out);
+extern RS_IMAGE16 *rs_image16_copy_double_mmx(RS_IMAGE16 *in, RS_IMAGE16 *out);
+
+/**
+ * Demosaics a RS_IMAGE16
+ * @param image The image to demosaic, this MUST be preprocessed, ie. doubled in size
+ * @param demosaic The demosaic algorithm to use
+ * @return FALSE if the image was not suited for demosaic, TRUE if we succeed
+ */
+gboolean rs_image16_demosaic(RS_IMAGE16 *image, RS_DEMOSAIC demosaic);
+
 #endif
