@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -20,6 +20,8 @@
 #include <gtk/gtk.h>
 #include <glib/gprintf.h>
 #include <config.h>
+#include <libxml/encoding.h>
+#include <libxml/xmlwriter.h>
 #include "rawstudio.h"
 #include "conf_interface.h"
 #include "gettext.h"
@@ -27,10 +29,22 @@
 #include "gtk-helper.h"
 #include "gtk-progress.h"
 #include "rs-cache.h"
+#include "rs-pixbuf.h"
+#include "eog-pixbuf-cell-renderer.h"
+#include "rs-preload.h"
 
 /* How many different icon views do we have (tabs) */
 #define NUM_VIEWS 6
 
+#define GROUP_XML_FILE "groups.xml"
+
+/* Overlay icons */
+static GdkPixbuf *icon_priority_1 = NULL;
+static GdkPixbuf *icon_priority_2 = NULL;
+static GdkPixbuf *icon_priority_3 = NULL;
+static GdkPixbuf *icon_priority_D = NULL;
+static GdkPixbuf *icon_exported = NULL;
+
 enum {
 	PIXBUF_COLUMN, /* The displayed pixbuf */
 	PIXBUF_CLEAN_COLUMN, /* The clean thumbnail */
@@ -38,9 +52,17 @@ enum {
 	FULLNAME_COLUMN, /* Full path to image */
 	PRIORITY_COLUMN,
 	EXPORTED_COLUMN,
+	TYPE_COLUMN,
+	GROUP_LIST_COLUMN,
 	NUM_COLUMNS
 };
 
+enum {
+	RS_STORE_TYPE_FILE,
+	RS_STORE_TYPE_GROUP,
+	RS_STORE_TYPE_GROUP_MEMBER
+};
+
 struct _RSStore
 {
 	GtkNotebook parent;
@@ -83,6 +105,13 @@ static void icon_get_selected_iters(GtkIconView *iconview, GtkTreePath *path, gp
 static void icon_get_selected_names(GtkIconView *iconview, GtkTreePath *path, gpointer user_data);
 static gboolean tree_foreach_names(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
 static gboolean tree_find_filename(GtkTreeModel *store, const gchar *filename, GtkTreeIter *iter, GtkTreePath **path);
+void store_group_select_n(GtkListStore *store, GtkTreeIter iter, guint n);
+gboolean store_iter_is_group(GtkListStore *store, GtkTreeIter *iter);
+gint store_selection_n_groups(RSStore *store, GList *selected);
+void store_save_groups(GtkListStore *store);
+void store_load_groups(GtkListStore *store);
+void store_group_photos_by_iters(GtkListStore *store, GList *members);
+void store_group_photos_by_filenames(GtkListStore *store, GList *members);
 
 /**
  * Class initializer
@@ -105,6 +134,14 @@ rs_store_class_init(RSStoreClass *klass)
 		G_TYPE_NONE,
 		1,
 		G_TYPE_STRING);
+	if (!icon_priority_1)
+	{
+		icon_priority_1	= gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority1.png", NULL);
+		icon_priority_2 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority2.png", NULL);
+		icon_priority_3 = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_priority3.png", NULL);
+		icon_priority_D = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_deleted.png", NULL);
+		icon_exported = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/rawstudio/overlay_exported.png", NULL);
+	}
 }
 
 /**
@@ -118,6 +155,8 @@ rs_store_init(RSStore *store)
 	gchar label_text[NUM_VIEWS][63];
 	GtkWidget **label = g_new(GtkWidget *, NUM_VIEWS);
 	GtkWidget *label_tt[NUM_VIEWS];
+	GtkCellRenderer *cell_renderer;
+	gboolean show_filenames;
 
 	store->store = gtk_list_store_new (NUM_COLUMNS,
 		GDK_TYPE_PIXBUF,
@@ -125,7 +164,9 @@ rs_store_init(RSStore *store)
 		G_TYPE_STRING,
 		G_TYPE_STRING,
 		G_TYPE_INT,
-		G_TYPE_BOOLEAN);
+		G_TYPE_BOOLEAN,
+		G_TYPE_INT,
+		G_TYPE_POINTER);
 
 	for (n=0;n<NUM_VIEWS;n++)
 	{
@@ -134,6 +175,28 @@ rs_store_init(RSStore *store)
 		/* New Icon view */
 		store->iconview[n] = gtk_icon_view_new();
 
+		/* New cell-renderer for thumbnails */
+		cell_renderer = eog_pixbuf_cell_renderer_new();
+
+		/* Use our own cell renderer */
+		gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (store->iconview[n]),
+			cell_renderer, FALSE);
+
+		/* Set everything up nice */
+		g_object_set (cell_renderer,
+			"follow-state", TRUE,
+			"height", 130,  
+			"width", 130, 
+			"yalign", 0.5,
+			"xalign", 0.5,
+			NULL);
+
+		/* Set pixbuf column */
+		gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (store->iconview[n]),
+			cell_renderer,
+			"pixbuf", PIXBUF_COLUMN,
+			NULL);
+
 		/* New filter */
 		filter = gtk_tree_model_filter_new(GTK_TREE_MODEL (store->store), NULL);
 
@@ -184,6 +247,10 @@ rs_store_init(RSStore *store)
 		gtk_notebook_append_page(notebook, make_iconview(store->iconview[n], store, priorities[n]), label_tt[n]);
 	}
 
+	/* Load show filenames state from config */
+	rs_conf_get_boolean_with_default(CONF_SHOW_FILENAMES, &show_filenames, FALSE);
+	rs_store_set_show_filenames(store, show_filenames);
+
 	/* Default to page 0 */
 	store->current_iconview = store->iconview[0];
 	store->current_priority = priorities[0];
@@ -209,6 +276,79 @@ switch_page(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, gpoint
 	return;
 }
 
+static void
+preload_iter(GtkTreeModel *model, GtkTreeIter *iter)
+{
+	gchar *filename;
+	gtk_tree_model_get(model, iter, FULLNAME_COLUMN, &filename, -1);
+
+	rs_preload_near_add(filename);
+}
+
+static void
+predict_preload(RSStore *store, gboolean initial)
+{
+	GList *selected = NULL;
+	gint n, near;
+	GtkTreeIter iter;
+	GtkIconView *iconview = GTK_ICON_VIEW(store->current_iconview);
+	GtkTreePath *path, *next, *prev;
+	GtkTreeModel *model = gtk_icon_view_get_model (iconview);
+
+	near = rs_preload_get_near_count();
+	if (near < 1) return;
+	rs_preload_near_remove_all();
+
+	/* Get a list of selected icons */
+	selected = gtk_icon_view_get_selected_items(iconview);
+	if (g_list_length(selected) == 1)
+	{
+		/* Preload current image - this is stupid thou! */
+		path = g_list_nth_data(selected, 0);
+		if (gtk_tree_model_get_iter(gtk_icon_view_get_model (iconview), &iter, path))
+			preload_iter(model, &iter);
+
+		/* Near */
+		next = gtk_tree_path_copy(path);
+		prev = gtk_tree_path_copy(path);
+		for(n=0;n<near;n++)
+		{
+			/* Travel forward */
+			gtk_tree_path_next(next);
+			if (gtk_tree_model_get_iter(gtk_icon_view_get_model (iconview), &iter, next))
+				preload_iter(model, &iter);
+			/* Travel backward */
+			if (gtk_tree_path_prev(prev))
+				if (gtk_tree_model_get_iter(gtk_icon_view_get_model (iconview), &iter, prev))
+					preload_iter(model, &iter);
+		}
+		gtk_tree_path_free(next);
+		gtk_tree_path_free(prev);
+	}
+	else if (((g_list_length(selected) == 0)) && initial)
+	{
+		path = gtk_tree_path_new_first();
+		if (gtk_tree_model_get_iter(gtk_icon_view_get_model (iconview), &iter, path))
+			preload_iter(model, &iter);
+
+		/* Next */
+		for(n=0;n<near;n++)
+		{
+			gtk_tree_path_next(path);
+			if (gtk_tree_model_get_iter(gtk_icon_view_get_model (iconview), &iter, path))
+				preload_iter(model, &iter);
+		}
+		gtk_tree_path_free(path);
+	}
+
+	/* Free the list */
+	if (g_list_length(selected) > 0)
+	{
+		g_list_foreach (selected, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free (selected);
+	}
+}
+
 static void
 selection_changed(GtkIconView *iconview, gpointer data)
 {
@@ -225,33 +365,26 @@ selection_changed(GtkIconView *iconview, gpointer data)
 		g_signal_emit (G_OBJECT (data), signals[THUMB_ACTIVATED_SIGNAL], 0, g_list_nth_data(selected, 0));
 
 	g_list_free(selected);
+
+	predict_preload(data, FALSE);
 }
 
 static GtkWidget *
 make_iconview(GtkWidget *iconview, RSStore *store, gint prio)
 {
 	GtkWidget *scroller;
-	gboolean show_filenames;
-
-	/* This must be before gtk_icon_view_set_text_column() if we're not to crash later (GTK+ <2.10) */
-	gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (iconview), PIXBUF_COLUMN);
-
-	rs_conf_get_boolean_with_default(CONF_SHOW_FILENAMES, &show_filenames, TRUE);
-
-	if (show_filenames)
-		gtk_icon_view_set_text_column (GTK_ICON_VIEW (iconview), TEXT_COLUMN);
-	else
-		gtk_icon_view_set_text_column (GTK_ICON_VIEW (iconview), -1);
 
 	/* We must be abletoselect multiple icons */
 	gtk_icon_view_set_selection_mode(GTK_ICON_VIEW (iconview), GTK_SELECTION_MULTIPLE);
 
+#if GTK_CHECK_VERSION(2,12,0)
+	/* Enable tooltips */
+	gtk_icon_view_set_tooltip_column(GTK_ICON_VIEW (iconview), TEXT_COLUMN);
+#endif
+
 	/* pack them as close af possible */
 	gtk_icon_view_set_column_spacing(GTK_ICON_VIEW (iconview), 0);
 
-	/* 160 pixels should be enough */
-	gtk_widget_set_size_request (iconview, -1, 160);
-
 	/* Let us know if selection changes */
 	g_signal_connect((gpointer) iconview, "selection_changed",
 		G_CALLBACK (selection_changed), store);
@@ -267,39 +400,46 @@ make_iconview(GtkWidget *iconview, RSStore *store, gint prio)
 static gboolean
 model_filter_prio(GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
-	gint p;
+	gint p,t;
 	gint prio = GPOINTER_TO_INT (data);
 	gtk_tree_model_get (model, iter, PRIORITY_COLUMN, &p, -1);
-	switch(prio)
+	gtk_tree_model_get (model, iter, TYPE_COLUMN, &t, -1);
+
+	if (t == RS_STORE_TYPE_GROUP_MEMBER)
+		return FALSE;
+	else 
 	{
-		case PRIO_ALL:
-			switch (p)
-			{
-				case PRIO_D:
-					return(FALSE);
-					break;
-				default:
-					return(TRUE);
-					break;
-			}
-		case PRIO_U:
-			switch (p)
-			{
-				case PRIO_1:
-				case PRIO_2:
-				case PRIO_3:
-				case PRIO_D:
-					return(FALSE);
-					break;
-				default:
-					return(TRUE);
-					break;
-			}
-		default:
-			if (prio==p) return(TRUE);
-			break;
+		switch(prio)
+		{
+			case PRIO_ALL:
+				switch (p)
+				{
+					case PRIO_D:
+						return(FALSE);
+						break;
+					default:
+						return(TRUE);
+						break;
+				}
+			case PRIO_U:
+				switch (p)
+				{
+					case PRIO_1:
+					case PRIO_2:
+					case PRIO_3:
+					case PRIO_D:
+						return(FALSE);
+						break;
+					default:
+						return(TRUE);
+						break;
+				}
+			default:
+				if (prio==p) return(TRUE);
+				break;
+		}
+		return(FALSE);
 	}
-	return(FALSE);
 }
 
 static gint
@@ -329,7 +469,7 @@ count_priorities(GtkTreeModel *treemodel, GtkTreePath *do_not_use1, GtkTreeIter
 	GtkWidget **label = (GtkWidget **) data;
 	GtkTreeIter iter;
 	GtkTreePath *path;
-	gint priority;
+	gint priority, type;
 	gint count_1 = 0;
 	gint count_2 = 0;
 	gint count_3 = 0;
@@ -344,23 +484,27 @@ count_priorities(GtkTreeModel *treemodel, GtkTreePath *do_not_use1, GtkTreeIter
 	{
 		do {
 			gtk_tree_model_get(treemodel, &iter, PRIORITY_COLUMN, &priority, -1);
-			switch (priority)
+			gtk_tree_model_get(treemodel, &iter, TYPE_COLUMN, &type, -1);
+			if (type != RS_STORE_TYPE_GROUP_MEMBER)
 			{
-				case PRIO_1:
-					count_1++;
-					break;
-				case PRIO_2:
-					count_2++;
-					break;
-				case PRIO_3:
-					count_3++;
-					break;
-				case PRIO_U:
-					count_u++;
-					break;
-				case PRIO_D:
-					count_d++;
-					break;
+				switch (priority)
+				{
+					case PRIO_1:
+						count_1++;
+						break;
+					case PRIO_2:
+						count_2++;
+						break;
+					case PRIO_3:
+						count_3++;
+						break;
+					case PRIO_U:
+						count_u++;
+						break;
+					case PRIO_D:
+						count_d++;
+						break;
+				}
 			}
 		} while(gtk_tree_model_iter_next (treemodel, &iter));
 	}	
@@ -596,7 +740,6 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 	GtkTreeIter iter;
 	GdkPixbuf *pixbuf;
 	GdkPixbuf *pixbuf_clean;
-	GError *error;
 	GDir *dir;
 	GtkTreeSortable *sortable;
 	gint priority;
@@ -627,7 +770,7 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 	missing_thumb = gtk_widget_render_icon(GTK_WIDGET(store),
 		GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG, NULL);
 
-	dir = g_dir_open(path, 0, &error);
+	dir = g_dir_open(path, 0, NULL);
 	if (dir == NULL)
 		return -1;
 
@@ -743,9 +886,17 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 		gtk_icon_view_set_model (GTK_ICON_VIEW (store->iconview[n]), tree);
 	}
 
+#ifdef EXPERIMENTAL
+	/* load group file and group photos */
+	store_load_groups(store->store);
+#endif
+
 	/* Free the progress bar */
 	gui_progress_free(rsp);
 
+	/* Start the preloader */
+	predict_preload(store, TRUE);
+
 	/* Return the number of files successfully recognized */
 	return items;
 }
@@ -891,6 +1042,8 @@ rs_store_get_selected_names(RSStore *store)
 	gtk_icon_view_selected_foreach(GTK_ICON_VIEW(store->current_iconview),
 		icon_get_selected_names, &selected);
 
+	selected = rs_store_sort_selected(selected);
+
 	return selected;
 }
 
@@ -938,9 +1091,15 @@ rs_store_set_show_filenames(RSStore *store, gboolean show_filenames)
 	for (i=0;i<NUM_VIEWS;i++)
 	{
 		if (show_filenames)
+		{
 			gtk_icon_view_set_text_column (GTK_ICON_VIEW (store->iconview[i]), TEXT_COLUMN);
+			gtk_widget_set_size_request (store->iconview[i], -1, 160);
+		}
 		else
+		{
 			gtk_icon_view_set_text_column (GTK_ICON_VIEW (store->iconview[i]), -1);
+			gtk_widget_set_size_request (store->iconview[i], -1, 130);
+		}
 	}
 
 	return;
@@ -1004,16 +1163,17 @@ rs_store_get_name(RSStore *store, GtkTreeIter *iter)
 /**
  * Selects the previous or next thumbnail
  * @param store A RSStore
+ * @param current_filename Current filename or NULL if none
  * @param direction 1: previous, 2: next
  */
 gboolean
-rs_store_select_prevnext(RSStore *store, guint direction)
+rs_store_select_prevnext(RSStore *store, const gchar *current_filename, guint direction)
 {
 	gboolean ret = FALSE;
 	GList *selected;
 	GtkIconView *iconview;
 	GtkTreeIter iter;
-	GtkTreePath *path, *newpath = NULL;
+	GtkTreePath *path = NULL, *newpath = NULL;
 
 	g_assert(RS_IS_STORE(store));
 
@@ -1046,10 +1206,44 @@ rs_store_select_prevnext(RSStore *store, guint direction)
 	}
 	else if (g_list_length(selected) == 0)
 	{
-		/* If nothing is selected, select first thumbnail */
-		newpath = gtk_tree_path_new_first();
-		if (gtk_tree_model_get_iter(gtk_icon_view_get_model (iconview), &iter, newpath))
-			ret = TRUE;
+		/* Get current GtkTreeModelFilter */
+		GtkTreeModel *model = gtk_icon_view_get_model (GTK_ICON_VIEW(store->current_iconview));
+
+		/* If we got a filename, try to select prev/next from that */
+		if (current_filename)
+		{
+			if (tree_find_filename(GTK_TREE_MODEL(store->store), current_filename, NULL, &newpath))
+			{
+				while (!(path = gtk_tree_model_filter_convert_child_path_to_path(GTK_TREE_MODEL_FILTER(model), newpath)))
+				{
+					ret = FALSE;
+					if (direction == 1) /* Previous */
+					{
+						if (!gtk_tree_path_prev(newpath))
+							break;
+					}
+					else
+					{
+						gtk_tree_path_next(newpath);
+						if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(store->store), &iter, newpath))
+							break;
+					}
+					ret = TRUE;
+				}
+				if (newpath)
+					gtk_tree_path_free(newpath);
+				newpath = path;
+			}
+		}
+
+		/* If we got no hit, fall back to this */
+		if (ret == FALSE)
+		{
+			/* If nothing is selected, select first thumbnail */
+			newpath = gtk_tree_path_new_first();
+			if (gtk_tree_model_get_iter(gtk_icon_view_get_model (iconview), &iter, newpath))
+				ret = TRUE;
+		}
 	}
 
 	if (newpath && ret)
@@ -1069,3 +1263,396 @@ rs_store_select_prevnext(RSStore *store, guint direction)
 
 	return ret;
 }
+
+GdkPixbuf *
+store_group_update_pixbufs(GdkPixbuf *pixbuf, GdkPixbuf *pixbuf_clean)
+{
+	gint width, height, new_width, new_height;
+	guint rowstride;
+	guchar *pixels;
+	gint channels;
+	GdkPixbuf *new_pixbuf, *pixbuf_scaled;
+
+	width = gdk_pixbuf_get_width(pixbuf_clean);
+	height = gdk_pixbuf_get_height(pixbuf_clean);
+	
+	new_pixbuf = gdk_pixbuf_new(gdk_pixbuf_get_colorspace(pixbuf_clean),
+								TRUE,
+								gdk_pixbuf_get_bits_per_sample(pixbuf_clean),
+								width,
+								height);
+
+	width -= 6;
+	height -= 6;
+	
+	rowstride = gdk_pixbuf_get_rowstride (new_pixbuf);
+	pixels = gdk_pixbuf_get_pixels (new_pixbuf);
+	new_width = gdk_pixbuf_get_width (new_pixbuf);
+	new_height = gdk_pixbuf_get_height (new_pixbuf);
+	channels = gdk_pixbuf_get_n_channels (new_pixbuf);
+
+	// draw horizontal lines
+	rs_pixbuf_draw_hline(new_pixbuf, 0, 0, width+2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_hline(new_pixbuf, width+2, 2, 2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_hline(new_pixbuf, width+4, 4, 2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_hline(new_pixbuf, 0, height+1, width+2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_hline(new_pixbuf, 2, height+3, width+2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_hline(new_pixbuf, 4, height+5, width+2, 0x00, 0x00, 0x00, 0xFF);
+	// draw vertical lines
+	rs_pixbuf_draw_vline(new_pixbuf, 0, 0, height+2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_vline(new_pixbuf, 2, height+2, 2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_vline(new_pixbuf, 4, height+4, 2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_vline(new_pixbuf, width+1, 0, height+2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_vline(new_pixbuf, width+3, 2, height+2, 0x00, 0x00, 0x00, 0xFF);
+	rs_pixbuf_draw_vline(new_pixbuf, width+5, 4, height+2, 0x00, 0x00, 0x00, 0xFF);
+	// fill spots with white
+	rs_pixbuf_draw_hline(new_pixbuf, 3, height+2, width, 0xFF, 0xFF, 0xFF, 0xFF);
+	rs_pixbuf_draw_hline(new_pixbuf, 5, height+4, width, 0xFF, 0xFF, 0xFF, 0xFF);
+	rs_pixbuf_draw_vline(new_pixbuf, width+2, 3, height, 0xFF, 0xFF, 0xFF, 0xFF);
+	rs_pixbuf_draw_vline(new_pixbuf, width+4, 5, height, 0xFF, 0xFF, 0xFF, 0xFF);
+
+	pixbuf_scaled = gdk_pixbuf_scale_simple(pixbuf_clean,
+							width,
+							height,
+							GDK_INTERP_BILINEAR);
+	gdk_pixbuf_copy_area(pixbuf_scaled,
+						 0, 0, width, height,
+						 new_pixbuf, 1, 1);
+	return new_pixbuf;
+}
+
+void
+store_group_select_n(GtkListStore *store, GtkTreeIter iter, guint n)
+{
+	GList *members;
+	GtkTreeIter *child_iter;
+	GdkPixbuf *pixbuf = NULL;
+	GdkPixbuf *pixbuf_clean = NULL;
+	gchar *fullname = NULL;
+	gchar *name = NULL;
+	guint priority;
+	gboolean exported;
+
+	gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
+					   GROUP_LIST_COLUMN, &members,
+					   -1);
+	child_iter = (GtkTreeIter *) g_list_nth_data(members, n);
+
+	gtk_tree_model_get(GTK_TREE_MODEL(store), child_iter,
+					   PIXBUF_COLUMN, &pixbuf,
+					   PIXBUF_CLEAN_COLUMN, &pixbuf_clean,
+					   TEXT_COLUMN, &name,
+					   FULLNAME_COLUMN, &fullname,
+					   PRIORITY_COLUMN, &priority,
+					   EXPORTED_COLUMN, &exported,
+					   -1);
+
+	pixbuf_clean = store_group_update_pixbufs(pixbuf, pixbuf_clean);
+	pixbuf = gdk_pixbuf_copy(pixbuf_clean);
+	
+	thumbnail_update(pixbuf, pixbuf_clean, priority, exported);
+
+	gtk_list_store_set (store, &iter,
+					PIXBUF_COLUMN, pixbuf,
+					PIXBUF_CLEAN_COLUMN, pixbuf_clean,
+					TEXT_COLUMN, name,
+					FULLNAME_COLUMN, fullname,
+					PRIORITY_COLUMN, priority,
+					EXPORTED_COLUMN, exported,
+					-1);
+
+	store_save_groups(store);
+	return;
+}
+
+/**
+ * Marks a selection of thumbnails as a group
+ * @param store A RSStore
+ */
+void
+rs_store_group_photos(RSStore *store)
+{
+	GList *selected = NULL;
+	gint selected_groups;
+
+	selected = rs_store_get_selected_iters(store);
+	selected_groups = store_selection_n_groups(store, selected);
+	gtk_icon_view_unselect_all(GTK_ICON_VIEW(store->current_iconview)); // Or automatic load of photo == wait time
+
+	if (selected_groups == 0)
+	{
+		store_group_photos_by_iters(store->store, selected);
+	}
+	else
+	{
+		GtkTreeIter *group = NULL, *s = NULL;
+		GList *members = NULL, *newmembers = NULL;
+
+ 		while (selected)
+		{
+			s = selected->data;
+			if (store_iter_is_group(store->store, s))
+			{
+				gtk_tree_model_get(GTK_TREE_MODEL(store->store), s,
+								   GROUP_LIST_COLUMN, &members,
+								   -1);
+
+				newmembers = g_list_concat(newmembers, members);
+
+				if (group)
+				{
+					gtk_list_store_remove(store->store, s);
+					selected = g_list_remove(selected, s);
+					selected = g_list_previous(selected);
+				}
+				else
+					group = s;
+			}
+			else
+			{
+				newmembers = g_list_append(newmembers, s);
+				gtk_list_store_set(store->store, s,
+								   TYPE_COLUMN, RS_STORE_TYPE_GROUP_MEMBER,
+								   -1);
+			}
+			selected = g_list_next(selected);
+		}
+
+		gtk_list_store_set(store->store, group,
+						   GROUP_LIST_COLUMN, newmembers,
+						   -1);
+	}
+	store_save_groups(store->store);
+}
+
+/**
+ * Ungroup a group or selection of groups
+ * @param store A RSStore
+ */
+void
+rs_store_ungroup_photos(RSStore *store)
+{
+	GList *members;
+	GtkTreeIter *child_iter;
+	GList *selected;
+	gint n,m;
+	GtkTreeIter *iter;
+
+	selected = rs_store_get_selected_iters(store);
+
+	for( n = 0; n < g_list_length(selected); n++)
+	{
+		if (store_iter_is_group(store->store, g_list_nth_data(selected, n)))
+		{
+			iter = (GtkTreeIter *) g_list_nth_data(selected, n);
+			gtk_tree_model_get(GTK_TREE_MODEL(store->store), iter,
+							   GROUP_LIST_COLUMN, &members,
+							   -1);
+			for( m = 0; m < g_list_length(members); m++)
+			{
+				child_iter = (GtkTreeIter *) g_list_nth_data(members, m);
+				gtk_list_store_set(store->store, child_iter,
+								   	TYPE_COLUMN, RS_STORE_TYPE_FILE,
+								   -1);
+			}
+			gtk_list_store_remove(store->store, iter);
+		}
+	}
+	store_save_groups(store->store);
+}
+
+gboolean
+store_iter_is_group(GtkListStore *store, GtkTreeIter *iter)
+{
+	guint t;
+
+	gtk_tree_model_get (GTK_TREE_MODEL(store), iter, TYPE_COLUMN, &t, -1);
+	if (t == RS_STORE_TYPE_GROUP)
+		return TRUE;
+	else
+		return FALSE;
+}
+
+gint
+store_selection_n_groups(RSStore *store, GList *selected)
+{
+	gint n, group = 0;
+
+	for(n = 0; n < g_list_length(selected); n++)
+	{
+		if (store_iter_is_group(store->store, g_list_nth_data(selected, n)))
+			group++;
+	}
+	return group;
+}
+
+void
+store_save_groups(GtkListStore *store) {
+	gchar *dotdir = NULL, *filename = NULL;
+	GtkTreeIter iter;
+	xmlTextWriterPtr writer;
+
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, FULLNAME_COLUMN, &filename, -1);
+
+	dotdir = rs_dotdir_get(filename);
+	GString *gs = g_string_new(dotdir);
+	g_string_append(gs, G_DIR_SEPARATOR_S);
+	g_string_append(gs, GROUP_XML_FILE);
+	gchar *xmlfile = gs->str;
+	g_string_free(gs, FALSE);
+
+	writer = xmlNewTextWriterFilename(xmlfile, 0);
+	if (!writer)
+		return;
+	xmlTextWriterSetIndent(writer, 1);
+	xmlTextWriterStartDocument(writer, NULL, "UTF-8", NULL);
+	xmlTextWriterStartElement(writer, BAD_CAST "rawstudio-groups");
+
+	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
+		do
+		{
+			if (store_iter_is_group(store, &iter))
+			{
+				gchar *selected;
+				GList *members;
+
+				xmlTextWriterStartElement(writer, BAD_CAST "group");
+
+				// Find selected member and place this first in XML
+				gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
+					   FULLNAME_COLUMN, &selected,
+					   -1);
+				xmlTextWriterWriteFormatElement(writer, BAD_CAST "member", "%s", selected);
+
+				gtk_tree_model_get(GTK_TREE_MODEL(store), &iter,
+					   GROUP_LIST_COLUMN, &members,
+					   -1);
+				gint m;
+				for( m = 0; m < g_list_length(members); m++)
+				{
+					GtkTreeIter *child_iter = (GtkTreeIter *) g_list_nth_data(members, m);
+					gtk_tree_model_get (GTK_TREE_MODEL(store), child_iter, FULLNAME_COLUMN, &filename, -1);
+					if (!g_str_equal(selected, filename))
+						xmlTextWriterWriteFormatElement(writer, BAD_CAST "member", "%s", filename);
+				}
+				xmlTextWriterEndElement(writer);
+			}
+		} while(gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter));
+
+	xmlTextWriterEndDocument(writer);
+	xmlFreeTextWriter(writer);
+
+	return;
+}
+
+void
+store_load_groups(GtkListStore *store) {
+	gchar *dotdir = NULL, *filename = NULL;
+	xmlDocPtr doc;
+	xmlNodePtr cur;
+	xmlNodePtr group = NULL;
+	GtkTreeIter iter;
+	GList *members = NULL;
+
+	g_assert(store != NULL);
+
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+	gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, FULLNAME_COLUMN, &filename, -1);
+	dotdir = rs_dotdir_get(filename);
+
+	GString *gs = g_string_new(dotdir);
+	g_string_append(gs, G_DIR_SEPARATOR_S);
+	g_string_append(gs, GROUP_XML_FILE);
+	gchar *xmlfile = gs->str;
+	g_string_free(gs, FALSE);
+	
+	if (!g_file_test(xmlfile, G_FILE_TEST_IS_REGULAR))
+		return;
+
+	doc = xmlParseFile(xmlfile);
+	if (!doc)
+		return;
+	
+	cur = xmlDocGetRootElement(doc);
+	cur = cur->xmlChildrenNode;
+
+	while(cur)
+	{
+		if ((!xmlStrcmp(cur->name, BAD_CAST "group")))
+		{
+			xmlChar *member = NULL;
+
+			group = cur->xmlChildrenNode;
+			while (group)
+			{
+				if ((!xmlStrcmp(group->name, BAD_CAST "member")))
+				{
+					member = xmlNodeListGetString(doc, group->xmlChildrenNode, 1);
+					if (member)
+					{
+						filename = (char *) member;
+						members = g_list_append(members, filename);
+					}
+					member = NULL;
+				}	
+			group = group->next;
+			}
+		store_group_photos_by_filenames(store, members);
+		members = NULL;
+		}
+		cur = cur->next;
+	}
+
+	xmlFreeDoc(doc);
+	return;
+}
+
+void
+store_group_photos_by_iters(GtkListStore *store, GList *members)
+{
+	GtkTreeIter iter;
+	gint n;
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+						PIXBUF_COLUMN, NULL,
+						PIXBUF_CLEAN_COLUMN, NULL,
+						TEXT_COLUMN, "",
+						FULLNAME_COLUMN, NULL,
+						PRIORITY_COLUMN, 0,
+						EXPORTED_COLUMN, 0,
+						TYPE_COLUMN, RS_STORE_TYPE_GROUP,
+						GROUP_LIST_COLUMN, members,
+						-1);
+	store_group_select_n(store, iter, 0);
+	
+	for(n = 0; n < g_list_length(members); n++)
+	{
+		gtk_list_store_set(store, g_list_nth_data(members,n),
+						   TYPE_COLUMN, RS_STORE_TYPE_GROUP_MEMBER,
+						   -1);
+	}
+}
+
+void
+store_group_photos_by_filenames(GtkListStore *store, GList *members)
+{
+	GList *members_iter = NULL;
+	GtkTreeIter iter;
+	GtkTreeIter *iter_copy;
+	while (members)
+	{
+		tree_find_filename(GTK_TREE_MODEL(store), (gchar *) members->data, &iter, NULL);
+		iter_copy = gtk_tree_iter_copy(&iter);
+		members_iter = g_list_append(members_iter, iter_copy);
+		members = g_list_next(members);
+	}
+	store_group_photos_by_iters(store, members_iter);
+}
+
+GList
+* rs_store_sort_selected(GList *selected)
+{
+	return g_list_sort(selected, (GCompareFunc) g_utf8_collate);
+}
