@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -26,16 +26,17 @@
 #include <tiffio.h>
 #include "rawstudio.h"
 #include "rs-tiff.h"
+#include "rs-image.h"
 
-static void rs_tiff_generic_init(TIFF *output, guint w, guint h, const gchar *profile_filename, gboolean uncompressed);
+static void rs_tiff_generic_init(TIFF *output, guint w, guint h, const guint samples_per_pixel, const gchar *profile_filename, gboolean uncompressed);
 
 static void
-rs_tiff_generic_init(TIFF *output, guint w, guint h, const gchar *profile_filename, gboolean uncompressed)
+rs_tiff_generic_init(TIFF *output, guint w, guint h, const guint samples_per_pixel, const gchar *profile_filename, gboolean uncompressed)
 {
 	TIFFSetField(output, TIFFTAG_IMAGEWIDTH, w);
 	TIFFSetField(output, TIFFTAG_IMAGELENGTH, h);
 	TIFFSetField(output, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
-	TIFFSetField(output, TIFFTAG_SAMPLESPERPIXEL, 3);
+	TIFFSetField(output, TIFFTAG_SAMPLESPERPIXEL, samples_per_pixel);
 	TIFFSetField(output, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
 	TIFFSetField(output, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
 	if (uncompressed)
@@ -72,11 +73,11 @@ rs_tiff8_save(RS_IMAGE8 *image, const gchar *filename, const gchar *profile_file
 
 	if((output = TIFFOpen(filename, "w")) == NULL)
 		return(FALSE);
-	rs_tiff_generic_init(output, image->w, image->h, profile_filename, uncompressed);
+	rs_tiff_generic_init(output, image->w, image->h, 3, profile_filename, uncompressed);
 	TIFFSetField(output, TIFFTAG_BITSPERSAMPLE, 8);
 	for(row=0;row<image->h;row++)
 	{
-		guchar *buf = image->pixels + image->rowstride * row;
+		guchar *buf = GET_PIXEL(image, 0, row);
 		TIFFWriteScanline(output, buf, row, 0);
 	}
 	TIFFClose(output);
@@ -86,18 +87,57 @@ rs_tiff8_save(RS_IMAGE8 *image, const gchar *filename, const gchar *profile_file
 gboolean
 rs_tiff16_save(RS_IMAGE16 *image, const gchar *filename, const gchar *profile_filename, gboolean uncompressed)
 {
+	static GStaticMutex filename_lock = G_STATIC_MUTEX_INIT;
 	TIFF *output;
 	gint row;
 
-	if((output = TIFFOpen(filename, "w")) == NULL)
-		return(FALSE);
-	rs_tiff_generic_init(output, image->w, image->h, profile_filename, uncompressed);
-	TIFFSetField(output, TIFFTAG_BITSPERSAMPLE, 16);
-	for(row=0;row<image->h;row++)
+	g_static_mutex_lock(&filename_lock);
+	if ((!g_file_test(filename, G_FILE_TEST_EXISTS)) && (output = TIFFOpen(filename, "w")))
 	{
-		gushort *buf = image->pixels + image->rowstride * row;
-		TIFFWriteScanline(output, buf, row, 0);
+		g_static_mutex_unlock(&filename_lock);
+		rs_tiff_generic_init(output, image->w, image->h, image->channels, profile_filename, uncompressed);
+		TIFFSetField(output, TIFFTAG_BITSPERSAMPLE, 16);
+		for(row=0;row<image->h;row++)
+		{
+			gushort *buf = GET_PIXEL(image, 0, row);
+			TIFFWriteScanline(output, buf, row, 0);
+		}
+		TIFFClose(output);
 	}
-	TIFFClose(output);
-	return(TRUE);
+	else
+	{
+		g_static_mutex_unlock(&filename_lock);
+		return FALSE;
+	}
+	return TRUE ;
+}
+
+RS_IMAGE16 *
+rs_tiff16_load(const gchar *filename)
+{
+	RS_IMAGE16 *image = NULL;
+
+	g_assert(filename != NULL);
+
+	TIFF* tif = TIFFOpen(filename, "r");
+
+	if (tif)
+	{
+		guint w=0, h=0, samples_per_pixel=0, row, bits_per_sample=0;
+		TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &w);
+		TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
+		TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &samples_per_pixel);
+		TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &bits_per_sample);
+
+		if ((bits_per_sample == 16) && w && h)
+			image = rs_image16_new(w, h, samples_per_pixel, 4);
+
+		if (image)
+			/* Write directly to pixel data */
+			for(row=0;row<h;row++)
+				TIFFReadScanline(tif, GET_PIXEL(image, 0, row), row, 0);
+		TIFFClose(tif);
+	}
+
+	return image;
 }
