@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -26,6 +26,11 @@
 #include "toolbox.h"
 #include "conf_interface.h"
 #include "gettext.h"
+#include "color.h"
+#include "rs-spline.h"
+#include "rs-curve.h"
+#include "rs-preview-widget.h"
+#include "rs-histogram.h"
 
 /* used for gui_adj_reset_callback() */
 struct reset_carrier {
@@ -34,10 +39,8 @@ struct reset_carrier {
 };
 
 GtkLabel *infolabel;
-static GtkWidget *scale;
 static GtkWidget *toolbox;
 
-static GtkWidget *gui_hist(RS_BLOB *rs, const gchar *label, gboolean show);
 static GtkWidget *gui_box(const gchar *title, GtkWidget *in, gboolean expanded);
 static void gui_transform_rot90_clicked(GtkWidget *w, RS_BLOB *rs);
 static void gui_transform_rot180_clicked(GtkWidget *w, RS_BLOB *rs);
@@ -46,7 +49,6 @@ static void gui_transform_mirror_clicked(GtkWidget *w, RS_BLOB *rs);
 static void gui_transform_flip_clicked(GtkWidget *w, RS_BLOB *rs);
 static GtkWidget *gui_transform(RS_BLOB *rs, gboolean show);
 static GtkWidget *gui_tool_warmth(RS_BLOB *rs, gint n, gboolean show);
-static GtkWidget *gui_slider(GtkObject *adj, const gchar *label, gboolean expanded);
 static gboolean gui_adj_reset_callback(GtkWidget *widget, GdkEventButton *event, struct reset_carrier *rc);
 static GtkWidget *gui_make_scale_from_adj(RS_BLOB *rs, GCallback cb, GtkObject *adj, gint mask);
 static void curve_context_callback_save(GtkMenuItem *menuitem, gpointer user_data);
@@ -54,33 +56,6 @@ static void curve_context_callback_open(GtkMenuItem *menuitem, gpointer user_dat
 static void curve_context_callback_reset(GtkMenuItem *menuitem, gpointer user_data);
 static void curve_context_callback(GtkWidget *widget, gpointer user_data);
 static void gui_notebook_callback(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, RS_BLOB *rs);
-static void scale_expand_callback(GObject *object, GParamSpec *param_spec, gpointer user_data);
-
-static GtkWidget *
-gui_hist(RS_BLOB *rs, const gchar *label, gboolean show)
-{
-	GdkPixbuf *pixbuf;
-	gint height;
-	guint rowstride;
-	guchar *pixels;
-
-	if (!rs_conf_get_integer(CONF_HISTHEIGHT, &height))
-		height = 128;
-
-	/* creates the pixbuf containing the histogram */
-	pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, 256, height);
-	
-	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-	pixels = gdk_pixbuf_get_pixels (pixbuf);
-
-	/* sets all the pixels black */
-	memset(pixels, 0x00, rowstride*height);
-
-	/* creates an image from the histogram pixbuf */
-	rs->histogram_image = (GtkImage *) gtk_image_new_from_pixbuf(pixbuf);
-
-	return(gui_box(label, (GtkWidget *)rs->histogram_image, show));
-}
 
 static GtkWidget *
 gui_box(const gchar *title, GtkWidget *in, gboolean expanded)
@@ -104,7 +79,7 @@ gui_transform_rot90_clicked(GtkWidget *w, RS_BLOB *rs)
 {
 	if (!rs->photo) return;
 	rs_photo_rotate(rs->photo, 1, 0.0);
-	update_preview(rs, FALSE, TRUE);
+	rs_update_preview(rs);
 }
 
 static void
@@ -112,7 +87,7 @@ gui_transform_rot180_clicked(GtkWidget *w, RS_BLOB *rs)
 {
 	if (!rs->photo) return;
 	rs_photo_rotate(rs->photo, 2, 0.0);
-	update_preview(rs, FALSE, TRUE);
+	rs_update_preview(rs);
 }
 
 static void
@@ -120,7 +95,7 @@ gui_transform_rot270_clicked(GtkWidget *w, RS_BLOB *rs)
 {
 	if (!rs->photo) return;
 	rs_photo_rotate(rs->photo, 3, 0.0);
-	update_preview(rs, FALSE, TRUE);
+	rs_update_preview(rs);
 }
 
 static void
@@ -128,7 +103,7 @@ gui_transform_mirror_clicked(GtkWidget *w, RS_BLOB *rs)
 {
 	if (!rs->photo) return;
 	rs_photo_mirror(rs->photo);
-	update_preview(rs, FALSE, TRUE);
+	rs_update_preview(rs);
 }
 
 static void
@@ -136,7 +111,7 @@ gui_transform_flip_clicked(GtkWidget *w, RS_BLOB *rs)
 {
 	if (!rs->photo) return;
 	rs_photo_flip(rs->photo);
-	update_preview(rs, FALSE, TRUE);
+	rs_update_preview(rs);
 }
 
 static GtkWidget *
@@ -207,16 +182,6 @@ gui_tool_warmth(RS_BLOB *rs, gint n, gboolean show)
 	return(gui_box(_("Warmth/tint"), box, show));
 }
 
-static GtkWidget *
-gui_slider(GtkObject *adj, const gchar *label, gboolean expanded)
-{
-	GtkWidget *hscale;
-	hscale = gtk_hscale_new (GTK_ADJUSTMENT (adj));
-	gtk_scale_set_value_pos( GTK_SCALE(hscale), GTK_POS_LEFT);
-	gtk_scale_set_digits(GTK_SCALE(hscale), 2);
-	return(gui_box(label, hscale, expanded));
-}
-
 static gboolean
 gui_adj_reset_callback(GtkWidget *widget, GdkEventButton *event, struct reset_carrier *rc)
 {
@@ -349,11 +314,19 @@ curve_context_callback_reset(GtkMenuItem *menuitem, gpointer user_data)
 	rs_curve_widget_add_knot(curve, 1.0,1.0);
 }
 
+static void
+curve_context_callback_white_black_point(GtkMenuItem *menuitem, gpointer user_data)
+{
+	RS_BLOB *rs = user_data;
+	rs_white_black_point(rs);
+}
+
 static void
 curve_context_callback(GtkWidget *widget, gpointer user_data)
 {
 	GtkWidget *i, *menu = gtk_menu_new();
 	gint n=0;
+	RS_BLOB *rs = user_data;
 
 	i = gtk_menu_item_new_with_label (_("Open curve ..."));
 	gtk_widget_show (i);
@@ -367,6 +340,10 @@ curve_context_callback(GtkWidget *widget, gpointer user_data)
 	gtk_widget_show (i);
 	gtk_menu_attach (GTK_MENU (menu), i, 0, 1, n, n+1); n++;
 	g_signal_connect (i, "activate", G_CALLBACK (curve_context_callback_reset), widget);
+	i = gtk_menu_item_new_with_label (_("Auto adjust curve ends"));
+	gtk_widget_show (i);
+	gtk_menu_attach (GTK_MENU (menu), i, 0, 1, n, n+1); n++;
+	g_signal_connect (i, "activate", G_CALLBACK (curve_context_callback_white_black_point), rs);
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, GDK_CURRENT_TIME);
 }
 
@@ -374,32 +351,11 @@ static void
 gui_notebook_callback(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, RS_BLOB *rs)
 {
 	rs->current_setting = page_num;
-	if (rs->photo) {
-		rs->photo->current_setting = rs->current_setting;
-		update_previewtable_callback(NULL, rs);
-	}
-}
-
-static void
-scale_expand_callback(GObject *object, GParamSpec *param_spec, gpointer user_data)
-{
-	RS_BLOB *rs = (RS_BLOB *) user_data;
-
-	if(gtk_expander_get_expanded (GTK_EXPANDER (object)))
+	if (rs->photo)
 	{
-		rs->zoom_to_fit = FALSE;
-		update_preview(rs, FALSE, TRUE);
+		rs_update_preview(rs);
+		rs_preview_widget_set_snapshot(RS_PREVIEW_WIDGET(rs->preview), 0, page_num);
 	}
-	else
-		rs_zoom_to_fit(rs);
-	return;
-}
-
-void
-scale_expand_set(gboolean expanded)
-{
-	gtk_expander_set_expanded (GTK_EXPANDER (scale), expanded);
-	return;
 }
 
 void
@@ -459,6 +415,7 @@ make_toolbox(RS_BLOB *rs)
 	GtkWidget *toolboxviewport;
 	gint n;
 	gboolean show;
+	gint height;
 
 	toolbox_label[0] = gtk_label_new(_(" A "));
 	toolbox_label[1] = gtk_label_new(_(" B "));
@@ -492,7 +449,7 @@ make_toolbox(RS_BLOB *rs)
 
 		rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_CONTRAST, &show, DEFAULT_CONF_SHOW_TOOLBOX_CONTRAST);
 		toolbox_contrast[n] = gui_box(_("Contrast"), gui_make_scale_from_adj(rs,
-			G_CALLBACK(update_previewtable_callback), rs->settings[n]->contrast, MASK_CONTRAST), show);
+			G_CALLBACK(update_preview_callback), rs->settings[n]->contrast, MASK_CONTRAST), show);
 		gtk_box_pack_start (GTK_BOX (tbox[n]), toolbox_contrast[n], FALSE, FALSE, 0);
 		g_signal_connect_after(toolbox_contrast[n], "activate", G_CALLBACK(gui_expander_toggle_callback), toolbox_contrast);
 		g_signal_connect_after(toolbox_contrast[n], "activate", G_CALLBACK(gui_expander_save_status_callback), CONF_SHOW_TOOLBOX_CONTRAST);
@@ -505,8 +462,8 @@ make_toolbox(RS_BLOB *rs)
 
 		rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_CURVE, &show, DEFAULT_CONF_SHOW_TOOLBOX_CURVE);
 		gtk_widget_set_size_request(rs->settings[n]->curve, 64, 64);
-		g_signal_connect(rs->settings[n]->curve, "changed", G_CALLBACK(update_previewtable_callback), rs);
-		g_signal_connect(rs->settings[n]->curve, "right-click", G_CALLBACK(curve_context_callback), NULL);
+		g_signal_connect(rs->settings[n]->curve, "changed", G_CALLBACK(update_preview_callback), rs);
+		g_signal_connect(rs->settings[n]->curve, "right-click", G_CALLBACK(curve_context_callback), rs);
 		toolbox_curve[n] = gui_box(_("Curve"), rs->settings[n]->curve, show);
 		gtk_box_pack_start (GTK_BOX (tbox[n]), toolbox_curve[n], TRUE, FALSE, 0);
 		g_signal_connect_after(toolbox_curve[n], "activate", G_CALLBACK(gui_expander_toggle_callback), toolbox_curve);
@@ -516,18 +473,19 @@ make_toolbox(RS_BLOB *rs)
 	}
 	g_signal_connect(notebook, "switch-page", G_CALLBACK(gui_notebook_callback), rs);
 
-	scale = gui_slider(rs->scale, _("Scale"), FALSE);
-	g_signal_connect(scale, "notify::expanded", G_CALLBACK (scale_expand_callback), rs);
-
 	toolbox = gtk_vbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (toolbox), notebook, FALSE, FALSE, 0);
 	rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_TRANSFORM, &show, DEFAULT_CONF_SHOW_TOOLBOX_TRANSFORM);
 	toolbox_transform = gui_transform(rs, show);
 	gtk_box_pack_start (GTK_BOX (toolbox), toolbox_transform, FALSE, FALSE, 0);
 	g_signal_connect_after(toolbox_transform, "activate", G_CALLBACK(gui_expander_save_status_callback), CONF_SHOW_TOOLBOX_TRANSFORM);
-	gtk_box_pack_start (GTK_BOX (toolbox), scale, FALSE, FALSE, 0); // FIXME: Needs to save state
+
+	if (!rs_conf_get_integer(CONF_HISTHEIGHT, &height))
+		height = 70;
+	rs->histogram = rs_histogram_new();
+	gtk_widget_set_size_request(rs->histogram, 64, height);
 	rs_conf_get_boolean_with_default(CONF_SHOW_TOOLBOX_HIST, &show, DEFAULT_CONF_SHOW_TOOLBOX_HIST);
-	toolbox_hist = gui_hist(rs, _("Histogram"), show);
+	toolbox_hist = gui_box(_("Histogram"), rs->histogram, show);
 	gtk_box_pack_start (GTK_BOX (toolbox), toolbox_hist, FALSE, FALSE, 0);
 	g_signal_connect_after(toolbox_hist, "activate", G_CALLBACK(gui_expander_save_status_callback), CONF_SHOW_TOOLBOX_HIST);
 
